/*
 * Created on Mar 31, 2004, 9:13:59 AM
 * author: VEZIER Antoine
 * Internship Master : Hamon Catherine
 *  
 */
 
package org.objectweb.eclipsespeedo;

import java.io.IOException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.eclipsespeedo.drivers.xml.DriverDefinitionManager;
import org.objectweb.eclipsespeedo.jdoextensions.xml.JDOExtensionsManager;
import org.objectweb.eclipsespeedo.jdoextensions.xml.VendorExtensions;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
;
/**
 * The main plugin class to be used in the desktop.
 */
public class EclipseSpeedoPlugin extends AbstractUIPlugin {
	private JDOExtensionsManager jdoExtensionManager;
	private DriverDefinitionManager driverDefinitionManager;
	private URL installedURL;
	//The shared instance.
	private static EclipseSpeedoPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	public static LoggerFactory loggerFactory;
	private Logger logger;  
	
	/**
	 * The constructor.
	 */
	public EclipseSpeedoPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		installedURL = descriptor.getInstallURL();
		try {
			loggerFactory = Monolog.initialize();
			logger = loggerFactory.getLogger(EclipseSpeedoPlugin.class.getName());
			logger.log(BasicLevel.DEBUG, "Entering Plugin constructor");
			String s = Platform.resolve(installedURL).getFile();
			logger.log(BasicLevel.DEBUG, "installed URL "+s);
			this.driverDefinitionManager = new DriverDefinitionManager(new Path(s));
			this.jdoExtensionManager = new JDOExtensionsManager(new Path(s));
			for (Iterator ite = this.jdoExtensionManager.getVendorExtensions().iterator();ite.hasNext();){
				VendorExtensions ve = (VendorExtensions) ite.next();
				System.out.println(ve.getDriver());
			}
			EclipseSpeedoPlugin.getDefault().getPreferenceStore().setDefault("Vendor name prefixe","speedo");
			resourceBundle= ResourceBundle.getBundle("org.objectweb.eclipsespeedo.Bundle");
		} catch (MissingResourceException x) {
			resourceBundle = null;
			logger.log(BasicLevel.ERROR, x);
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR, e);
		}
	}
	
	/**
	 * Gets the installedURL.
	 * @return Returns a URL
	 */
	public URL getInstalledURL() {
		return installedURL;
	}

	/**
	 * Returns the shared instance.
	 */
	public static EclipseSpeedoPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= EclipseSpeedoPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}


	public static String getFormattedString(String key, Object arg)
	{
		return MessageFormat.format(
			getResourceString(key),
			new Object[] { arg });
	}

	public static String getFormattedString(
		String key,
		Object arg0,
		Object arg1)
	{
		return MessageFormat.format(
			getResourceString(key),
			new Object[] { arg0, arg1 });
	}

	public static String getFormattedString(
		String key,
		Object arg0,
		Object arg1,
		Object arg2)
	{
		return MessageFormat.format(
			getResourceString(key),
			new Object[] { arg0, arg1, arg2 });
	}

	public static String getFormattedString(String key, boolean b)
	{
		return MessageFormat.format(
			getResourceString(key),
			new Object[] { String.valueOf(b)});
	}
	public static String getFormattedString(String key, int i)
	{
		return MessageFormat.format(
			getResourceString(key),
			new Object[] { new Integer(i)});
	}
	/**
	 * Gets a string from the resource bundle and formats it with arguments
	 */
	public static String getFormattedString(String key, Object[] args)
	{
		return MessageFormat.format(getResourceString(key), args);
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	
	static public Shell getShell() {
		IWorkbench workbench = PlatformUI.getWorkbench();
		return workbench.getActiveWorkbenchWindow().getShell();
	}
	
	/**
	 * @return  DriverDefinitionManager
	 */
	public DriverDefinitionManager getDriverDefinitionManager() {
		return driverDefinitionManager;
	}
	
	/**
	 * Sets the  DriverDefinitionManager.
	 * @param  DriverDefinitionManager The  DriverDefinitionManager to set
	 */
	public void setDriverDefinitionManager(DriverDefinitionManager driverDefinitionManager) {
		this.driverDefinitionManager = driverDefinitionManager;
	}
	
	/**
	 * @return  DriverDefinitionManager
	 */
	public JDOExtensionsManager getJDOExtensionsManager() {
		return jdoExtensionManager;
	}
}
