/*
 * Created on Apr 13, 2004
 * author: VEZIER Antoine
 * Internship Master: Hamon Catherine
 * 
 */
 
package org.objectweb.eclipsespeedo.actions;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.ImportDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.objectweb.eclipsespeedo.utils.ASTFixFlattener;
import org.objectweb.eclipsespeedo.utils.JavaCodeHelper;
import org.objectweb.eclipsespeedo.wizards.MainCreationWizardPage;



public class MainCreation {
	private MainCreationWizardPage page;
	private IProject project;
	private IJavaProject jp;
	public MainCreation(MainCreationWizardPage page, IProject project) {
		this.page = page;
		this.project = project;
		jp = JavaCore.create(project);
	}

	/**
	 * Make modifications on a java class file ( a compilationUnit)
	 * @return if the change is effective
	 */
	public boolean doChange() {
		try	{
			ICompilationUnit unit = page.getClasse().getCompilationUnit();
			int tdecl = page.getASTClass();
			String nouvcontents = getNewContent(unit, tdecl);
			if (nouvcontents.equals(""))
				return false;
			unit.getBuffer().setContents(nouvcontents);
			unit.save(new NullProgressMonitor(), true);
		}
		catch (Exception e)	{
			e.printStackTrace();
		}
		return true;
	}

	


	/**
	 * Return a string that represent the modified unit from the user parameters
	 * @param unit
	 * @return
	 */
	public String getNewContent(ICompilationUnit unit, int classe) {
		try	{
			// Then we get back the contents
			// we try it another way
			CompilationUnit cu = AST.parseCompilationUnit(unit, true);
			AST ast = cu.getAST();
			ImportDeclaration newImport;

			// add the jdo import
			if (page.getimportJDOButton() && (!page.hasJDOOImport())) {
				newImport = ast.newImportDeclaration();
				newImport.setName(ast.newQualifiedName(ast.newName(new String[] { "javax" }), ast.newSimpleName("jdo")));
				newImport.setOnDemand(true);
				cu.imports().add(newImport);
			}
			MethodDeclaration mainMethod ;
			if (page.hasMainMethod()) {
				mainMethod = JavaCodeHelper.getMethod(((TypeDeclaration) cu.types().get(classe)).getMethods(),"main");
			}
			else {
				mainMethod = JavaCodeHelper.createMainMethod((TypeDeclaration) cu.types().get(classe));
			}

			if (JavaCodeHelper.adjustPropertiesAndManagers(mainMethod, this.page)) {	
				//add the java util import for the Properties
				if (!page.hasJavaUtilImport()) {
					newImport = ast.newImportDeclaration();
					newImport.setName(ast.newQualifiedName(ast.newName(new String[] { "java" }),ast.newSimpleName("util")));
					newImport.setOnDemand(true);
					cu.imports().add(newImport);
				}
				//add the java io import for the Properties
				if (!page.hasJavaIOImport()) {
					newImport = ast.newImportDeclaration();
					newImport.setName(ast.newQualifiedName(ast.newName(new String[] { "java" }),ast.newSimpleName("io")));
					newImport.setOnDemand(true);
					cu.imports().add(newImport);
				}
			}
			// this ASTVistor is not in the offical API but is internal
			// as there isn't one in the API we will use this one 
			// It's use to visite the AST and get it to output with indentation
			ASTFixFlattener visitor = new ASTFixFlattener();
			cu.accept(visitor);
			return visitor.getFormattedResult(0, "\n");
		}
		catch (Exception e)	{
			e.printStackTrace();
			return "";
		}
	}
}
