package org.objectweb.eclipsespeedo.actions;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.utils.SelectionTools;
import org.objectweb.eclipsespeedo.utils.UITools;
import org.objectweb.eclipsespeedo.wizards.MainCreationWizard;

/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class MainHelperAction implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public MainHelperAction() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action){
		if (SelectionTools.ensureSpeedoElementSelected(window)){
			try {					
				EclipseSpeedoPlugin p = EclipseSpeedoPlugin.getDefault();
				StructuredSelection ss = (StructuredSelection) p.getWorkbench().getActiveWorkbenchWindow().getSelectionService().getSelection();
				IJavaElement javaEl = (IJavaElement) ss.getFirstElement();;
				MainCreationWizard ew = new MainCreationWizard(javaEl);
				WizardDialog dialog = new WizardDialog(ew.getShell(), ew);
				dialog.open();
			}
			catch (Exception e){
				UITools.selectionError("For this action, you must before select a Speedo element : Project, Package, Java file or Class", window);
				e.printStackTrace();
			}
		}
	}

	

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
}