/*
 * Created on May 5, 2004
 * author: VEZIER Antoine
 * Internship Master: Hamon Catherine
 * 
 */

package org.objectweb.eclipsespeedo.drivers.xml;

import java.io.CharArrayReader;
import java.io.CharArrayWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import org.apache.xerces.parsers.SAXParser;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.helpers.DefaultHandler;



/**
 * Builds the config objects from their XML definitions
 * 
 * @author B. Grkem ERCAN
 */
public class ConfigBuilder extends DefaultHandler
{
	private BuildConfigInterface config = null;
	private CharArrayWriter contents = new CharArrayWriter();
	private static Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger("org.objectweb.eclipsespeedo.EclipseSpeedoPlugin");
	
	public ConfigBuilder(BuildConfigInterface buildConfig)	{
		super();
		this.config = buildConfig;
	}

	/**
	 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
	 */
	public void characters(char[] ch, int start, int length) throws SAXException {
		contents.write(ch, start, length);
	}

	/**
	 * @see org.xml.sax.ContentHandler#endElement(String, String, String)
	 */
	public void endElement(String nameSpaceURI, String localName, String qName)	throws SAXException {
		config.assesTagEnd(qName, localName, contents.toString());
	}

	/**
	 * @see org.xml.sax.ContentHandler#startElement(String, String, String, Attributes)
	 */
	public void startElement(String nameSpaceURI, String localName,	String qName, Attributes attributes) throws SAXException {
		contents.reset();
		config.assesTagStart(qName, localName,attributes);
	}

	/**
	 * Method getConfigObject.
	 * @return Object
	 */
	public Object getConfigObject()	{
		return this.config.getConfigObject();
	}
	

	
	public static DriverDefinition getDriverDefinition(String fileName) {
		try	{
			logger.log(BasicLevel.DEBUG, "Entering getDriverDefinition(), arg : "+fileName);
			return (DriverDefinition) doParse(fileName, new DriverDefinitionConfiguration());
		}
		catch (Exception e)	{
			e.printStackTrace();//J2EEPlugin.log(e);
			return null;
		}
	}


	
	private static Object doParse(String fileName, BuildConfigInterface cf)	throws	SAXNotRecognizedException,
																					SAXNotSupportedException,
																					FileNotFoundException,
																					SAXException,
																					IOException	{
		logger.log(BasicLevel.DEBUG, "Entering doParse(), arg : "+fileName);																						
		org.apache.xerces.parsers.SAXParser parser = new SAXParser();
		parser.setFeature("http://xml.org/sax/features/validation", false);
		parser.setFeature(
			"http://apache.org/xml/features/nonvalidating/load-external-dtd",
			false);
		parser.setContentHandler(new ConfigBuilder(cf));
		InputSource source = new InputSource(new FileReader(fileName));
		parser.parse(source);
		logger.log(BasicLevel.DEBUG, "Exiting doParse(), arg : "+fileName);	
		return cf.getConfigObject();
	}
	
	private static Object parseString(String content, BuildConfigInterface cf)
		throws
			SAXNotRecognizedException,
			SAXNotSupportedException,
			FileNotFoundException,
			SAXException,
			IOException
	{
		org.apache.xerces.parsers.SAXParser parser = new SAXParser();
		parser.setFeature("http://xml.org/sax/features/validation", false);
		parser.setFeature(
			"http://apache.org/xml/features/nonvalidating/load-external-dtd",
			false);
		parser.setContentHandler(new ConfigBuilder(cf));
		InputSource source = new InputSource(new CharArrayReader(content.toCharArray()));
		parser.parse(source);
		return cf.getConfigObject();
	}
	


}
