/*
 * Created on May 7, 2004
 * author: VEZIER Antoine
 * Internship Master: Hamon Catherine
 * 
 */

package org.objectweb.eclipsespeedo.drivers.xml;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.util.monolog.api.Logger;



public class DriverDefinition  implements IPropertySource {
	

	private List fDescriptors;
	private File definitionFile;
	private String name;

	private EclipsePrefConf Eclipseprefconf;
	private JDOProjectConf JDOprojconf;


	private static Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(DriverDefinition.class.getName());
	
	public DriverDefinition() {
		Eclipseprefconf = new EclipsePrefConf();
		JDOprojconf = new JDOProjectConf();
	}

	public void addDriverClasspath(ClasspathItem cpi) {
		Eclipseprefconf.addDriverClasspath(cpi);
	}

	public void setDriverClasspath(ArrayList list) {
		Eclipseprefconf.setDriverClassPath(list);
	}
	
	public void setProjClasspath(ArrayList list) {
		Eclipseprefconf.setProjClasspath(list);
	}
	
	/**
	 * @return ArrayList
	 */
	public ArrayList getEclipsePreferencesProperties() {
		return Eclipseprefconf.getProperties();
	}
	
	public JDOProjectConfGroup getJDOProjectGroup(String name) {
		return JDOprojconf.getGroup(name);
	}
	
	public void addJDOProjectGroup(JDOProjectConfGroup grp) {
		JDOprojconf.addGroup(grp);
	}

	public DriverDefinitionProperty getPropertyNamed(String id) {
		Iterator itr = getEclipsePreferencesProperties().iterator();
		while (itr.hasNext()) {
			DriverDefinitionProperty element = (DriverDefinitionProperty) itr.next();
			if (id.equals(element.getId()))
				return element;
		}
		
		itr = JDOprojconf.getGroups().iterator();
		while (itr.hasNext()) {
			Iterator ite = ((JDOProjectConfGroup) itr.next()).getProperties().iterator();
			while (ite.hasNext()) {
				DriverDefinitionProperty element = (DriverDefinitionProperty) ite.next();
				if (id.equals(element.getId()))
					return element;
			}
		}
		return null;
	}

	/**
	 * Sets the properties.
	 * @param properties The properties to set
	 */
	public void setProperties(EclipsePrefConf epc) {
		this.Eclipseprefconf = epc;
	}

	/**
	 * @return String
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name.
	 * @param name The name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	public String toString() {
		Iterator props;
		
		String xml = "\n<driverDefinition name=\""+ getName()+ "\">\n";
		xml += Eclipseprefconf.toString();
		xml += JDOprojconf.toString();		
		xml += "\n</driverDefinition>";
		return xml;
	}

	/**
	 * @return File
	 */
	public File getDefinitionFile() {
		return definitionFile;
	}

	/**
	 * Sets the definitionFile.
	 * @param definitionFile The definitionFile to set
	 */
	public void setDefinitionFile(File definitionFile) {
		this.definitionFile = definitionFile;
	}


	/**
	 * @param str
	 * @return
	 */
	private String fixPassthroughProperties(String str) {
		String resolvedString = str;
		if (isPassPropertyLeft(resolvedString)) {
			resolvedString = fixParam(resolvedString);
			resolvedString = fixPassthroughProperties(resolvedString);
		}
		return resolvedString;
	}

	private String resolvePropertiesFromCache(
		String proppedString,
		HashMap cache) {
		String resolvedString = proppedString;
		if (isPropertyLeft(resolvedString)) {
			resolvedString = resolveProperty(resolvedString, cache);
			resolvedString = resolvePropertiesFromCache(resolvedString, cache);
		}
		return resolvedString;
	}

	private boolean isPropertyLeft(String str) {
		return str.indexOf("${") >= 0;
	}
	private boolean isPassPropertyLeft(String str) {
		return str.indexOf("%{") >= 0;
	}

	private String resolveProperty(String proppedString, HashMap cache) {
		String str = proppedString;
		int start = str.indexOf("${");
		int end = str.indexOf("}", start);
		String key = str.substring(start + 2, end);

		return str.substring(0, start)
			+ cache.get(key)
			+ str.substring(end + 1);
	}
	
	private String fixParam(String proppedString) {
		String str = proppedString;
		int start = str.indexOf("%{");
		return str.substring(0, start)
			+ "${"
			+ str.substring(start+2);
	}
	
	public Object getEditableValue()
	{
		return this;
	}

	public IPropertyDescriptor[] getPropertyDescriptors(){
		if(fDescriptors == null){
			fDescriptors = new ArrayList();
			fDescriptors.add(new PropertyDescriptor("targeted_server_name", "Name"));
			fDescriptors.add(new PropertyDescriptor("definition_file", "Definition file"));
			fDescriptors.add(new PropertyDescriptor("start_class", "Start class"));
			fDescriptors.add(new PropertyDescriptor("stop_class", "Stop class"));
			fDescriptors.add(new PropertyDescriptor("start_VM_arguments", "Start VM arguments"));
			fDescriptors.add(new PropertyDescriptor("server_home", "Server home"));
			fDescriptors.add(new PropertyDescriptor("stop_vm_arguments", "Stop VM arguments"));
			fDescriptors.add(new PropertyDescriptor("start_program_arguments", "Start program arguments"));
			fDescriptors.add(new PropertyDescriptor("stop_program_arguments", "Stop program arguments"));
			fDescriptors.add(new PropertyDescriptor("jndi_initial_context_factory", "JNDI Initial context factory"));
			fDescriptors.add(new PropertyDescriptor("jndi_url", "JNDI Provider URL"));
			fDescriptors.add(new PropertyDescriptor("start_working_dir", "Start working directory"));
			fDescriptors.add(new PropertyDescriptor("stop_working_dir", "Stop working directory"));

			for(int i = 0; i < fDescriptors.size(); i++){
				PropertyDescriptor tDesc = (PropertyDescriptor)fDescriptors.get(i);
				tDesc.setCategory("Derived server properties");
			}

			/*for(int i = 0; i < properties.size(); i++) {
				DriverDefinitionProperty p = (DriverDefinitionProperty)properties.get(i);
				PropertyDescriptor desc = new TextPropertyDescriptor(p.getId(), p.getLabel() + "(" + p.getId() + ")");
				desc.setCategory("Server definition properties");
				fDescriptors.add(desc);
			}*/
		}
		return (IPropertyDescriptor[])fDescriptors.toArray(new IPropertyDescriptor[fDescriptors.size()]);
	}

	public Object getPropertyValue(Object id)	{
		if(id.equals("definition_file"))
			return definitionFile.getPath();
		DriverDefinitionProperty prop = getPropertyNamed((String)id);
		if(prop != null)
			return prop.getDefaultValue();
		else
			return null;
	}

	public boolean isPropertySet(Object id)
	{
		return false;
	}

	public void resetPropertyValue(Object obj)	{
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
		DriverDefinitionProperty prop = getPropertyNamed((String)id);
		int idx = Eclipseprefconf.getProperties().indexOf(prop);
		if(idx >= 0) {
			prop.setDefaultValue((String)value);
			Eclipseprefconf.getProperties().set(idx, prop);
			EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().update(this);
			return;
		}
		Iterator itr = JDOprojconf.getGroups().iterator();
		while (itr.hasNext()) {
			JDOProjectConfGroup dogrp = (JDOProjectConfGroup) itr.next();
			idx = dogrp.getProperties().indexOf(prop);
			if(idx >= 0) {
				prop.setDefaultValue((String)value);
				dogrp.getProperties().set(idx, prop);
				EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().update(this);
				return;
			}
		}
	}
	/**
	 * @return
	 */
	public EclipsePrefConf getEclipseprefconf() {
		return Eclipseprefconf;
	}

	/**
	 * @return
	 */
	public JDOProjectConf getJDOprojconf() {
		return JDOprojconf;
	}

	/**
	 * @param conf
	 */
	public void setEclipseprefconf(EclipsePrefConf conf) {
		Eclipseprefconf = conf;
	}

	/**
	 * @param conf
	 */
	public void setJDOprojconf(JDOProjectConf conf) {
		JDOprojconf = conf;
	}



}
