/*
 * Created on May 7, 2004
 * author: VEZIER Antoine
 * Internship Master: Hamon Catherine
 * 
 */

package org.objectweb.eclipsespeedo.drivers.xml;

import java.util.ArrayList;

import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.util.monolog.api.Logger;
import org.xml.sax.Attributes;


public class DriverDefinitionConfiguration implements BuildConfigInterface {


	static final int ECONFPOINT = 0;
	static final int PCONFPOINT = 1;
	static final int UNDEFCONFPOINT = -1;
	
	static final int DCPATH = 0;
	static final int PCPATH = 1;
	static final int UNDEFCPATH = -1;
	
	DriverDefinition currentDefinition;
	private int curConfPoint = UNDEFCONFPOINT;
	private int curCPath = UNDEFCPATH;
	private String curJDOProjGroup = "";
	private ClasspathItem currentClasspathItem;
	private ArrayList driverClasspath;
	private ArrayList projClasspath;
	private Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(DriverDefinitionConfiguration.class.getName());
	

	public void assesTagEnd(String qName, String tagName, String content) {
		if ("driverClassPath".equals(tagName)) {
			currentDefinition.setDriverClasspath(driverClasspath);
			curCPath = UNDEFCPATH;
		} else if ("projectClassPath".equals(tagName)){
			currentDefinition.setProjClasspath(projClasspath);
			curCPath = UNDEFCPATH;
		} else if ("EclipsePreferences".equals(tagName)){
			curConfPoint = UNDEFCONFPOINT;
		} else if ("JDOProject".equals(tagName)){
			curConfPoint = UNDEFCONFPOINT;
		} else if ("Group".equals(tagName)){
			curJDOProjGroup = "";
		} else if ("jar".equals(tagName)) {
			currentClasspathItem.setClasspath(content);
			if (curCPath == DCPATH){
				driverClasspath.add(currentClasspathItem);
			}
			else if (curCPath == PCPATH){
				projClasspath.add(currentClasspathItem);
			}
		}
	}


	public void assesTagStart(String qName, String tagName, Attributes attributes) {
		if ("driverDefinition".equals(tagName)) {
			currentDefinition = new DriverDefinition();
			currentDefinition.setName(attributes.getValue("name"));
		} else if ("EclipsePreferences".equals(tagName)){
			curConfPoint = ECONFPOINT;
		} else if ("JDOProject".equals(tagName)){
			curConfPoint = PCONFPOINT;
		} else if ("Group".equals(tagName)){
			curJDOProjGroup = attributes.getValue("name");
			currentDefinition.addJDOProjectGroup(new JDOProjectConfGroup(curJDOProjGroup, attributes.getValue("description")));
		} else if ("property".equals(tagName)) {
			DriverDefinitionProperty property = new DriverDefinitionProperty();
			property.setId(attributes.getValue("id"));
			property.setLabel(attributes.getValue("label"));
			property.setDefaultValue(attributes.getValue("default"));
			property.setTypeFromStr(attributes.getValue("type"));
			property.setModifiable(attributes.getValue("modifiable"));
			property.setDesc(attributes.getValue("description"));
			if (curConfPoint == ECONFPOINT){
				currentDefinition.getEclipsePreferencesProperties().add(property);
			}
			else if (curConfPoint == PCONFPOINT){
				currentDefinition.getJDOProjectGroup(curJDOProjGroup).addProperty(property);
			}
		} else if ("jar".equals(tagName)) {
			currentClasspathItem = new ClasspathItem();
			if (curCPath == DCPATH){
				currentClasspathItem.setTypeStr(attributes.getValue("type"));
			}
			else if (curCPath == PCPATH){
				currentClasspathItem.setTypeStr(attributes.getValue("type"));
			}
		} else if ("driverClassPath".equals(tagName)) {
			curCPath = DCPATH;
			driverClasspath = new ArrayList();
		} else if ("projectClassPath".equals(tagName)) {
			curCPath = PCPATH;
			projClasspath = new ArrayList();
		}
	}

	public Object getConfigObject() {
		return currentDefinition;
	}
}
