/*
 * Created on May 7, 2004
 * author: VEZIER Antoine
 * Internship Master: Hamon Catherine
 * 
 */

package org.objectweb.eclipsespeedo.drivers.xml;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.runtime.Path;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;


public class DriverDefinitionManager {
	
	ArrayList definitions;
	File sourceDir;
	Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(this.getClass().getName());
	
	public DriverDefinitionManager(Path installUrl) {
		logger.log(BasicLevel.DEBUG, "Creating driver definition manager["+installUrl+"]");
		String s1 = installUrl.append("/drivers").toOSString();
		sourceDir = new File(s1);
		refresh();
	}
	
	
	public void refresh() {
		definitions = scanFiles(sourceDir);
	}
	
	public void update(DriverDefinition element) {
		toFile(element);
	}	
	
	public void update() {
		Iterator defs = definitions.iterator();
		while (defs.hasNext()) {
			DriverDefinition element = (DriverDefinition) defs.next();
			update(element);
		}
	}
	
	private void toFile(DriverDefinition def)	{
		try {
			def.getDefinitionFile().renameTo(new File(def.getDefinitionFile().getCanonicalFile()+".bak"));
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR,e);
		}
		try {
			FileOutputStream out = new FileOutputStream(def.getDefinitionFile());
			out.write(def.toString().getBytes());
			out.close();
		} catch (FileNotFoundException e) {
			logger.log(BasicLevel.ERROR,e);
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR,e);
		}
	}
	
	private  ArrayList scanFiles(File dir) {
		logger.log(BasicLevel.DEBUG, "Entering DriverDefinitionManager.ScanFiles()");
		ArrayList all = new ArrayList();
		try {
			if(dir.isDirectory()){
				File[] allDrivers = dir.listFiles(new FilenameFilter(){
					public boolean accept(File dir, String name) {
						if(name.endsWith(".driver"))
							return true;
						return false;
					}
				});
				
				for (int i = 0; i < allDrivers.length; i++) {
					File file = allDrivers[i];
					logger.log(BasicLevel.DEBUG, "Starting reading driver definition["+file+"]");
					DriverDefinition def = (DriverDefinition) ConfigBuilder.getDriverDefinition(file.getCanonicalPath());
					logger.log(BasicLevel.DEBUG, "Ending reading driver definition["+file+"]");
					if(def != null){
						def.setDefinitionFile(file);
						all.add(def);
					}
				}
			}
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR,e);
		}
		catch (Exception e) {
				   logger.log(BasicLevel.ERROR,e);
		}
		return all;
	}

	/**
	 * @return ArrayList
	 */
	public ArrayList getDefinitions() {
		return definitions;
	}

	/**
	 * @return ArrayList
	 */
	public DriverDefinition getDefinitionNamed(String name) {
		refresh();
		Iterator defs = getDefinitions().iterator();
		while (defs.hasNext()) {
			DriverDefinition elem = (DriverDefinition) defs.next();
			if(name.equals(elem.getName()))
				return elem;			
		}		
		return null;
	}

	/**
	 * Sets the definitions.
	 * @param definitions The definitions to set
	 */
	public void setDefinitions(ArrayList definitions) {
		this.definitions = definitions;
	}

}
