/*
 * Created on Apr 22, 2004
 * author: VEZIER Antoine
 * Internship Master: Hamon Catherine
 * 
 */

package org.objectweb.eclipsespeedo.enhancer;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.ui.DebugUITools;
import org.eclipse.jdt.internal.core.JavaProject;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.Event;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.IEclipseSpeedoConstants;
import org.objectweb.eclipsespeedo.drivers.xml.ClasspathItem;
import org.objectweb.eclipsespeedo.drivers.xml.DriverDefinition;
import org.objectweb.eclipsespeedo.project.EclipseSpeedoProjectHelper;
import org.objectweb.eclipsespeedo.project.ProjectProperties;


public class StartAction extends Action implements IAction, IJavaLaunchConfigurationConstants {
	
	protected IVMInstall jre;
	protected JavaProject jproject;

	public StartAction(IVMInstall vm) {
		super(vm.getName());
		jre = vm;
	}

	public StartAction(IVMInstall vm, JavaProject project) {
		super(vm.getName());
		jproject = project;  
		jre = vm;
	}

	/**
	 * @see IAction#run()
	 */
	public void run() {
		try {
			EclipseSpeedoProjectHelper.removeClassFiles(jproject);
			jproject.getProject().build(IncrementalProjectBuilder.FULL_BUILD, null);
			
			CmdLineHelper cmdHelper = new CmdLineHelper();
			ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();
			ILaunchConfigurationType type = manager.getLaunchConfigurationType(ID_JAVA_APPLICATION);

			DriverDefinition def = EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().getDefinitionNamed(ProjectProperties.getProjectProperty(jproject.getProject(), "Driver"));
			
			ILaunchConfiguration[] configurations = manager.getLaunchConfigurations(type);
			for (int i = 0; i < configurations.length; i++) {
				ILaunchConfiguration configuration = configurations[i];
				if (configuration.getName().equals(ProjectProperties.getProjectProperty(jproject.getProject(), "Driver"))) {
					configuration.delete();
					break;
				}
			}
			
			ILaunchConfigurationWorkingCopy workingCopy = type.newInstance(null, ProjectProperties.getProjectProperty(jproject.getProject(), "Driver"));
			
			// specify a JRE
			workingCopy.setAttribute(ATTR_VM_INSTALL_NAME, jre.getName());
			workingCopy.setAttribute(ATTR_VM_INSTALL_TYPE, jre.getVMInstallType().getId());
			
			// specify main type and program arguments
			workingCopy.setAttribute(ATTR_MAIN_TYPE_NAME, ProjectProperties.getSelDriverProperty(jproject.getProject(), "EclipseSpeedo.enhancerMain"));
			if (ProjectProperties.getProjectProperty(jproject.getProject(), "Driver").equals("Speedo")) workingCopy.setAttribute(ATTR_PROGRAM_ARGUMENTS, cmdHelper.getSpeedoArgs());
			else workingCopy.setAttribute(ATTR_PROGRAM_ARGUMENTS, cmdHelper.getArgs());
		
			// specify classpath
			File jdkHome = jre.getInstallLocation();
			IPath toolsPath = new Path(jdkHome.getAbsolutePath()).append("lib").append("tools.jar");
			IRuntimeClasspathEntry toolsEntry = JavaRuntime.newArchiveRuntimeClasspathEntry(toolsPath);
			toolsEntry.setClasspathProperty(IRuntimeClasspathEntry.ARCHIVE);

			IPath systemLibsPath = new Path(JavaRuntime.JRE_CONTAINER);
			IRuntimeClasspathEntry systemLibsEntry = JavaRuntime.newRuntimeContainerClasspathEntry(systemLibsPath, IRuntimeClasspathEntry.STANDARD_CLASSES);
			
			IRuntimeClasspathEntry ProjEntry = JavaRuntime.newRuntimeContainerClasspathEntry(cmdHelper.getProjectAbsolutePath(), IRuntimeClasspathEntry.USER_CLASSES);
			ProjEntry.setClasspathProperty(IRuntimeClasspathEntry.USER_CLASSES);
			
			List classpath = new ArrayList();
			classpath.add(toolsEntry.getMemento());
			classpath.add(systemLibsEntry.getMemento());
			classpath.add(ProjEntry.getMemento());
			for (Iterator ite = def.getEclipseprefconf().getDriverClassPath().iterator();ite.hasNext();){
				ClasspathItem cpitem = (ClasspathItem)ite.next();
				classpath.add(createMemento(new Path (cpitem.getClasspath()).toOSString()));
			}

			workingCopy.setAttribute(ATTR_WORKING_DIRECTORY, Platform.getLocation().toString()+jproject.getPath().toString());
			workingCopy.setAttribute(ATTR_CLASSPATH, classpath);
			List l = workingCopy.getAttribute(ATTR_CLASSPATH, new ArrayList());
			//	for (Iterator i = l.iterator(); i.hasNext();) System.out.println((String) i.next());
			workingCopy.setAttribute(ATTR_DEFAULT_CLASSPATH, false);
			// save and launch
			ILaunchConfiguration configuration = workingCopy.doSave();
			jproject.getProject().build(IncrementalProjectBuilder.FULL_BUILD, null);
			
			DebugUITools.launch(configuration, ILaunchManager.RUN_MODE);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	static public IRuntimeClasspathEntry getEntry(String jar){
		IPluginRegistry registry = Platform.getPluginRegistry();
		IPluginDescriptor desciptor = registry.getPluginDescriptor(IEclipseSpeedoConstants.PLUGIN_ID);
		IPath Path = new Path(EclipseSpeedoProjectHelper.computeJarPath(desciptor.getInstallURL(), null, jar));
		IRuntimeClasspathEntry Entry = JavaRuntime.newArchiveRuntimeClasspathEntry(Path);
		Entry.setClasspathProperty(IRuntimeClasspathEntry.ARCHIVE);
		return Entry;
	}
	
	static public IPath getPath(String jar){
		IPluginRegistry registry = Platform.getPluginRegistry();
		IPluginDescriptor desciptor = registry.getPluginDescriptor(IEclipseSpeedoConstants.PLUGIN_ID);
		IPath Path = new Path(EclipseSpeedoProjectHelper.computeJarPath(desciptor.getInstallURL(), null, jar));
		return Path;
	}
	
	private String createMemento(String arg) {
		return "<runtimeClasspathEntry externalArchive=\""
			+ arg
			+ "\" path=\"3\" type=\"2\"/>";
	}
	
	public void runWithEvent(Event event) {
		run();
	}
}
