package org.objectweb.eclipsespeedo.metadata.initializers;

import java.util.HashMap;

import org.objectweb.eclipsespeedo.metadata.JDOClass;
import org.objectweb.eclipsespeedo.metadata.JDOField;
import org.objectweb.eclipsespeedo.metadata.JDOObject;
import org.objectweb.eclipsespeedo.metadata.JDOPackage;
import org.objectweb.eclipsespeedo.metadata.jdt.initializers.Initializer;


public class InitializerRegistry {

	static protected InitializerRegistry registry = null;

	protected HashMap extensionRegistry = new HashMap();
	protected HashMap classRegistry = new HashMap();
	protected HashMap fieldRegistry = new HashMap();
	protected HashMap packageRegistry = new HashMap();

	static protected InitializerRegistry getRegistry()	{
		if (registry == null) {
			registry = new InitializerRegistry();
			JDOInitializer initializer = new Initializer();
			initializer.registerInitializers();
		}
		return registry;
	}

	protected void registerClass(Class clazz, JDOClassInitializer initializer) {
		classRegistry.put(clazz, initializer);
	}

	static public void registerClassInitializer(Class clazz, JDOClassInitializer initializer) {
		getRegistry().registerClass(clazz, initializer);
	}

	protected void unregisterClass(Class clazz)	{
		classRegistry.remove(clazz);
	}

	static public void unregisterClassInitializer(Class clazz)	{
		getRegistry().unregisterClass(clazz);
	}

	static public void initializerClass(JDOClass jdoClass, Object context)	{
		getRegistry().doInitializeClass(jdoClass, context);
	}

	protected void doInitializeClass(JDOClass jdoClass, Object context)	{
		JDOClassInitializer classInitializer = (JDOClassInitializer) getInitializer(classRegistry,	context.getClass());
		if (classInitializer != null) {
			classInitializer.initialize(jdoClass, context);
		}
	}
	
	protected void registerExtension(Class clazz, JDOExtensionInitializer initializer) {
		classRegistry.put(clazz, initializer);
	}

	static public void registerExtensionInitializer(Class clazz, JDOExtensionInitializer initializer) {
		getRegistry().registerExtension(clazz, initializer);
	}

	protected void unregisterExtension(Class clazz)	{
		extensionRegistry.remove(clazz);
	}

	static public void unregisterExtensionInitializer(Class clazz)	{
		getRegistry().unregisterExtension(clazz);
	}

	static public void initializerExtension(JDOObject jdoObject, Object context) {
		getRegistry().doInitializeExtension(jdoObject, context);
	}

	protected void doInitializeExtension(JDOObject jdoObject, Object context) {
		JDOExtensionInitializer extensionInitializer =
			(JDOExtensionInitializer) getInitializer(extensionRegistry,
				context.getClass());
		if (extensionInitializer != null) {
			extensionInitializer.initialize(jdoObject, context);
		}
	}
	
	protected void registerField(Class clazz, JDOFieldInitializer initializer)	{
		fieldRegistry.put(clazz, initializer);
	}

	static public void registerFieldInitializer(Class clazz, JDOFieldInitializer initializer){
		getRegistry().registerField(clazz, initializer);
	}

	protected void unregisterField(Class clazz)	{
		fieldRegistry.remove(clazz);
	}

	static public void unregisterFieldInitializer(Class clazz)	{
		getRegistry().unregisterField(clazz);
	}

	protected void doInitializeField(JDOField jdoField, Object context)	{
		JDOFieldInitializer fieldInitializer =	(JDOFieldInitializer) getInitializer(fieldRegistry,	context.getClass());
		if (fieldInitializer != null) {
			fieldInitializer.initialize(jdoField, context);
		}
	}

	static public void initializerField(JDOField jdoField, Object context) {
		getRegistry().doInitializeField(jdoField, context);
	}

	protected void registerPackage(Class clazz,	JDOPackageInitializer initializer) {
		packageRegistry.put(clazz, initializer);
	}

	static public void registerPackageInitializer(Class clazz,	JDOPackageInitializer initializer) {
		getRegistry().registerPackage(clazz, initializer);
	}

	protected void unregisterPackage(Class clazz) {
		packageRegistry.remove(clazz);
	}

	static public void unregisterPackageInitializer(Class clazz) {
		getRegistry().unregisterPackage(clazz);
	}

	protected void doInitializePackage(JDOPackage jdoPackage, Object context) {
		JDOPackageInitializer packageInitializer = (JDOPackageInitializer) getInitializer(packageRegistry,	context.getClass());
		if (packageInitializer != null)	{
			packageInitializer.initialize(jdoPackage, context);
		}
	}

	static public void initializerPackage(JDOPackage jdoPackage, Object context) {
		getRegistry().doInitializePackage(jdoPackage, context);
	}

	private static Object getInitializer(HashMap map, Class clazz) {
		if (clazz == null) {
			return null;
		}
		Object object = map.get(clazz);
		if (object != null)	{
			return object;
		}

		Class[] classes = clazz.getInterfaces();
		for(int i = 0; i<classes.length; i++) {
			Object initializer = getInitializer(map, classes[i]);
			if (initializer != null) {
				return initializer;
			}
		}
		Class superClass = clazz.getSuperclass();
		return getInitializer(map, superClass);
	}
}
