
package org.objectweb.eclipsespeedo.metadata.jdt.initializers;

import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.objectweb.eclipsespeedo.metadata.JDOField;
import org.objectweb.eclipsespeedo.metadata.MetadataFactory;
import org.objectweb.eclipsespeedo.metadata.initializers.JDOFieldInitializer;


public class FieldInitializer implements JDOFieldInitializer
{
	/** 
	 * Check if the field is an array, a collection aor a map and set the model
	 */
	public void initialize(JDOField jdoField, Object context)
	{
		IField field = (IField) context;
		IJavaProject javaProject = field.getJavaProject();
		IType type = (IType) field.getParent();
		try	{
			String signature = field.getTypeSignature();
			int count = Signature.getArrayCount(signature);
			// The field is an array if count>0
			if (count != 0)	{
				jdoField.setArray(MetadataFactory.eINSTANCE.createJDOArray());
				return;
			}
			String typeName = Signature.toString(signature);
			String [][] arrays = type.resolveType(typeName);
			if (arrays == null || arrays.length == 0){
				return;   // Type cannot be resolved
			}
			
			// ex: for object arays is {{java.lang},{Oblect}}
			String qualifierName = arrays[0][0] + "." + arrays[0][1];
						
			IType javaType = javaProject.findType(qualifierName);
			if (javaType != null){
				if (isSubTypeOf(javaType, "java.util.Collection"))	{
					jdoField.setCollection(MetadataFactory.eINSTANCE.createJDOCollection());
					return;
				}
				if (isSubTypeOf(javaType, "java.util.Map"))	{
					jdoField.setMap(MetadataFactory.eINSTANCE.createJDOMap());
					return;
				}
			}
		}
		catch (JavaModelException e){}
	}

	static public boolean isSubTypeOf(IType type, String typeName)	{
		String name = type.getFullyQualifiedName('.');
		if (name.equals(typeName)){
			return true;
		}
		try	{
			String[] superInterfaceNames = type.getSuperInterfaceNames();
			for (int i = 0; i < superInterfaceNames.length; i++){
				String[][] arrays = type.resolveType(superInterfaceNames[i]);
				if (arrays != null && arrays.length > 0){
					String qualifierName = arrays[0][0] + "." + arrays[0][1];
					IType superType =
						type.getJavaProject().findType(qualifierName);
					if (superType != null && isSubTypeOf(superType, typeName)){
						return true;
					}
				}
			}
		}
		catch (JavaModelException e){}
		return false;
	}
}
