/*
 * Created on Apr 2, 2004
 * author: VEZIER Antoine
 * Internship Master: Hamon Catherine
 * 
 */
package org.objectweb.eclipsespeedo.project;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.PropertyDialogAction;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.IEclipseSpeedoConstants;
import org.objectweb.eclipsespeedo.drivers.xml.ClasspathItem;
import org.objectweb.eclipsespeedo.metadata.MetadataModelManager;
import org.objectweb.eclipsespeedo.utils.JavaCodeHelper;
import org.objectweb.eclipsespeedo.utils.PropertiesFileGen;
import org.objectweb.util.monolog.api.Logger;

public class EclipseSpeedoProjectHelper {
	
	static private Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(EclipseSpeedoProjectHelper.class.getName());
	

	static public void addEclipseSpeedoNature(IJavaProject javaProject) {
		IProject project = javaProject.getProject();
		

		try	{
			if (!project.hasNature(JavaCore.NATURE_ID))	{
				MessageDialog.openError(EclipseSpeedoPlugin.getShell(),	
										"Error", 
										"This Project is not a Java Project");
				return;
			}
			if (project.hasNature(IEclipseSpeedoConstants.PROJECT_NATURE_ID)) {
				return;
			}
			IProjectDescription description = project.getDescription();
			String[] natures = description.getNatureIds();
			String[] newNatures = new String[natures.length + 1];
			System.arraycopy(natures, 0, newNatures, 1, natures.length);
			newNatures[0] = IEclipseSpeedoConstants.PROJECT_NATURE_ID;
			description.setNatureIds(newNatures);
			project.setDescription(description, null);
			// Generating initial logging property file			
			try {
				HashMap map1 = PropertiesFileGen.setLoggersLevels("logE", "DEBUG");
				HashMap map2 = PropertiesFileGen.setLoggersLevels("logR", "DEBUG");
				File outputFile = new File(Platform.getLocation().toOSString()+javaProject.getPath().toOSString()+File.separator+"logs.properties");
				PropertiesFileGen.genPropertiesFile("org/objectweb/eclipsespeedo/monologproperties.properties",outputFile, map1, map2, javaProject);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			try {
				ArrayList speedoPrCP = EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().getDefinitionNamed("Speedo").getEclipseprefconf().getProjClasspath();
				IClasspathEntry[] oldcpe = javaProject.getRawClasspath();
				IClasspathEntry[] newcpe = new IClasspathEntry[oldcpe.length+speedoPrCP.size()];
				System.arraycopy(oldcpe, 0, newcpe, 0, oldcpe.length);
				int index = oldcpe.length;
				ClasspathItem cpItem;
				String oldPCP="";
				for (Iterator ite = speedoPrCP.iterator(); ite.hasNext();){
					cpItem = (ClasspathItem) ite.next();
					newcpe[index++] = JavaCore.newLibraryEntry(new Path(cpItem.getClasspath()),null,null);
					oldPCP = oldPCP.concat(cpItem.getClasspath()+ " ");
				}
				ProjectProperties.setProjectProperty(project, oldPCP, "oldPCP");
				ProjectProperties.setProjectProperty(project, "Speedo", "Driver");
				javaProject.setRawClasspath(newcpe, new NullProgressMonitor());
				runProperty(project);
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
		catch (Exception e)	{
			MessageDialog.openError(EclipseSpeedoPlugin.getShell(),	
									"Error", 
									"an error has occured during adding Speedo nature to the project:\n"+e.toString());
			e.printStackTrace();
			return;
		}
	}
	
	static public String collectAllFiles(IJavaProject javaProject, String extension)	{
		StringBuffer buffer = new StringBuffer();
		IProject project = javaProject.getProject();
		/*
		 * look for anywhere
		 */
		try	{
			String outputLocation =	javaProject.getOutputLocation().removeFirstSegments(1).toString();
			if (project.getName().equalsIgnoreCase(outputLocation))	{
				outputLocation = "";
			}
			collectFiles(project, extension, buffer);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return buffer.toString();
	}
	
	static protected void collectFiles(IResource resource,	String extension, StringBuffer buffer)	{
		if (resource.getType() == IResource.FOLDER	|| resource.getType() == IResource.PROJECT	|| resource.getType() == IResource.ROOT){
			String path = resource.getProjectRelativePath().toString();

			try	{
				IResource[] membres = ((IContainer) resource).members();
				for (int i = 0; i < membres.length; i++) {
					collectFiles(membres[i], extension, buffer);
				}
			}
			catch (Exception e)	{
				e.printStackTrace();
			}
		}
		
		String ext = resource.getFileExtension();

		if (ext != null	&& ext.equalsIgnoreCase(extension)) {
			IPath path = resource.getProjectRelativePath();
			if (path != null) {
				buffer.append(" "+ path.toString() + " ");
			}
		}
	}
	
	static public String collectMetadataFiles(IJavaProject javaProject, String classes)	{
		StringBuffer buffer = new StringBuffer();
		StringTokenizer tokenizer = new StringTokenizer(classes);
		IProject project = javaProject.getProject();
		HashSet cache = new HashSet();
		while (tokenizer.hasMoreTokens()){
			String typeName = tokenizer.nextToken();
			if (typeName.endsWith(".class")){
				typeName = typeName.substring(0, typeName.length() - 6);
			}
			typeName = typeName.replace('\\', '.');
			typeName = typeName.replace('/', '.');
			try	{
				IType type = javaProject.findType(typeName);
				if (type != null)	{
					IResource resource = MetadataModelManager.INSTANCE.findMetadataFile(type);
					if (resource != null && !cache.contains(resource))	{
						buffer.append(" \"" + resource.getProjectRelativePath() + "\"");
						cache.add(resource);
					}
					for (IType superType = JavaCodeHelper.getSuperclass(type);superType != null;superType = JavaCodeHelper.getSuperclass(superType)){
						resource =	MetadataModelManager.INSTANCE.findMetadataFile(superType);
						if (resource != null && !cache.contains(resource))	{
							buffer.append(" \"" + resource.getProjectRelativePath() + "\"");
							cache.add(resource);
						}
					}
				}
			}
			catch (Exception e1)	{
				e1.printStackTrace();
			}
		}

			buffer.append(" ");

			return buffer.toString();
		}

	
	
	public static String computeJarPath(
		URL location,
		String driverPath,
		String name)
	{
		String jarPath = "";
		try
		{
			Path namePath = new Path(name);
			if (namePath.isAbsolute())
			{
				return name;
			}

			if (driverPath != null)
			{
				String fileSeparator = System.getProperty("file.separator");
				if (driverPath.endsWith(fileSeparator))
				{
					return driverPath + name;
				}
				return driverPath + fileSeparator + name;
			}

			URL installURL = new URL(location, name);
			URL localURL = Platform.asLocalURL(installURL);
			jarPath = (new Path(localURL.getPath())).toOSString();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		return jarPath;
	}
	
	
	static public void runProperty(IProject project)
	{
		Shell shell = EclipseSpeedoPlugin.getShell();
		ISelectionProvider provider = new ISelectionProvider(){
			ISelection selection;
			public void addSelectionChangedListener(ISelectionChangedListener listener)
			{
			}

			public ISelection getSelection()
			{
				return selection;
			}

			public void removeSelectionChangedListener(ISelectionChangedListener listener)
			{
			}

			public void setSelection(ISelection selection)
			{
				this.selection = selection;
			}
		};

		StructuredSelection selection =	new StructuredSelection(new Object[] { project });
		provider.setSelection(selection);

		PropertyDialogAction action = new PropertyDialogAction(shell, provider);
		action.run();
	}
	
	static public String getProjectProperty(IProject project, QualifiedName qn) throws CoreException {
		String inputFolder = project.getPersistentProperty(qn);
		if (inputFolder == null) inputFolder = "";
		return inputFolder;
	}

	static public void setProjectProperty(IProject project, String value, QualifiedName qn) throws CoreException {
		project.setPersistentProperty(qn, value);
	}
	
	static public void setProjectProperty(IProject project, Integer value, QualifiedName qn) throws CoreException {
		project.setPersistentProperty(qn, value.toString());
	}
	public static String findMetadataDocType(){
		return IEclipseSpeedoConstants.METADATA_DOC_TYPE;
	}


	public static void removeClassFiles(IJavaProject jproject) {
		String files = collectAllFiles(jproject, "class");
		StringTokenizer st = new StringTokenizer(files);
		while (st.hasMoreTokens()) {
			(new File(st.nextToken())).delete();
		}
	}
}
