

package org.objectweb.eclipsespeedo.ui;

import java.util.Vector;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;


public class JDTTreeScopeContentProvider implements ITreeContentProvider
{
	private IResourceScopeValider valider;

	private Object[] empty = new Object[0];
	/**
	 * 
	 */
	public JDTTreeScopeContentProvider(IResourceScopeValider valider)
	{
		super();
		this.valider = valider;
	}

	public JDTTreeScopeContentProvider()
	{
		super();
		this.valider = null;
	}

	private void collectTypes(ICompilationUnit unit, Vector collector)
	{
		try
		{
			IType[] types = unit.getTypes();
			for (int i = 0; i < types.length; i++)
			{
				if (valider == null || valider.isValid(types[i]))
				{
					collector.addElement(types[i]);
				}
			}
		}
		catch (JavaModelException e)
		{
			e.printStackTrace();
		}
	}

	private void collectTypes(IType type, Vector collector)
	{
		try
		{
			IType[] types = type.getTypes();
			for (int i = 0; i < types.length; i++)
			{
				if (valider == null || valider.isValid(types[i]))
				{
					collector.addElement(types[i]);
				}
			}
		}
		catch (JavaModelException e)
		{
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement)
	{
		Vector vector = new Vector();
		if (parentElement instanceof IResource)
		{
			IResource resource = (IResource) parentElement;
			collectChildren(resource, vector);
		}
		else if (parentElement instanceof IJavaElement)
		{
			collectChildren((IJavaElement) parentElement, vector);
		}

		return vector.toArray();
	};

	public void collectChildren(IResource resource, Vector collector)
	{
		IProject project = resource.getProject();
		IJavaProject javaProject = JavaCore.create(project);

		try
		{
			switch (resource.getType())
			{
				case IResource.FILE :
					{
						IJavaElement element =
							javaProject.findPackageFragment(
								resource.getFullPath());

						if (element != null && element.exists())
						{
							collector.add(element);
						}

						IContainer container = (IContainer) resource;
						try
						{
							IResource[] members = container.members();
							for (int i = 0; i < members.length; i++)
							{
								if (members[i].getType() == IResource.FOLDER)
								{
									collectChildren(members[i], collector);
								}
							}
						}
						catch (CoreException e1)
						{
							e1.printStackTrace();
						}
					}
					break;
				case IResource.FOLDER :
					{
						IJavaElement element =
							javaProject.findPackageFragment(
								resource.getFullPath());

						if (element != null
							&& element.exists()
							&& (valider == null || valider.isValid(element)))
						{
							collector.add(element);
						}

						IContainer container = (IContainer) resource;
						try
						{
							IResource[] members = container.members();
							for (int i = 0; i < members.length; i++)
							{
								if (members[i].getType() == IResource.FOLDER)
								{
									collectChildren(members[i], collector);
								}
							}
						}
						catch (CoreException e1)
						{
							e1.printStackTrace();
						}
					}
			}
		}
		catch (JavaModelException e1)
		{
			e1.printStackTrace();
		}
	}

	public void collectChildren(IJavaElement javaElement, Vector collector)
	{
		try
		{
			switch (javaElement.getElementType())
			{
				case IJavaElement.JAVA_PROJECT :
					{
						IPackageFragmentRoot[] roots =
							((IJavaProject) javaElement)
								.getPackageFragmentRoots();
						for (int i = 0; i < roots.length; i++)
						{
							if (roots[i].getKind()
								== IPackageFragmentRoot.K_SOURCE)
							{
								IJavaElement[] packageElements =
									roots[i].getChildren();
								for (int p = 0;
									p < packageElements.length;
									p++)
								{
									if (hasChildren(packageElements[p]))
									{
										if (valider == null
											|| valider.isValid(
												packageElements[p]))
											collector.add(packageElements[p]);
									}
								}
							}
						}
						break;
					}
				case IJavaElement.PACKAGE_FRAGMENT :
					{
						IPackageFragment root = (IPackageFragment) javaElement;
						ICompilationUnit[] units = root.getCompilationUnits();
						for (int i = 0; i < units.length; i++)
						{
							collectTypes(units[i], collector);
						}

						break;
					}
				case IJavaElement.TYPE :
					{
						IType type = (IType) javaElement;
						collectTypes(type, collector);
						IField[] fields = type.getFields();

						for (int i = 0; i < fields.length; i++)
						{
							if (valider == null || valider.isValid(fields[i]))
							{
								collector.addElement(fields[i]);
							}
						}
					}
					break;
			}
		}
		catch (JavaModelException e)
		{
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element)
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element)
	{
		return getChildren(element).length > 0;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement)
	{
		return getChildren(inputElement);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose()
	{
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
	{
	}

	//	private boolean okToAdd(IJavaElement javaElement)
	//	{
	//		int nbA =
	//			javaElement.getPath().matchingFirstSegments(
	//				myFile.getFullPath().removeFileExtension());
	//		int nbB = myFile.getFullPath().segmentCount();
	//		if (nbA == nbB) //the package is "in the folder" of the metadatafile
	//			return true;
	//
	//		//TODO : if the metadata filename corresponds to a Java class. it Doesn't work.
	//
	//		return false;
	//	}
}
