package org.objectweb.eclipsespeedo.ui;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.JavaElementLabelProvider;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.exceptions.ModificationException;
import org.objectweb.eclipsespeedo.metadata.JDOFile;
import org.objectweb.eclipsespeedo.metadata.MetadataModelManager;
import org.objectweb.eclipsespeedo.ui.metadataeditor.MetadataSwitch;

/**
 * 
 * Main JDO Medata graphic editor 
 */
public class MetadataEditor	extends EditorPart implements ISelectionChangedListener {
	
	private IFile metadataFile;
	SashForm splitter;
	Composite composerContainer;
	private TreeViewer fileTree;
	private boolean disposed;
	private MetadataSwitch composer;
	private DirtyAdapter dirtyListener = new DirtyAdapter();
	class DirtyAdapter extends AdapterImpl {
		public void notifyChanged(Notification msg)
		{
			updateTitle();
			firePropertyChange(PROP_DIRTY);
		}
	};

	/**
	 * 
	 */
	public MetadataEditor()	{
		super();
	}

	public String getContent() {
		return composer.getMeta().getMetadataContent();
	}

	public void dispose() {
		composer.getResource().eAdapters().remove(dirtyListener);
	}

	protected void updateTitle() {
		String modelDisplayName = metadataFile.getName();
		setTitle(modelDisplayName);
	}

	public IProject getProject() {
		return metadataFile.getProject();
	}

	public IJavaProject getJavaProject() {
		return JavaCore.create(getProject());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		try {
			IProject project = getJavaProject().getProject();

			Resource mainResouce = composer.getResource();
			ResourceSet resourceSet = mainResouce.getResourceSet();
			Iterator iterator = resourceSet.getResources().iterator();
			while (iterator.hasNext()) {
				Resource resource = (Resource) iterator.next();
				if (resource.isModified() && resource != mainResouce) {
					List content = resource.getContents();
					if (!content.isEmpty())	{
						JDOFile jdoFile = (JDOFile) content.get(0);
						jdoFile.shrink();
					}
					resource.save(Collections.EMPTY_MAP);
				}
			}

			List content = mainResouce.getContents();
			if (!content.isEmpty())	{
				JDOFile jdoFile = (JDOFile) content.get(0);
				jdoFile.shrink();
			}
			mainResouce.save(Collections.EMPTY_MAP);

			if (MessageDialog.openConfirm(EclipseSpeedoPlugin.getShell(),EclipseSpeedoPlugin.getResourceString("Dialog.confirm.title"), EclipseSpeedoPlugin.getFormattedString("Project.rebuild.description",project.getName())))	{
				try	{
					project.build(IncrementalProjectBuilder.FULL_BUILD, monitor);
				}
				catch (CoreException e)	{
					e.printStackTrace();
				}
			}
		}
		catch (Exception e)	{
			e.printStackTrace();
		}
		firePropertyChange(PROP_DIRTY);
	}


	public void doSaveAs() {}

	public void gotoMarker(IMarker marker) {}


	public void init(IEditorSite site, IEditorInput input)	throws PartInitException {
		IFileEditorInput fileInput = null;
		if (input instanceof IFileEditorInput) {
			fileInput = (IFileEditorInput) input;
			metadataFile = fileInput.getFile();

			Resource resource =	MetadataModelManager.INSTANCE.getResource(metadataFile, true);
			resource.setTrackingModification(true);
			this.composer = new MetadataSwitch(resource);

			this.composer.getResource().eAdapters().add(dirtyListener);
		}
		setSite(site);
		setInput(input);
		updateTitle();
	}

	public boolean isDirty() {
		Resource mainResouce = composer.getResource();
		ResourceSet resourceSet = mainResouce.getResourceSet();
		Iterator iterator = resourceSet.getResources().iterator();
		while (iterator.hasNext()) {
			Resource resource = (Resource) iterator.next();
			if (resource.isModified())	{
				return true;
			}
		}
		return false;
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	public void createPartControl(Composite parent) {
		try	{
			//main container
			splitter = new SashForm(parent, SWT.NONE);
			//Tree			
			initializeTree();
			initializeComposer();
			this.composer.setRootParent(this.composerContainer);
			splitter.setWeights(new int[] { 35, 65 });
		}
		catch (Exception e)	{
			e.printStackTrace();
		}
	}

	/**
	 * 
	 * @param container
	 */
	private void initializeComposer() {
		composerContainer = new Composite(splitter, SWT.NONE);
		composerContainer.setLayoutData(new GridData(GridData.FILL_BOTH));
		composerContainer.setVisible(true);

		composerContainer.setLayout(new GridLayout());
	}

	/**
	 * We create the tree from the java project ressource.
	 * We get the packages , the classes and the attributes
	 *
	 */
	private void initializeTree() {
		fileTree = new TreeViewer(splitter);
		JavaElementLabelProvider labelProvider = new JavaElementLabelProvider();

		Object scopeObject = getScopeObject();
		if (scopeObject instanceof IType) {
			IType root = ((IType) scopeObject);
			IPackageFragment parent = root.getPackageFragment();
			IResourceScopeValider scopeValider = new ClassScopeValider(parent, root);
			fileTree.setContentProvider(new JDTTreeScopeContentProvider(scopeValider));
			fileTree.setInput(parent);
		}
		else if (scopeObject instanceof IFolder) {
			IFolder folder = (IFolder) scopeObject;
			IJavaProject javaProject = JavaCore.create(getProject());
			IPackageFragment parent = null;
			try	{
				parent = (IPackageFragment) javaProject.findPackageFragment(folder.getFullPath());
			}
			catch (JavaModelException e1) {}

			if (parent != null)	{
				IResourceScopeValider scopeValider = new PackageScopeValider(parent);
				fileTree.setContentProvider(new JDTTreeScopeContentProvider(scopeValider));
			}
			else {
				fileTree.setContentProvider(new JDTTreeScopeContentProvider());
			}
			fileTree.setInput(folder);
		}
		else {
			fileTree.setContentProvider(new JDTTreeScopeContentProvider());
			fileTree.setInput(scopeObject);
		}
		fileTree.setLabelProvider(labelProvider);
		fileTree.addSelectionChangedListener(this);
	}

	protected Object getScopeObject() {
		//
		// check if it is class scope
		//
		IPath filePath = metadataFile.getProjectRelativePath();
		IPath parentPath = filePath.removeLastSegments(1);
		String typeName = filePath.removeFileExtension().lastSegment();
		IProject project = metadataFile.getProject();
		try	{
			IJavaProject javaProject = JavaCore.create(project);
			IContainer parentResource = metadataFile.getParent();
			IPackageFragment parentPackageElement =	javaProject.findPackageFragment(parentResource.getFullPath());

			IType type = null;
			if (parentPackageElement != null && parentPackageElement.exists()) {
				type = javaProject.findType(parentPackageElement.getElementName(), typeName);
			}

			if (type == null || !type.exists())	{
				type = javaProject.findType(typeName);
			}
			if (type != null && type.exists()) {
				return type;
			}
		}
		catch (JavaModelException e) {
			e.printStackTrace();
		}

		//if ("package.jdo".equalsIgnoreCase(filePath.lastSegment()))	{
			return project.getFolder(parentPath);
		//}

		//filePath = filePath.removeFileExtension();
		//IFolder folder = project.getFolder(filePath);
		//return folder;
	}

	/**
	 * Call a specific method , depending on the item categoty (package , class , attribute)
	 *
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		try	{
			IStructuredSelection selection = (IStructuredSelection) event.getSelection();
			//We only get the first selection

			IJavaElement selectedObject = (IJavaElement) selection.getFirstElement();

			//if there is no selection we do nothing
			if (selection == null)	return;

			switch (selectedObject.getElementType()) {
				case IJavaElement.PACKAGE_FRAGMENT :
					if (this.composer.packageSelected((IPackageFragment) selectedObject)) {
						composerContainer.update();
						splitter.update();
					}
					break;
				case IJavaElement.TYPE :
				try {
					if (this.composer.classSelected((IType) selectedObject)) {
						composerContainer.layout();
						splitter.update();
					}
				}
				catch (Exception e) {e.printStackTrace();
				}
					break;
				case IJavaElement.FIELD :
					if (this.composer.attributeSelected((IField) selectedObject)) {
						composerContainer.layout();
						splitter.update();
					}
					break;
			}
		}
		catch (ModificationException me) {
			//TODO utiliser aussi les status pour dire que le choix n'est pas valid?
			// mettre en relation status et metadata avec une methode donnant l'etat de isDataValidate
			// an modificationException is raised when the user didn't validate the changes and
			// select an other class or package
			me.printStackTrace();
		}
	}

	/**
	 * Search all the field in a class
	 * if there is no field it returns null
	 * @param packageElement 
	 * @param className
	 * @return IField[]
	 */
	private IField[] getAttributeClass(IJavaElement packageElement,	String className) {
		try	{
			IPackageFragment pf = getJavaProject().findPackageFragment(packageElement.getPath());
			if (pf == null)	return null;
			ICompilationUnit unit = pf.getCompilationUnit(className);
			if (unit == null) return null;
			if (!unit.exists()) return null;
			IType type = unit.getType(className.substring(0, className.length() - 5));
			if (type == null) return null;
			return type.getFields();
		}
		catch (Exception e)	{
			e.printStackTrace();
		}
		return null;
	}


	public void setFocus()	{}
}
