package org.objectweb.eclipsespeedo.ui;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.objectweb.eclipsespeedo.ui.XMLEditor.XMLEditor;

/**
 * This is the main editor in the Metadata editor. It manages two 
 * editors: <code>MetadataEditor</code> and <code>XMLEditor</code>
 */
public class MetadataEditorMain extends MultiPageEditorPart
{
	private MetadataEditor editor;
	private XMLEditor xmlEditor;
	private IFile file;

	/**
	 *
	 */
	public MetadataEditorMain() {}

	void createPage0() {
		try	{
			editor = new MetadataEditor();
			int index = addPage(editor, getEditorInput());
			setPageText(index, "Composer");
		}
		catch (PartInitException e)	{
			ErrorDialog.openError(getSite().getShell(),	"Error creating nested text editor",null,e.getStatus());
		}
	}

	void createPage1(){
		try	{
			xmlEditor = new XMLEditor();
			int index = addPage(xmlEditor, getEditorInput());
			setPageText(index, "Content");
		}
		catch (PartInitException e)	{
			ErrorDialog.openError(getSite().getShell(),"Error creating nested text editor",	null, e.getStatus());
		}
	}

	/**
	 * Creates the pages of the multi-page editor.
	 */
	protected void createPages()	{
		createPage0();
		createPage1();
	}

	/**
	 * Saves the multi-page editor's document.
	 */
	public void doSave(IProgressMonitor monitor){
		editor.doSave(monitor);
	}


	public void doSaveAs() {
		editor.doSaveAs();
	}


	public void gotoMarker(IMarker marker)	{}


	/**
	 * The <code>MultiPageEditorExample</code> implementation of this method
	 * checks that the input is an instance of <code>IFileEditorInput</code>.
	 */
	public void init(IEditorSite site, IEditorInput editorInput) throws PartInitException {
		if (!(editorInput instanceof IFileEditorInput))	throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		super.init(site, editorInput);

		IFileEditorInput fileInput = null;
		if (editorInput instanceof IFileEditorInput) {
			fileInput = (IFileEditorInput) editorInput;
		}
	}

	public boolean isDirty() {
		if (editor == null)	return false;
		return editor.isDirty();
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	/**
	 * Calculates the contents of page 2 when the it is activated.
	 */
	protected void pageChange(int newPageIndex)	{
		super.pageChange(newPageIndex);
		if (newPageIndex == 1){
			String content = editor.getContent();
			xmlEditor.setInput(new MemoryEditorInput(content));
		}
	}

	public void setFocus()	{}
}
