package org.objectweb.eclipsespeedo.ui.metadataeditor;

import java.util.Vector;

import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.jdoextensions.xml.JDOExtensionsConfiguration;
import org.objectweb.eclipsespeedo.metadata.JDOClass;
import org.objectweb.eclipsespeedo.metadata.JDOIdentityType;
import org.objectweb.eclipsespeedo.metadata.MetadataModelModifier;
import org.objectweb.util.monolog.api.Logger;

/**
 * JDO Class editor.
 * 
 */
public class ClassEditor extends AbstractJDOEditor
{
	protected Label classLabel;

	// elements of the class container
	protected Composite classAttributesContainer;
	protected Button classCheckBox;
	protected Combo classIdentityType;
	protected Text classObjectIDType;
	protected Button classRequiresExtent;
	protected Text classPersistenceCapableSuperclass;

	//Extension
	protected ExtensionConfigurationBlock classExtensionBlock;
	protected Control classExtensionControl;
	private Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(ClassEditor.class.getName());
	public ClassEditor(MetadataModelModifier modifier, IJavaProject project)
	{
		super(modifier, project);
	}

	/**
	 * 
	 *Initialize the class view 
	 */
	public Composite initializeClassContainer(Composite parent)
	{
		container = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		container.setLayoutData(gridData);
		GridLayout layout = new GridLayout();
		layout.verticalSpacing = 12;
		layout.numColumns = 2;
		container.setLayout(layout);

		Label classNameLabel = new Label(container, SWT.NONE);
		classNameLabel.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_className_label"));
		classNameLabel.setFont(new Font(container.getDisplay(),	classNameLabel.getFont().getFontData()[0].getName(), 8,	SWT.BOLD));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		classNameLabel.setLayoutData(gridData);

		classLabel = new Label(container, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		classLabel.setLayoutData(gridData);

		Label classPersistenceLabel = new Label(container, SWT.NONE);
		classPersistenceLabel.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_Persistence_label"));
		classPersistenceLabel.setFont(new Font(container.getDisplay(),classPersistenceLabel.getFont().getFontData()[0].getName(),8,	SWT.BOLD));
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		classPersistenceLabel.setLayoutData(gridData);

		classCheckBox = new Button(container, SWT.CHECK);
		classCheckBox.addListener(SWT.Selection, new Listener()	{
			public void handleEvent(Event e)
			{
				handleClassPersistenceChanged();
			}
		});

		//class attribute container
		classAttributesContainer = new Composite(container, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		classAttributesContainer.setLayoutData(gridData);
		layout = new GridLayout();
		layout.horizontalSpacing = 7;
		layout.verticalSpacing = 12;
		layout.numColumns = 3;
		classAttributesContainer.setLayout(layout);

		Label label = new Label(classAttributesContainer, SWT.NONE);
		label.setText(EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_identityType_label"));

		classIdentityType =	new Combo(classAttributesContainer, SWT.BORDER | SWT.READ_ONLY);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		classIdentityType.setLayoutData(gridData);
		classIdentityType.setItems(JDOCLASS_IDENTITY_TYPE_STRINGS);
		classIdentityType.addModifyListener(new ModifyListener(){
			public void modifyText(ModifyEvent e){
				if (classCheckBox.getSelection()&& (modifier.isClassPersistence()))	{
					JDOClass jdoClass = modifier.getJdoClass();
					int index = classIdentityType.getSelectionIndex();
					if (index < 0)	{
						index = 0;
					}
					jdoClass.setIdentityType(JDOIdentityType.get(index));
				}
				container.update();
			}
		});
		
		
		{
			label = new Label(classAttributesContainer, SWT.NONE);
			label.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_ObjectID_label"));

			classObjectIDType =	new Text(classAttributesContainer, SWT.BORDER);
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			classObjectIDType.setLayoutData(gridData);
			classObjectIDType.addModifyListener(new ModifyListener(){
				public void modifyText(ModifyEvent e){
					if (classCheckBox.getSelection()&& (modifier.isClassPersistence()))	{
						JDOClass jdoClass = modifier.getJdoClass();
						jdoClass.setObjectIdClass(classObjectIDType.getText());
					}
					container.update();
				}
			});
			
			Button button = new Button(classAttributesContainer, SWT.NONE);
			button.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_Browse_label"));
			gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
			button.setLayoutData(gridData);

			button.addSelectionListener(new SelectionListener()	{
				protected void handleSelected()	{
					String className = selectClassInProject();
					if (className != null){
						classObjectIDType.setText(className);
					}
				}

				public void widgetSelected(SelectionEvent e){
					handleSelected();
				}

				public void widgetDefaultSelected(SelectionEvent e)	{
					handleSelected();
				}
			});
		}


		label = new Label(classAttributesContainer, SWT.NONE);
		label.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_RequiresExtent_label"));
		classRequiresExtent = new Button(classAttributesContainer, SWT.CHECK);
		classRequiresExtent.setSelection(true);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		classRequiresExtent.setLayoutData(gridData); //
		classRequiresExtent.addSelectionListener(new SelectionListener()		{
			protected void handleSelected()	{
				if (classCheckBox.getSelection()&& (modifier.isClassPersistence()))	{
					JDOClass jdoClass = modifier.getJdoClass();
					jdoClass.setRequiresExtent(
						classRequiresExtent.getSelection());
				}
				container.update();
			}

			public void widgetSelected(SelectionEvent e){
				handleSelected();
			}

			public void widgetDefaultSelected(SelectionEvent e){
				handleSelected();
			}
		});

		{
			label = new Label(classAttributesContainer, SWT.NONE);
			label.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_PersistenceCapableSuperclass_label"));

			classPersistenceCapableSuperclass =	new Text(classAttributesContainer, SWT.BORDER);
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			classPersistenceCapableSuperclass.setLayoutData(gridData);
			classPersistenceCapableSuperclass.addModifyListener(new ModifyListener(){		
			public void modifyText(ModifyEvent e) {
					modifier.getJdoClass().setPersistenceCapableSuperclass(
						classPersistenceCapableSuperclass.getText());

					container.update();
				}
			});

			Button button = new Button(classAttributesContainer, SWT.NONE);
			button.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_Browse_label"));
			gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
			button.setLayoutData(gridData);

			button.addSelectionListener(new SelectionListener()	{
				protected void handleSelected()	{
					String className = selectClassInProject();
					if (className != null){
						modifier.getJdoClass().setPersistenceCapableSuperclass(className);
						classPersistenceCapableSuperclass.setText(className);
						container.update();
					}
				}

				public void widgetSelected(SelectionEvent e){
					handleSelected();
				}

				public void widgetDefaultSelected(SelectionEvent e)	{
					handleSelected();
				}
			});
		}
		
		this.classExtensionBlock = new ExtensionConfigurationBlock(null, currentJavaProject, JDOExtensionsConfiguration.CLASS);
		
		classExtensionControl =	this.classExtensionBlock.createContents(container);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 3;
		classExtensionControl.setLayoutData(gridData);

		classCheckBox.setSelection(false);

		classExtensionBlock.getExtensionList().getTableViewer().getTable().addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e)
			{

				Vector ext = classExtensionBlock.getExtension();

				modifier.setClassExtension(ext);
				container.update();
			}
		});

		return container;
	}

	/**
	 * When a class is selected, we change the current view and put the class container
	 * and we get values of this class
	 *
	 */
	public void updateClass() {

		boolean persistence = modifier.isClassPersistence();
		setEnabledClassAttributeContainer(persistence);

		classLabel.setText(modifier.getJavaClassName());
		classLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		classLabel.pack();
		classCheckBox.setSelection(persistence);

		if (persistence){
			classAttributesContainer.update();
			container.update();

			JDOClass jdoClass = modifier.getJdoClass();

			int index =	classIdentityType.indexOf(jdoClass.getIdentityType().getName());
			classIdentityType.select(index);
			if (index == -1) classIdentityType.setText("");

			classObjectIDType.setText(jdoClass.getObjectIdClass());
			classRequiresExtent.setSelection(jdoClass.isRequiresExtent());
			classExtensionBlock.setExtension(modifier.getClassExtension());
			String superClass = jdoClass.getPersistenceCapableSuperclass();
			if (superClass == null)	{
				superClass = "";
			}
			classPersistenceCapableSuperclass.setText(superClass);
			//classAttributesContainer.update();
		}
		else{
			containerInitialize = true;
			classIdentityType.setText("");
			classObjectIDType.setText("");
			classRequiresExtent.setText("");
			classExtensionBlock.removeAllElements();
			modifier.cleanClass();
			containerInitialize = false;
		}
		classCheckBox.setSelection(persistence);
	}

	private void setEnabledClassAttributeContainer(boolean persistence) {
		Control allChildrencontrol[] = classAttributesContainer.getChildren();
		for (int i = 0; i < allChildrencontrol.length; i++)	{
			allChildrencontrol[i].setEnabled(persistence);
		}
		if (classExtensionBlock == null) return;
		classExtensionBlock.setEnabled(persistence);
		classExtensionBlock.getExtensionList().setEnabled(persistence);
		classExtensionBlock.getExtensionList().getTableViewer().getTable().setEnabled(persistence);
	}

	/**
	 * When the user change the persistence state of a class
	 * If he unset, all  information related to his children is remove too. 
	 *
	 */
	public void handleClassPersistenceChanged()	{
		try		{
			container.update();
			classAttributesContainer.update();
			container.update();
			boolean check = classCheckBox.getSelection();
			modifier.setClassPersistence(check);

			updateClass();
		}
		catch (Exception e)	{
			e.printStackTrace();
		}
	}

}
