package org.objectweb.eclipsespeedo.ui.metadataeditor;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.internal.ui.dialogs.StatusInfo;
import org.eclipse.jdt.internal.ui.util.PixelConverter;
import org.eclipse.jdt.internal.ui.util.SWTUtil;
import org.eclipse.jdt.internal.ui.wizards.IStatusChangeListener;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.DialogField;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.IDialogFieldListener;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.IListAdapter;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.LayoutUtil;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.ListDialogField;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Table;
import org.objectweb.eclipsespeedo.metadata.JDOExtension;
/**
 * 
 */
public class ExtensionConfigurationBlock extends OptionsConfigurationBlock
{
	private int level;

	private PixelConverter fPixelConverter;

	private IStatus fExtensionStatus;
	private ListDialogField fExtensionList;
	private List fExtensions;
	private IJavaProject jproject;

	private static class ExtensionLabelProvider	extends LabelProvider implements ITableLabelProvider
	{


		public Image getImage(Object element){
			return null;
			// JavaPluginImages.get(JavaPluginImages.IMG_OBJS_REFACTORING_INFO);
		}

		public String getText(Object element){
			return getColumnText(element, 0);
		}

		public Image getColumnImage(Object element, int columnIndex){
			return null;
		}

		public String getColumnText(Object element, int columnIndex) {
			JDOExtension task = (JDOExtension) element;
			if (columnIndex == 0){
				return task.getVendorName();
			}
			else if (columnIndex == 1)	{
				return task.getKey();
			}
			else if (columnIndex == 2)	{
				return task.getValue();
			}
			else {
				return new Boolean(!task.getExtensions().isEmpty()).toString();
			}
		}
	}

	public ExtensionConfigurationBlock(IStatusChangeListener context, IJavaProject project, int lev) {
		super(context, project);
		this.jproject = project;
		this.level = lev;
		ExtensionAdapter adapter = new ExtensionAdapter();
		String[] buttons = new String[] {
			/* 0 */
			"Add", 
			/* 1 */
			"Remove",
			null,
			/* 3 */
			"Edit",
		};
		fExtensionList = new ListDialogField(adapter, buttons, new ExtensionLabelProvider());
		fExtensionList.setDialogFieldListener(adapter);

		fExtensionList.setLabelText("Extension");
		fExtensionList.setRemoveButtonIndex(1);
		String[] columnsHeaders = new String[] { "Vendor",
			"Key",
			"Value"
		};

		fExtensionList.setTableColumns(
			new ListDialogField.ColumnsDescription(columnsHeaders, true));
		//setExtension();
		if (fExtensionList.getSize() > 0){
			fExtensionList.selectFirstElement();
		}
		else{
			fExtensionList.enableButton(3, false);
		}

		fExtensionStatus = new StatusInfo();
	}

	public class ExtensionAdapter implements IListAdapter, IDialogFieldListener	{

		private boolean canEdit(ListDialogField field)	{
			return field.getSelectedElements().size() == 1;
		}

		public void customButtonPressed(ListDialogField field, int index)	{
			doTodoButtonPressed(index);
		}

		public void selectionChanged(ListDialogField field)	{
			field.enableButton(3, canEdit(field));
		}

		public void doubleClicked(ListDialogField field){
			if (canEdit(field))	{
				doTodoButtonPressed(3);
			}
		}

		public void dialogFieldChanged(DialogField field){
			try	{
				fExtensionList.getTableViewer().getTable().notifyListeners(SWT.Selection, new Event());
			}
			catch (Exception e)	{

			}
		}
	}

	public Control createContents(Composite parent) {
		fPixelConverter = new PixelConverter(parent);
		setShell(parent.getShell());

		Composite markersComposite = createMarkersTabContent(parent);

		//		validateSettings(null, null);

		return markersComposite;
	}

	private Composite createMarkersTabContent(Composite folder) {

		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.numColumns = 2;

		Composite markersComposite = new Composite(folder, SWT.NULL);
		markersComposite.setLayout(layout);
		markersComposite.setLayoutData(new GridData(GridData.FILL_BOTH));

		fExtensionList.doFillIntoGrid(markersComposite, 3);
		LayoutUtil.setHorizontalSpan(fExtensionList.getLabelControl(null), 2);

		Table table = fExtensionList.getTableViewer().getTable();
		GridData data =
			(GridData) fExtensionList.getListControl(null).getLayoutData();
		data.grabExcessHorizontalSpace = true;
		data.verticalAlignment = 0;
		data.heightHint = SWTUtil.getTableHeightHint(table, 6);

		return markersComposite;
	}

	public boolean performOk(boolean enabled){
		return super.performOk(enabled);
	}

	public void updateControls(){
	}

	public void setExtension(List extensions){
		this.fExtensions = extensions;
		fExtensionList.removeAllElements();
		if (fExtensions == null)
			fExtensions = new Vector();

		ArrayList elements = new ArrayList(fExtensions.size());
		for (int i = 0; i < fExtensions.size(); i++){
			JDOExtension ext = (JDOExtension) fExtensions.get(i);
			elements.add(ext);
		}
		fExtensionList.setElements(elements);
	}

	public Vector getExtension(){
		Vector vect = new Vector();
		List list = fExtensionList.getElements();
		for (int i = 0; i < list.size(); i++)	{
			JDOExtension elem = (JDOExtension) list.get(i);
			vect.addElement(elem);
		}
		return vect;
	}

	public void removeAllElements()	{
		fExtensionList.removeAllElements();
	}

	private void doTodoButtonPressed(int index)	{

		JDOExtension edited = null;
		if (index != 0)	{
			edited = (JDOExtension) fExtensionList.getSelectedElements().get(0);
		}

		ExtensionInputDialog dialog =
			new ExtensionInputDialog(
				getShell(),
				edited,
				jproject,
				this.level);

		dialog.create();
		int x = dialog.getShell().getBounds().x;
		int y = dialog.getShell().getBounds().y;
		int h = dialog.getShell().getBounds().height;
		int w = dialog.getShell().getBounds().width;
		dialog.getShell().setBounds(x + 10, y + 10, w, h);

		if (dialog.open() == ExtensionInputDialog.OK&&(dialog.getResult()!=null))
		{
			if ((edited != null)) {
				fExtensionList.replaceElement(edited, dialog.getResult());
			}
			else {
				fExtensionList.addElement(dialog.getResult());
			}
		}
	}

	public void setEnabled(boolean bool)
	{
		this.fExtensionList.setEnabled(bool);

	}

	public ListDialogField getExtensionList()
	{
		return this.fExtensionList;
	}

}
