package org.objectweb.eclipsespeedo.ui.metadataeditor;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.internal.ui.dialogs.StatusDialog;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.ComboDialogField;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.DialogField;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.IDialogFieldListener;
import org.eclipse.jdt.ui.IJavaElementSearchConstants;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.jdoextensions.xml.Extension;
import org.objectweb.eclipsespeedo.jdoextensions.xml.JDOExtensionsConfiguration;
import org.objectweb.eclipsespeedo.jdoextensions.xml.VendorExtensions;
import org.objectweb.eclipsespeedo.metadata.JDOExtension;
import org.objectweb.eclipsespeedo.metadata.MetadataFactory;
import org.objectweb.eclipsespeedo.project.ProjectProperties;
import org.objectweb.util.monolog.api.Logger;

/**
 * Dialog to enter a na new task tag
 */
public class ExtensionInputDialog extends StatusDialog implements SelectionListener{
	private class CompilerExtensionInputAdapter implements IDialogFieldListener	{
		public void dialogFieldChanged(DialogField field) {
			doValidation(field);
		}
	}

	private Text descText;
	private int level;
	private ComboDialogField fKeyDialogField;
	private Control Value;
	private IJavaProject jproject;
	private Composite valueContainer;
	private Composite rootComposite;
	private Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(ExtensionInputDialog.class.getName());
	private Extension curExt;

	private JDOExtension task;  

	public ExtensionInputDialog(
		Shell parent,
		JDOExtension task,
		IJavaProject project,
		int lev) //if we give an Extension that means we are in editing mode otherwise we are in add mode
	{
		super(parent);
		this.task = task;
		this.level = lev;
		this.jproject = project;
		try {
		
			if (task == null) {
				setTitle("New "+ProjectProperties.getProjectProperty(project.getProject(), "Driver")+" extension");
			}
			else {
				setTitle("Edit "+ProjectProperties.getProjectProperty(project.getProject(), "Driver") +" extension"); 
			}
	
			//this is the listener
			CompilerExtensionInputAdapter adapter =	new CompilerExtensionInputAdapter();
	
			fKeyDialogField = new ComboDialogField(2|SWT.READ_ONLY);
			fKeyDialogField.setLabelText("Key :");
			fKeyDialogField.setDialogFieldListener(adapter);
			fKeyDialogField.setText((task != null) ? task.getKey() : "");
			
			VendorExtensions ves = EclipseSpeedoPlugin.getDefault().getJDOExtensionsManager().getDriverExtensions(ProjectProperties.getProjectProperty(project.getProject(), "Driver"));
			if (ves !=null){
			
				ArrayList list = new ArrayList();
				for (Iterator ite = ves.getExtensions().iterator(); ite.hasNext();){
					Extension cur = (Extension) ite.next();
					if (cur.getLevel() == this.level) {
						list.add(cur.getKey());
					}
					else if (this.level == JDOExtensionsConfiguration.COLLECTION && cur.getLevel()==JDOExtensionsConfiguration.FIELD){
						list.add(cur.getKey());
					}
					else if (this.level == JDOExtensionsConfiguration.MAP && cur.getLevel()==JDOExtensionsConfiguration.FIELD){
						list.add(cur.getKey());
					}
				}
				String[] stab = new String[list.size()];
				int i = 0;
				for (Iterator ite = list.iterator();ite.hasNext();){
					stab[i++] = (String) ite.next();
				}
				fKeyDialogField.setItems(stab);
			}
			else {
				fKeyDialogField.setItems(new String[] {"No extension available for "+ProjectProperties.getProjectProperty(project.getProject(), "Driver")});
				fKeyDialogField.selectItem(0);
			}
		}
		catch (Exception e){
			e.printStackTrace();
		}
	}

	public void createValueContainer(int numcol){
		this.valueContainer = new Composite(this.rootComposite, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = numcol;
		layout.makeColumnsEqualWidth = false;
		this.valueContainer.setLayout(layout);
		GridData gridData =	new GridData(GridData.FILL_BOTH | GridData.GRAB_VERTICAL);
		this.valueContainer.setLayoutData(gridData);
	}

	public JDOExtension getResult() {
		try  {
			VendorExtensions ves = EclipseSpeedoPlugin.getDefault().getJDOExtensionsManager().getDriverExtensions(ProjectProperties.getProjectProperty(jproject.getProject(), "Driver"));
			if (ves !=null){
				this.task.setVendorName(this.curExt.getVendor());
				this.task.setKey(fKeyDialogField.getText().trim());
				this.task.getExtensions().clear();
				return this.task;
			}
			else return null;
				
		}
		catch (Exception e){
			e.printStackTrace();
			return null;
		}
	}

	protected Control createDialogArea(Composite parent) {
		try{
			String d = "";
			Composite composite = (Composite) super.createDialogArea(parent);
			Composite inner = new Composite(composite, SWT.NONE);
			GridLayout layout = new GridLayout();
			layout.marginHeight = 10;
			layout.marginWidth = 10;
			layout.numColumns = 2;
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL |GridData.GRAB_VERTICAL );
			gridData.widthHint = 600;
			gridData.heightHint = 110;
			inner.setLayout(layout);
			inner.setLayoutData(gridData);
		
			fKeyDialogField.doFillIntoGrid(inner, 2);
			this.rootComposite = inner;
			fKeyDialogField.getComboControl(null).addSelectionListener(this);
			Label l = new Label(inner,SWT.NONE);
			gridData = new GridData(GridData.FILL_HORIZONTAL |GridData.GRAB_VERTICAL );
			gridData.widthHint = 50;
			l.setText("Value :");
			l.setLayoutData(gridData);
			if (this.task == null) {
				this.task = MetadataFactory.eINSTANCE.createJDOExtension();
				createValueContainer(1);
			}
			else {
				VendorExtensions ves = EclipseSpeedoPlugin.getDefault().getJDOExtensionsManager().getDriverExtensions(ProjectProperties.getProjectProperty(this.jproject.getProject(), "Driver"));
				Extension e = ves.getExtensionNamed(this.task.getKey());
				this.curExt = e;
				d = e.getDesc();
				switch (e.getType()){
					case Extension.STRING : createValueContainer(1);
											this.Value = new Text(this.valueContainer, SWT.BORDER);
											((Text)this.Value).setText(this.task.getValue());
											break;	
					case Extension.COMBO_EDITABLE : createValueContainer(1);
													this.Value = new Combo(this.valueContainer, SWT.BORDER);
													((Combo)this.Value).setText(this.task.getValue());
													break;	
					case Extension.COMBO_UNEDITABLE : createValueContainer(1);
													  this.Value = new Combo(this.valueContainer, SWT.BORDER);
													  ((Combo)this.Value).setText(this.task.getValue());
													  break;	
					case Extension.CLASSBROWSER : createValueContainer(2);
												  this.Value = new Text(this.valueContainer, SWT.BORDER|SWT.READ_ONLY);
												  Button browse = new Button(this.valueContainer, SWT.NONE);
												  browse.setText("Browse...");
												  ((Text)this.Value).setText(this.task.getValue());
												  browse.addSelectionListener(new MySelectionListener((Text) this.Value));
												  break;																	
				}
			}
			
			Group DescView = new Group(createContainer(parent), SWT.NONE | SWT.CENTER);
			DescView.setText("Description");
			GridLayout lay = new GridLayout();
			layout.numColumns = 1;

			layout.makeColumnsEqualWidth = false;
			DescView.setLayout(lay);
			GridData gridD = new GridData(GridData.FILL_HORIZONTAL);
			DescView.setLayoutData(gridD);

			gridData = new GridData(GridData.FILL_HORIZONTAL |GridData.GRAB_HORIZONTAL );
			gridData.widthHint = 610;
			gridData.heightHint = 70;
			this.descText = new Text(DescView, SWT.BORDER | SWT.READ_ONLY |SWT.WRAP );
			this.descText.setLayoutData(gridData);
			this.descText.setText(d);
			applyDialogFont(composite);
			return composite;
		}
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}



	private void doValidation() {

	}
	
	public void okPressed(){
		VendorExtensions ves;
		try {
			ves = EclipseSpeedoPlugin.getDefault().getJDOExtensionsManager().getDriverExtensions(ProjectProperties.getProjectProperty(jproject.getProject(), "Driver"));
			if (ves !=null){
				switch (this.curExt.getType()){
					case Extension.STRING : 
					case Extension.CLASSBROWSER :	this.task.setValue(((Text)this.Value).getText());
													break;	
					case Extension.COMBO_EDITABLE : 
					case Extension.COMBO_UNEDITABLE : this.task.setValue(((Combo)this.Value).getText());
													  break;	
				}
			}
			super.okPressed();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private void doValidation(DialogField diag) {


	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		try {
			//System.out.println("aaaaaaaa "+fKeyDialogField.getText()+ " "+ fVendorNameDialogField.getText());
			VendorExtensions ves = EclipseSpeedoPlugin.getDefault().getJDOExtensionsManager().getDriverExtensions(ProjectProperties.getProjectProperty(this.jproject.getProject(), "Driver"));
			if (ves == null) return;
			Extension ext = ves.getExtensionNamed(fKeyDialogField.getText());
			this.curExt = ext;
			
			this.valueContainer.dispose();
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_VERTICAL);
			gridData.widthHint=300;
			switch (ext.getType()){
				case Extension.STRING : createValueContainer(1);
										this.Value = new Text(this.valueContainer, SWT.BORDER);

										break;	
				case Extension.COMBO_EDITABLE : createValueContainer(1);
												this.Value = new Combo(this.valueContainer, SWT.BORDER);
												StringTokenizer st = new StringTokenizer(this.curExt.getDefaultValue(),",");
												while (st.hasMoreTokens()) {
													((Combo)this.Value).add((String) st.nextToken());
												}
						 						break;	
				case Extension.COMBO_UNEDITABLE : createValueContainer(1);
												  this.Value = new Combo(this.valueContainer, SWT.BORDER|SWT.READ_ONLY);
												  st = new StringTokenizer(this.curExt.getDefaultValue(),",");
												  while (st.hasMoreTokens()) {
													((Combo)this.Value).add((String) st.nextToken());
												  }
												  break;	
				case Extension.CLASSBROWSER : createValueContainer(2);
											  this.Value = new Text(this.valueContainer, SWT.BORDER|SWT.READ_ONLY);
											  Button browse = new Button(this.valueContainer, SWT.NONE);
											  browse.setText("Browse...");
											  browse.addSelectionListener(new MySelectionListener((Text) this.Value));
											  break;																	
			}
			this.descText.setText(this.curExt.getDesc());
			this.Value.setLayoutData(gridData);
			this.rootComposite.layout(true);
			this.valueContainer.setRedraw(true);
			this.rootComposite.redraw();
			this.valueContainer.redraw();
			this.rootComposite.update();


			//System.out.println("Key: "+ext.getKey());
			//System.out.println("bbbbbbb "+ ext.getKey()+" "+ext.getLevel()+" "+ext.getDefaultValue()+" "+ext.getType());
		}
		catch (Exception e1){
			e1.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent e) {
		widgetSelected(e);
	}


	protected String selectClassInProject(){
		Shell shell = EclipseSpeedoPlugin.getShell();
		try	{
			SelectionDialog dialog =
				JavaUI.createTypeDialog(
					shell,
					new ProgressMonitorDialog(shell),
					SearchEngine.createJavaSearchScope(
						new IJavaElement[] { this.jproject }),
					IJavaElementSearchConstants.CONSIDER_TYPES,
					false);
			dialog.setTitle(EclipseSpeedoPlugin.getResourceString("MetadataComposer_typeSelection_title"));
			dialog.setMessage(EclipseSpeedoPlugin.getResourceString("MetadataComposer_typeSelection_message"));
			if (dialog.open() == IDialogConstants.CANCEL_ID)	{
				return null;
			}
	
			Object[] types = dialog.getResult();
			if (types != null && types.length > 0)	{
				IType type = (IType) types[0];
				return type.getFullyQualifiedName('.');
			}
		}
		catch (JavaModelException e){
			e.printStackTrace();
		}
		return null;
	}
	
	public class MySelectionListener implements SelectionListener{

		Text toBeFilled;
		
		public MySelectionListener(Text t){
			toBeFilled = t;
		}

		protected void handleSelected()	{
			String className = selectClassInProject();
			toBeFilled.setText(className);
		}
										
		public void widgetSelected(SelectionEvent e){
			handleSelected();
		}
												
		public void widgetDefaultSelected(SelectionEvent e)	{
			handleSelected();
		}
	}
	
	protected Composite createContainer(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.makeColumnsEqualWidth = false;
		container.setLayout(layout);
		GridData gridData =
			new GridData(
				GridData.FILL_BOTH
					| GridData.GRAB_VERTICAL
					| GridData.GRAB_HORIZONTAL);
		return container;
	}

}