package org.objectweb.eclipsespeedo.ui.metadataeditor;

import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.SelectionButtonDialogFieldGroup;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.metadata.JDOArray;
import org.objectweb.eclipsespeedo.metadata.JDOBoolean;
import org.objectweb.eclipsespeedo.metadata.JDOCollection;
import org.objectweb.eclipsespeedo.metadata.JDOField;
import org.objectweb.eclipsespeedo.metadata.JDOMap;
import org.objectweb.eclipsespeedo.metadata.JDONullValue;
import org.objectweb.eclipsespeedo.metadata.JDOPersistenceModifier;
import org.objectweb.eclipsespeedo.metadata.MetadataModelModifier;

import java.util.Vector;

/**
 * JDO Field editor.
 * 
 */
public class FieldEditor extends AbstractJDOEditor
{
	protected Button fieldCheckBox;
	protected Composite fieldAttributesContainer;
	protected Combo fieldPersistentModifier;
	protected Button fieldPrimaryKey;
	protected Combo fieldNullValue;
	protected Combo fieldDefaultFetchGroup;
	protected Combo fieldEmbedded;
	protected boolean focus;
	protected boolean disposed;

	protected Control fieldExtensionControl;
	protected ExtensionConfigurationBlock fieldExtensionBlock;

	protected Label fieldLabel;

	//Field Child element
	protected Composite fieldChildContainer;
	protected Composite collectionContainer;
	protected Composite mapContainer;
	protected Composite arrayContainer;
	//	protected Combo comboFieldChildChoose;
	SelectionButtonDialogFieldGroup radioFieldChildChoose;
	protected Group fieldTypeGroup;

	//collection
	protected Text textCollectionElementType;
	protected Combo comboCollectionEmbeddedElement;

	//map
	protected Text textMapKeyType;
	protected Combo comboMapEmbeddedKey;
	protected Text textMapValueType;
	protected Combo comboMapEmbeddedValue;

	//array
	protected Combo comboArrayEmbeddedElement;

	public FieldEditor(MetadataModelModifier modifier, IJavaProject project) {
		super(modifier, project);
	}

	/**
	 * Initialize the field view 
	 *
	 */
	public Composite initializeFieldContainer(Composite parent)
	{
		container = new Composite(parent, SWT.NONE);

		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = 5;
		container.setLayoutData(gridData);

		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = 12;
		layout.verticalSpacing = 12;
		layout.numColumns = 2;
		container.setLayout(layout);

		Label fieldNameLabel = new Label(container, SWT.NONE);
		fieldNameLabel.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_fieldName_label"));
		fieldNameLabel.setFont(
			new Font(
				container.getDisplay(),
				fieldNameLabel.getFont().getFontData()[0].getName(),
				8,
				SWT.BOLD));
		gridData =
			new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		fieldNameLabel.setLayoutData(gridData);

		fieldLabel = new Label(container, SWT.NONE);
		fieldLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Label fieldPersistenceLabel = new Label(container, SWT.NONE);
		fieldPersistenceLabel.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_Persistence_label"));
		fieldPersistenceLabel.setFont(
			new Font(
				container.getDisplay(),
				fieldPersistenceLabel.getFont().getFontData()[0].getName(),
				8,
				SWT.BOLD));
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		fieldPersistenceLabel.setLayoutData(gridData);

		fieldCheckBox = new Button(container, SWT.CHECK);
		fieldCheckBox.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{
				fieldPersistenceChanged();
			}
		});
		fieldCheckBox.setSelection(false);

		fieldAttributesContainer = new Composite(container, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		fieldAttributesContainer.setLayoutData(gridData);
		layout = new GridLayout();
		layout.horizontalSpacing = 7;
		layout.verticalSpacing = 12;
		layout.numColumns = 2;
		gridData.horizontalSpan = 2;
		fieldAttributesContainer.setLayout(layout);

		Label label = new Label(fieldAttributesContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_PersistenceModifier_label"));

		fieldPersistentModifier =
			new Combo(fieldAttributesContainer, SWT.BORDER | SWT.READ_ONLY);
		fieldPersistentModifier.setItems(JDOPERSISTENCE_MODIFIER8STRING);

		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		fieldPersistentModifier.setLayoutData(gridData);
		fieldPersistentModifier.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (containerInitialize)
				{
					return;
				}
				int index = fieldPersistentModifier.getSelectionIndex();
				if (index < 0)
				{
					index = 0;
				}
				modifier.getJdoField().setPersistenceModifier(
					JDOPersistenceModifier.get(index));
				container.update();
			}
		});

		label = new Label(fieldAttributesContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_PrimaryKey_label"));

		fieldPrimaryKey = new Button(fieldAttributesContainer, SWT.CHECK);
		fieldPrimaryKey.setSelection(false);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		fieldPrimaryKey.setLayoutData(gridData);
		fieldPrimaryKey.addSelectionListener(new SelectionListener()
		{
			public void widgetSelected(SelectionEvent e)
			{
				modifier.getJdoField().setPrimaryKey(
					fieldPrimaryKey.getSelection());
				container.update();
			}
			public void widgetDefaultSelected(SelectionEvent e)
			{
				modifier.getJdoField().setPrimaryKey(
					fieldPrimaryKey.getSelection());
				container.update();
			}
		});

		label = new Label(fieldAttributesContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_NullValue_label"));

		fieldNullValue =
			new Combo(fieldAttributesContainer, SWT.BORDER | SWT.READ_ONLY);
		fieldNullValue.setItems(JDONULLVALUE_STRINGS);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		fieldNullValue.setLayoutData(gridData);
		fieldNullValue.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				int index = fieldNullValue.getSelectionIndex();
				if (index < 0)
				{
					index = 0;
				}
				modifier.getJdoField().setNullValue(JDONullValue.get(index));

				container.update();
			}
		});
		label = new Label(fieldAttributesContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_DefaultFetchGroup_label"));

		fieldDefaultFetchGroup =
			new Combo(fieldAttributesContainer, SWT.BORDER | SWT.READ_ONLY);
		fieldDefaultFetchGroup.setItems(JDOBOOLEAN_STRINGS);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		fieldDefaultFetchGroup.setLayoutData(gridData);
		fieldDefaultFetchGroup.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (containerInitialize)
				{
					return;
				}
				int index = fieldDefaultFetchGroup.getSelectionIndex();
				if (index < 0)
				{
					index = 0;
				}
				modifier.getJdoField().setDefaultFetchGroup(
					JDOBoolean.get(index));
				container.update();
			}
		});

		label = new Label(fieldAttributesContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_Embedded_label"));

		fieldEmbedded =
			new Combo(fieldAttributesContainer, SWT.BORDER | SWT.READ_ONLY);
		fieldEmbedded.setItems(JDOBOOLEAN_STRINGS);
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		fieldEmbedded.setLayoutData(gridData);
		fieldEmbedded.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (containerInitialize)
				{
					return;
				}
				int index = fieldEmbedded.getSelectionIndex();
				if (index < 0)
				{
					index = 0;
				}
				modifier.getJdoField().setEmbedded(JDOBoolean.get(index));
				container.update();
			}
		});

		this.fieldExtensionBlock = new ExtensionConfigurationBlock(null, currentJavaProject, modifier.getLevel());

		fieldExtensionControl =
			this.fieldExtensionBlock.createContents(container);

		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		fieldExtensionControl.setLayoutData(gridData);

		fieldExtensionBlock
			.getExtensionList()
			.getTableViewer()
			.getTable()
			.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{
				Vector ext = fieldExtensionBlock.getExtension();
				modifier.setFieldExtension(ext);
				container.update();
			}
		});

		/*
		 * 
		 */
		//		label = new Label(fieldAttributesContainer, SWT.NONE);
		fieldTypeGroup = new Group(fieldAttributesContainer, SWT.NONE);
		fieldTypeGroup.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_FieldType_label"));
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.horizontalSpan = 2;
		fieldTypeGroup.setLayoutData(gridData);
		fieldTypeGroup.setLayout(new GridLayout());

		radioFieldChildChoose =
			new SelectionButtonDialogFieldGroup(
				SWT.RADIO,
				JDOLIST_STRINGS_LABEL,
				4);

		radioFieldChildChoose.doFillIntoGrid(fieldTypeGroup, 2);
		//		LayoutUtil.setHorizontalGrabbing(radioFieldChildChoose.getSelectionButtonsGroup(null));

		//		attributeVisibilityField.setLabelText(
		//			UMLResources.getString(
		//				Constants.AttributeProperties_visibility_label));

		//		comboFieldChildChoose =
		//			new Combo(fieldTypeGroup, SWT.BORDER | SWT.READ_ONLY);
		//		comboFieldChildChoose.setItems(JDOLIST_STRINGS);
		//		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		//		gridData.horizontalAlignment = GridData.FILL;
		//		comboFieldChildChoose.setLayoutData(gridData);
		//		comboFieldChildChoose.addModifyListener(new ModifyListener()
		//		{
		//			public void modifyText(ModifyEvent e)
		//			{
		//				fieldListChanged();
		//			}
		//		});

		for (int i = 0; i < JDOLIST_STRINGS_LABEL.length; i++)
		{
			Button button = radioFieldChildChoose.getSelectionButton(i);
			button.addSelectionListener(new SelectionListener()
			{
				public void widgetSelected(SelectionEvent e)
				{
					Button source = (Button) e.widget;
					fieldListChanged(getSelectedFieldType(source));
				}
				public void widgetDefaultSelected(SelectionEvent e)
				{
					Button source = (Button) e.widget;
					fieldListChanged(getSelectedFieldType(source));
				}
			});
		}

		//			{
		//				public void modifyText(ModifyEvent e)
		//				{
		//					fieldListChanged();
		//				}
		//			});

		initializeFieldChildContainer();

		fieldChildContainer.layout(true);

		return container;
	}

	protected int getSelectedFieldType(Button eventButton)
	{
		String text = eventButton.getText();
		for (int i = 0; i < JDOLIST_STRINGS_LABEL.length; i++)
		{
			if (text.equals(JDOLIST_STRINGS_LABEL[i]))
			{
				return i;
			}
		}
		return FIELD_TYPE_NONE;
	}
	/**
	 * When a field is selected, we change the current view and put the field container
	 * and we get values of this field
	 *
	 */
	public void updateField()
	{
		boolean persistence = modifier.isFieldPersistence();
		fieldCheckBox.setSelection(persistence);
		fieldLabel.setText(modifier.getJavaFieldName());
		fieldLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		fieldLabel.pack();

		if (persistence)
		{
			JDOField jdoField = modifier.getJdoField();
			int index =
				fieldPersistentModifier.indexOf(
					jdoField.getPersistenceModifier().getName());

			fieldPersistentModifier.select(index);
			if (index == -1)
				fieldPersistentModifier.setText("");

			fieldPrimaryKey.setSelection(jdoField.isPrimaryKey());

			fieldNullValue.select(jdoField.getNullValue().getValue());

			fieldDefaultFetchGroup.select(
				jdoField.getDefaultFetchGroup().getValue());
			fieldEmbedded.select(jdoField.getEmbedded().getValue());
			fieldExtensionBlock.setExtension(modifier.getFieldExtension());

			//Manage Collection / Map / Array
			JDOCollection collection = jdoField.getCollection();
			JDOMap map = jdoField.getMap();
			JDOArray array = jdoField.getArray();
			if (collection != null)
			{
				radioFieldChildChoose.setSelection(1, true);

				this.fieldListContainerToDisplay(FIELD_TYPE_COLLECTION);

				this.textCollectionElementType.setText(
					collection.getElementType());
				this.comboCollectionEmbeddedElement.select(
					collection.getEmbeddedElemnt().getValue());
			}
			else if (map != null)
			{
				radioFieldChildChoose.setSelection(2, true);
				this.fieldListContainerToDisplay(FIELD_TYPE_MAP);

				this.textMapKeyType.setText(map.getKeyType());
				this.comboMapEmbeddedKey.select(
					map.getEmbeddedKey().getValue());

				this.textMapValueType.setText(map.getValueType());
				this.comboMapEmbeddedValue.select(
					map.getEmbeddedValue().getValue());

			}
			else if (array != null)
			{
				radioFieldChildChoose.setSelection(3, true);
				this.fieldListContainerToDisplay(FIELD_TYPE_ARRAY);

				this.comboArrayEmbeddedElement.select(
					array.getEmbeddedElement().getValue());
			}
			else
			{
				resetFieldChildContainer();
			}
		}
		else
		{
			containerInitialize = true;
			fieldPersistentModifier.setText("");
			fieldPrimaryKey.setText("");
			fieldNullValue.setText("");
			fieldDefaultFetchGroup.setText("");
			fieldEmbedded.setText("");

			fieldExtensionBlock.removeAllElements();

			resetFieldChildContainer();
			containerInitialize = false;
		}

		//if persistence == false then all this attribute's fields are disabled.
		setEnabledFieldAttributeContainer(persistence);

		if (!persistence)
		{
			modifier.cleanField(); // cf packageSeclected
		}
	}

	/**
	 * Define what to do when the check box that represent the field persistence is changed 
	 * 
	 *
	 */
	public void fieldPersistenceChanged()
	{
		try
		{
			boolean check = fieldCheckBox.getSelection();
			modifier.setFieldPersistence(check);
			boolean persistence = modifier.isFieldPersistence();
			setEnabledFieldAttributeContainer(persistence);
			updateField();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void fieldListChanged(int listType)
	{
		if (!radioFieldChildChoose.isSelected(listType))
		{
			return;
		}

		if (fieldCheckBox.getSelection() && (modifier.isFieldPersistence()))
		{
			try
			{
				if (listType == FIELD_TYPE_NONE
					&& modifier.getJdoField().getCollection() == null
					&& modifier.getJdoField().getArray() == null
					&& modifier.getJdoField().getMap() == null)
				{
					return;
				}
				fieldListContainerToDisplay(listType);

				modifier.changeFieldList(JDOLIST_STRINGS[listType]);

				updateField();
				container.getParent().layout();
			}
			catch (Exception ee)
			{
				ee.printStackTrace();
			}
		}
	}

	private void resetFieldChildContainer()
	{
		containerInitialize = true;

		//we reset the container of map / array/collection
		//		comboFieldChildChoose.select(0);
		radioFieldChildChoose.setSelection(0, true);
		//		comboFieldChildChoose.notifyListeners(SWT.Modify, new Event());

		//array

		if (this.collectionContainer != null)
		{
			this.textCollectionElementType.setText("");
			this.comboCollectionEmbeddedElement.setText("");
			this.comboCollectionEmbeddedElement.select(0);
		}

		if (this.mapContainer != null)
		{
			this.textMapKeyType.setText("");
			this.comboMapEmbeddedKey.select(0);
			this.textMapValueType.setText("");
			this.comboMapEmbeddedValue.select(0);
		}

		if (this.arrayContainer != null)
		{
			this.comboArrayEmbeddedElement.select(0);
		}

		fieldChildContainer.layout(true);
		containerInitialize = false;
	}

	private void setEnabledFieldAttributeContainer(boolean bool)
	{
		Control allChildrencontrol[] = fieldAttributesContainer.getChildren();
		for (int i = 0; i < allChildrencontrol.length; i++)
		{
			allChildrencontrol[i].setEnabled(bool);
		}
		if (fieldExtensionBlock == null)
			return;
		fieldExtensionBlock.setEnabled(bool);
		fieldExtensionBlock.getExtensionList().setEnabled(bool);
		fieldExtensionBlock
			.getExtensionList()
			.getTableViewer()
			.getTable()
			.setEnabled(
			bool);
	}

	private void initializeFieldChildContainer()
	{
		fieldChildContainer = new Composite(fieldTypeGroup, SWT.NONE);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 2;
		fieldChildContainer.setLayoutData(data);

		fieldChildContainer.setLayout(new GridLayout());
	}

	private void initializeCollectionContainer()
	{
		collectionContainer = new Composite(fieldChildContainer, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		collectionContainer.setLayout(layout);

		collectionContainer.setLayoutData(new GridData(GridData.FILL_BOTH));

		Label label = new Label(collectionContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_ElementType_label"));

		this.textCollectionElementType =
			new Text(collectionContainer, SWT.BORDER);
		GridData gridData =
			new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		gridData.horizontalSpan = 1;
		this.textCollectionElementType.setLayoutData(gridData); //
		this.textCollectionElementType.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (fieldCheckBox.getSelection()
					&& (modifier.isFieldPersistence()))
				{
					JDOCollection collection =
						modifier.getJdoField().getCollection();
					if (collection != null)
					{
						collection.setElementType(
							textCollectionElementType.getText());
					}
				}
			}
		});

		{
			Button button = new Button(collectionContainer, SWT.NONE);
			button.setText(
				EclipseSpeedoPlugin.getResourceString(
					"MetadataComposer_Browse_label"));
			gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
			gridData.horizontalSpan = 1;
			button.setLayoutData(gridData); //

			button.addSelectionListener(new SelectionListener()
			{
				protected void handleSelected()
				{
					String className = selectClassInProject();
					if (className != null)
					{
						modifier.getJdoField().getCollection().setElementType(
							className);
						textCollectionElementType.setText(className);
					}
				}

				public void widgetSelected(SelectionEvent e)
				{
					handleSelected();
				}

				public void widgetDefaultSelected(SelectionEvent e)
				{
					handleSelected();
				}
			});
		}

		label = new Label(collectionContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_EmbeddedElement_label"));

		this.comboCollectionEmbeddedElement =
			new Combo(collectionContainer, SWT.BORDER | SWT.READ_ONLY);
		this.comboCollectionEmbeddedElement.setItems(JDOBOOLEAN_STRINGS);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		//		gridData.horizontalSpan = 2;
		this.comboCollectionEmbeddedElement.setLayoutData(gridData);
		this
			.comboCollectionEmbeddedElement
			.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (fieldCheckBox.getSelection()
					&& (modifier.isFieldPersistence()))
				{
					JDOCollection collection =
						modifier.getJdoField().getCollection();
					if (collection != null)
					{
						int index =
							comboCollectionEmbeddedElement.getSelectionIndex();
						if (index == -1)
						{
							index = 0;
						}
						collection.setEmbeddedElemnt(JDOBoolean.get(index));
					}
				}
			}
		});

		//return collectionContainer;
	}

	private void initializeMapContainer()
	{
		mapContainer = new Composite(fieldChildContainer, SWT.NONE);

		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = 7;
		layout.verticalSpacing = 12;
		layout.numColumns = 3;
		mapContainer.setLayout(layout);

		mapContainer.setLayoutData(new GridData(GridData.FILL_BOTH));

		Label label = new Label(mapContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_KeyType_label"));

		this.textMapKeyType = new Text(mapContainer, SWT.BORDER);
		GridData gridData = new GridData(GridData.FILL_BOTH);
		textMapKeyType.setLayoutData(gridData); //
		textMapKeyType.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (fieldCheckBox.getSelection()
					&& (modifier.isFieldPersistence()))
				{
					JDOMap map = modifier.getJdoField().getMap();
					if (map != null)
					{
						map.setKeyType(textMapKeyType.getText());
					}
				}
			}
		});

		Button button = new Button(mapContainer, SWT.NONE);
		button.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_Browse_label"));
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		button.setLayoutData(gridData);

		button.addSelectionListener(new SelectionListener()
		{
			protected void handleSelected()
			{
				String className = selectClassInProject();
				if (className != null)
				{
					modifier.getJdoField().getMap().setKeyType(className);
					textMapKeyType.setText(className);
				}
			}

			public void widgetSelected(SelectionEvent e)
			{
				handleSelected();
			}

			public void widgetDefaultSelected(SelectionEvent e)
			{
				handleSelected();
			}
		});

		label = new Label(mapContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_EmbeddedKey_label"));

		this.comboMapEmbeddedKey =
			new Combo(mapContainer, SWT.BORDER | SWT.READ_ONLY);
		this.comboMapEmbeddedKey.setItems(JDOBOOLEAN_STRINGS);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		//		gridData.horizontalSpan = 2;
		this.comboMapEmbeddedKey.setLayoutData(gridData);
		this.comboMapEmbeddedKey.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (fieldCheckBox.getSelection()
					&& (modifier.isFieldPersistence()))
				{
					JDOMap map = modifier.getJdoField().getMap();
					if (map != null)
					{
						int index = comboMapEmbeddedKey.getSelectionIndex();
						if (index == -1)
						{
							index = 0;
						}
						map.setEmbeddedKey(JDOBoolean.get(index));
					}
				}
			}
		});

		new Label(mapContainer, SWT.NONE);

		label = new Label(mapContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_ValueType_label"));
		gridData = new GridData(GridData.FILL_HORIZONTAL);

		this.textMapValueType = new Text(mapContainer, SWT.BORDER);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		//		gridData.horizontalAlignment = GridData.FILL;
		textMapValueType.setLayoutData(gridData); //
		textMapValueType.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (fieldCheckBox.getSelection()
					&& (modifier.isFieldPersistence()))
				{
					JDOMap map = modifier.getJdoField().getMap();
					if (map != null)
					{
						map.setValueType(textMapValueType.getText());
					}
				}
			}
		});

		button = new Button(mapContainer, SWT.NONE);
		button.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_Browse_label"));
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		button.setLayoutData(gridData);

		button.addSelectionListener(new SelectionListener()
		{
			protected void handleSelected()
			{
				String className = selectClassInProject();
				if (className != null)
				{
					modifier.getJdoField().getMap().setValueType(className);
					textMapValueType.setText(className);
				}
			}

			public void widgetSelected(SelectionEvent e)
			{
				handleSelected();
			}

			public void widgetDefaultSelected(SelectionEvent e)
			{
				handleSelected();
			}
		});

		label = new Label(mapContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_EmbeddedValue_label"));

		this.comboMapEmbeddedValue =
			new Combo(mapContainer, SWT.BORDER | SWT.READ_ONLY);
		comboMapEmbeddedValue.setItems(JDOBOOLEAN_STRINGS);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		//		gridData.horizontalAlignment = GridData.FILL;
		//		gridData.horizontalSpan = 2;
		comboMapEmbeddedValue.setLayoutData(gridData);
		comboMapEmbeddedValue.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (fieldCheckBox.getSelection()
					&& (modifier.isFieldPersistence()))
				{
					JDOMap map = modifier.getJdoField().getMap();
					if (map != null)
					{
						int index = comboMapEmbeddedValue.getSelectionIndex();
						if (index == -1)
						{
							index = 0;
						}
						map.setEmbeddedValue(JDOBoolean.get(index));
					}
				}
			}
		});
	}

	private void initializeArrayContainer()
	{
		arrayContainer = new Composite(fieldChildContainer, SWT.NONE);

		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = 7;
		layout.verticalSpacing = 12;
		layout.numColumns = 2;
		arrayContainer.setLayout(layout);

		arrayContainer.setLayoutData(new GridData(GridData.FILL_BOTH));

		Label label = new Label(arrayContainer, SWT.NONE);
		label.setText(
			EclipseSpeedoPlugin.getResourceString(
				"MetadataComposer_EmbeddedElement_label"));

		this.comboArrayEmbeddedElement =
			new Combo(arrayContainer, SWT.BORDER | SWT.READ_ONLY);
		comboArrayEmbeddedElement.setItems(JDOBOOLEAN_STRINGS);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		comboArrayEmbeddedElement.setLayoutData(gridData);
		comboArrayEmbeddedElement.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				if (fieldCheckBox.getSelection()
					&& (modifier.isFieldPersistence()))
				{
					JDOArray array = modifier.getJdoField().getArray();
					if (array != null)
					{
						int index =
							comboArrayEmbeddedElement.getSelectionIndex();
						if (index == -1)
						{
							index = 0;
						}
						array.setEmbeddedElement(JDOBoolean.get(index));
					}
				}
			}
		});

		//return arrayContainer;
	}

	private void fieldListContainerToDisplay(int fieldType)
	{
		//collection selected
		switch (fieldType)
		{
			case FIELD_TYPE_NONE :
				if (this.collectionContainer != null)
				{
					this.collectionContainer.dispose();
					this.collectionContainer = null;
				}

				if (this.mapContainer != null)
				{
					this.mapContainer.dispose();
					this.mapContainer = null;
				}

				if (this.arrayContainer != null)
				{
					this.arrayContainer.dispose();
					this.arrayContainer = null;
				}

				break;
			case FIELD_TYPE_COLLECTION :
				if (this.collectionContainer != null)
				{
					return;
				}
				if (this.mapContainer != null)
				{
					this.mapContainer.dispose();
					this.mapContainer = null;
				}
				if (this.arrayContainer != null)
				{
					this.arrayContainer.dispose();
					this.arrayContainer = null;
				}
				initializeCollectionContainer();
				break;
			case FIELD_TYPE_MAP :
				if (this.mapContainer != null)
				{
					return;
				}

				if (this.collectionContainer != null)
				{
					this.collectionContainer.dispose();
					this.collectionContainer = null;
				}

				if (this.arrayContainer != null)
				{
					this.arrayContainer.dispose();
					this.arrayContainer = null;
				}
				initializeMapContainer();
				break;
			case FIELD_TYPE_ARRAY :
				if (this.arrayContainer != null)
				{
					return;
				}

				if (this.collectionContainer != null)
				{
					this.collectionContainer.dispose();
					this.collectionContainer = null;
				}

				if (this.mapContainer != null)
				{
					this.mapContainer.dispose();
					this.mapContainer = null;
				}
				initializeArrayContainer();
				break;
		}
	}

	public void dispose()
	{
		super.dispose();
		if (this.collectionContainer != null)
		{
			this.collectionContainer.dispose();
			this.collectionContainer = null;
		}

		if (this.mapContainer != null)
		{
			this.mapContainer.dispose();
			this.mapContainer = null;
		}

		if (this.arrayContainer != null)
		{
			this.arrayContainer.dispose();
			this.arrayContainer = null;
		}
	}
}
