package org.objectweb.eclipsespeedo.ui.metadataeditor;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.swt.widgets.Composite;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.exceptions.ModificationException;
import org.objectweb.eclipsespeedo.metadata.MetadataModelModifier;

/**
 * Abstract class that manage the composer view. It's the core of the composer
 * Methods corresponding to the Button are specific to the context
 */
public class MetadataSwitch {
	protected MetadataModelModifier modifier = null;

	protected PackageEditor packageEditor;
	protected ClassEditor classEditor;
	protected FieldEditor fieldEditor;

	//elements of the field container
	//Project
	protected IJavaProject currentJavaProject;

	protected Composite rootParent;

	boolean containerInitialize = false;
	public MetadataSwitch(Resource resource)
	{
		modifier = new MetadataModelModifier(resource);
		URI uri = resource.getURI();
		String path = uri.path();
		path = path.substring("/resource".length()); //$NON-NLS-1$
		IWorkspaceRoot workspaceRoot =
			EclipseSpeedoPlugin.getWorkspace().getRoot();
		IResource fileResource = workspaceRoot.findMember(path);
		IProject project = fileResource.getProject();
		currentJavaProject = JavaCore.create(project);

		packageEditor = new PackageEditor(modifier, currentJavaProject);
		classEditor = new ClassEditor(modifier, currentJavaProject);
		fieldEditor = new FieldEditor(modifier, currentJavaProject);
	}

	public Resource getResource()
	{
		if (modifier == null)
		{
			return null;
		}
		return modifier.getResource();
	}

	/**
	 * @return
	 */
	public MetadataModelModifier getMeta()
	{
		return this.modifier;
	}

	/**
	 * @return
	 */
	public IJavaProject getCurrentJavaProject()
	{
		return currentJavaProject;
	}

	/**
	 * @param project
	 */
	public void setCurrentJavaProject(IJavaProject project)
	{
		currentJavaProject = project;
	}

	public boolean packageSelected(IPackageFragment selection)
		throws ModificationException
	{
		getMeta().setJavaPackage(selection);
		this.containerToDisplay("Package");
		packageEditor.updatePackage();
		return true;
	}

	/**
	 * This is what we do, when en an attribute is selected
	 * @param selection
	 */
	public boolean attributeSelected(IField field) throws ModificationException
	{
		// we set the current package
		if (!field.equals(getMeta().getJavaField()))
		{
			getMeta().setJavaField(field);
			this.containerToDisplay("Field");
			fieldEditor.updateField();
			return true;
		}
		return false;
	}

	/**
	 * This is what we do, when en an class is selected
	 * @param selection
	 */
	public boolean classSelected(IType type) throws ModificationException
	{
		//		if the parent selection has no parent then it is a class item
		getMeta().setJavaClass(type);
		this.containerToDisplay("Class");
		classEditor.updateClass();
		return true;
	}

	/**
	 * Display the appropriate container
	 * @param container
	 */
	private void containerToDisplay(String container)
	{
		this.packageEditor.dispose();
		this.classEditor.dispose();
		this.fieldEditor.dispose();

		//package selected
		if (container
			.equalsIgnoreCase("Package"))
		{
			this.packageEditor.initializePackageContainer(rootParent);
		}

		//class selected
		else if (
			container.equalsIgnoreCase("Class"))
		{
			this.classEditor.initializeClassContainer(this.rootParent);
		}

		//field selected
		else if (
			container.equalsIgnoreCase("Field"))
		{
			this.fieldEditor.initializeFieldContainer(this.rootParent);
		}
		this.rootParent.layout();
	}

	/**
	 * @param composite
	 */
	public void setRootParent(Composite composite)
	{
		rootParent = composite;
	}

}
