package org.objectweb.eclipsespeedo.ui.metadataeditor;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map;
import java.util.StringTokenizer;

import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.wizards.IStatusChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;

/**
  */
public abstract class OptionsConfigurationBlock
{

	protected static class ControlData
	{
		private String fKey;
		private String[] fValues;

		public ControlData(String key, String[] values)
		{
			fKey = key;
			fValues = values;
		}

		public String getKey()
		{
			return fKey;
		}

		public String getValue(boolean selection)
		{
			int index = selection ? 0 : 1;
			return fValues[index];
		}

		public String getValue(int index)
		{
			return fValues[index];
		}

		public int getSelection(String value)
		{
			for (int i = 0; i < fValues.length; i++)
			{
				if (value.equals(fValues[i]))
				{
					return i;
				}
			}
			throw new IllegalArgumentException();
		}
	}

	protected Map fWorkingValues;

	protected ArrayList fCheckBoxes;
	protected ArrayList fComboBoxes;
	protected ArrayList fTextBoxes;

	private SelectionListener fSelectionListener;

	protected IStatusChangeListener fContext;
	protected IJavaProject fProject; // project or null

	private Shell fShell;

	/**
	 * 
	 * @param context
	 * @param project
	 */
	public OptionsConfigurationBlock(
		IStatusChangeListener context,
		IJavaProject project)
	{
		fContext = context;
		fProject = project;

		fWorkingValues = getOptions(true);

		fCheckBoxes = new ArrayList();
		fComboBoxes = new ArrayList();
		fTextBoxes = new ArrayList(2);
	}

	/**
	 * 
	 * @param inheritJavaCoreOptions
	 * @return
	 */
	protected Map getOptions(boolean inheritJavaCoreOptions)
	{
		if (fProject != null)
		{
			return fProject.getOptions(inheritJavaCoreOptions);
		}
		else
		{
			return JavaCore.getOptions();
		}
	}

	protected Map getDefaultOptions()
	{
		return JavaCore.getDefaultOptions();
	}

	protected final void setOptions(Map map)
	{
		if (fProject != null)
		{
			fProject.setOptions(map);
		}
		else
		{
			JavaCore.setOptions((Hashtable) map);
		}
	}

	protected Shell getShell()
	{
		return fShell;
	}

	protected void setShell(Shell shell)
	{
		fShell = shell;
	}

	/**
	 * 
	 * @param parent
	 * @return
	 */
	protected abstract Control createContents(Composite parent);

	/**
	 * 
	 * @param parent
	 * @param label
	 * @param key
	 * @param values
	 * @param indent
	 */
	protected void addCheckBox(
		Composite parent,
		String label,
		String key,
		String[] values,
		int indent)
	{
		ControlData data = new ControlData(key, values);

		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gd.horizontalSpan = 2;
		gd.horizontalIndent = indent;

		Button checkBox = new Button(parent, SWT.CHECK);
		checkBox.setText(label);
		checkBox.setData(data);
		checkBox.setLayoutData(gd);
		checkBox.addSelectionListener(getSelectionListener());

		String currValue = (String) fWorkingValues.get(key);
		checkBox.setSelection(data.getSelection(currValue) == 0);

		fCheckBoxes.add(checkBox);
	}

	/**
	 * 
	 * @param parent
	 * @param label
	 * @param key
	 * @param values
	 * @param valueLabels
	 * @param indent
	 */
	protected void addComboBox(
		Composite parent,
		String label,
		String key,
		String[] values,
		String[] valueLabels,
		int indent)
	{
		ControlData data = new ControlData(key, values);

		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		gd.horizontalIndent = indent;

		Label labelControl = new Label(parent, SWT.LEFT | SWT.WRAP);
		labelControl.setText(label);
		labelControl.setLayoutData(gd);

		Combo comboBox = new Combo(parent, SWT.READ_ONLY);
		comboBox.setItems(valueLabels);
		comboBox.setData(data);
		comboBox.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		comboBox.addSelectionListener(getSelectionListener());

		String currValue = (String) fWorkingValues.get(key);
		comboBox.select(data.getSelection(currValue));

		fComboBoxes.add(comboBox);
	}

	/**
	 * 
	 * @param parent
	 * @param label
	 * @param key
	 * @param indent
	 * @param widthHint
	 * @return
	 */
	protected Text addTextField(
		Composite parent,
		String label,
		String key,
		int indent,
		int widthHint)
	{
		Label labelControl = new Label(parent, SWT.NONE);
		labelControl.setText(label);
		labelControl.setLayoutData(new GridData());

		Text textBox = new Text(parent, SWT.BORDER | SWT.SINGLE);
		textBox.setData(key);
		textBox.setLayoutData(new GridData());

		String currValue = (String) fWorkingValues.get(key);
		textBox.setText(currValue);

		GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		if (widthHint != 0)
		{
			data.widthHint = widthHint;
		}
		data.horizontalIndent = indent;
		textBox.setLayoutData(data);

		fTextBoxes.add(textBox);
		return textBox;
	}

	/**
	 * 
	 * @return
	 */
	protected SelectionListener getSelectionListener()
	{
		if (fSelectionListener == null)
		{
			fSelectionListener = new SelectionListener()
			{
				public void widgetDefaultSelected(SelectionEvent e)
				{
				}

				public void widgetSelected(SelectionEvent e)
				{
					controlChanged(e.widget);
				}
			};
		}
		return fSelectionListener;
	}

	/**
	 * 
	 * @param widget
	 */
	protected void controlChanged(Widget widget)
	{
		ControlData data = (ControlData) widget.getData();
		String newValue = null;
		if (widget instanceof Button)
		{
			newValue = data.getValue(((Button) widget).getSelection());
		}
		else if (widget instanceof Combo)
		{
			newValue = data.getValue(((Combo) widget).getSelectionIndex());
		}
		else
		{
			return;
		}
		fWorkingValues.put(data.getKey(), newValue);

	}

	/**
	 * 
	 * @param key
	 * @param value
	 * @return
	 */
	protected boolean checkValue(String key, String value)
	{
		return value.equals(fWorkingValues.get(key));
	}

	/**
	 * 
	 * @param text
	 * @param separator
	 * @return
	 */
	protected String[] getTokens(String text, String separator)
	{
		StringTokenizer tok = new StringTokenizer(text, separator); //$NON-NLS-1$
		int nTokens = tok.countTokens();
		String[] res = new String[nTokens];
		for (int i = 0; i < res.length; i++)
		{
			res[i] = tok.nextToken().trim();
		}
		return res;
	}

	/**
	 * 
	 * @param enabled
	 * @return
	 */
	public boolean performOk(boolean enabled)
	{
		if (enabled);
		return true;
	}

	/**
	 * 
	 *
	 */
	public void performDefaults()
	{
		fWorkingValues = getDefaultOptions();
		updateControls();
	}

	/**
	 * 
	 *
	 */
	protected void updateControls()
	{
		// update the UI
		for (int i = fCheckBoxes.size() - 1; i >= 0; i--)
		{
			Button curr = (Button) fCheckBoxes.get(i);
			ControlData data = (ControlData) curr.getData();

			String currValue = (String) fWorkingValues.get(data.getKey());
			curr.setSelection(data.getSelection(currValue) == 0);
		}
		for (int i = fComboBoxes.size() - 1; i >= 0; i--)
		{
			Combo curr = (Combo) fComboBoxes.get(i);
			ControlData data = (ControlData) curr.getData();

			String currValue = (String) fWorkingValues.get(data.getKey());
			curr.select(data.getSelection(currValue));
		}
		for (int i = fTextBoxes.size() - 1; i >= 0; i--)
		{
			Text curr = (Text) fTextBoxes.get(i);
			String key = (String) curr.getData();

			String currValue = (String) fWorkingValues.get(key);
			curr.setText(currValue);
		}
	}

}
