package org.objectweb.eclipsespeedo.ui.metadataeditor;

import java.util.Vector;

import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.exceptions.ModificationException;
import org.objectweb.eclipsespeedo.jdoextensions.xml.JDOExtensionsConfiguration;
import org.objectweb.eclipsespeedo.metadata.MetadataModelModifier;

/**
 * JDO Package editor.
 *
 */
public class PackageEditor extends AbstractJDOEditor {
	protected Label packageLabel;
	protected ExtensionConfigurationBlock packageExtensionBlock;
	protected Control packageExtensionControl;
	protected Button packageCheckBox;

	public PackageEditor(MetadataModelModifier modifier, IJavaProject project) {
		super(modifier, project);
	}
	/**
	 * Initialize the package view 
	 *
	 */
	public Composite initializePackageContainer(Composite parent)
	{
		container = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.horizontalSpacing = 12;
		layout.verticalSpacing = 12;
		container.setLayout(layout);

		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		container.setLayoutData(gridData);

		Label packageNameLabel = new Label(container, SWT.NONE);
		packageNameLabel.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_packageName_label"));
		packageNameLabel.setFont(
			new Font(
				container.getDisplay(),
				packageNameLabel.getFont().getFontData()[0].getName(),
				8,
				SWT.BOLD));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		packageNameLabel.setLayoutData(gridData);

		packageLabel = new Label(container, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalAlignment = GridData.FILL;
		gridData.widthHint = 100;
		packageLabel.setLayoutData(gridData);

		Label packagePersistenceLabel = new Label(container, SWT.NONE);
		packagePersistenceLabel.setText(EclipseSpeedoPlugin.getResourceString("MetadataComposer_Persistence_label"));
		packagePersistenceLabel.setFont(
			new Font(
				container.getDisplay(),
				packagePersistenceLabel.getFont().getFontData()[0].getName(),
				8,
				SWT.BOLD));
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		gridData.horizontalAlignment = GridData.FILL;
		packagePersistenceLabel.setLayoutData(gridData);

		packageCheckBox = new Button(container, SWT.CHECK);
		packageCheckBox.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{
				handlePackagePersistenceChanged();
			}
		});

		this.packageExtensionBlock =
			new ExtensionConfigurationBlock(null, currentJavaProject, JDOExtensionsConfiguration.PACKAGE);
		packageExtensionControl =
			this.packageExtensionBlock.createContents(container);

		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		packageExtensionControl.setLayoutData(gridData);

		packageCheckBox.setSelection(false);
		setEnabledPackageAttributeContainer(false);

		packageExtensionBlock
			.getExtensionList()
			.getTableViewer()
			.getTable()
			.addListener(SWT.Selection, new Listener()
		{
			public void handleEvent(Event e)
			{
				Vector ext = packageExtensionBlock.getExtension();

				modifier.setPackageExtension(ext);

				container.update();
			}
		});

		return container;
	}

	/**
	 * When the user change the persistence state of a package
	 * If he unset, all information related to his children is remove too. 
	 *
	 */
	public void handlePackagePersistenceChanged() {
		try	{
			boolean check = packageCheckBox.getSelection();
			modifier.setPackagePersistence(check);
			boolean persistence = modifier.isPackagePersistence();
			setEnabledPackageAttributeContainer(persistence);

			updatePackage();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	* When a package is selected, we change the current view and put the package container
	*
	*/
	public void updatePackage() throws ModificationException {
		boolean persistence = modifier.isPackagePersistence();

		packageLabel.setText(modifier.getJavaPackageName());
		packageLabel.pack();
		packageCheckBox.setSelection(persistence);

		if (persistence) {
			Vector ext = modifier.getPackageExtension();
			packageExtensionBlock.removeAllElements();
			packageExtensionBlock.setExtension(ext);
		}
		else{
			containerInitialize = true;
			packageExtensionBlock.removeAllElements();
			modifier.cleanPackage();
			containerInitialize = false;

			// if we don't clean, because of the use of pacakgeExtensionBlock, the current package is add in the current dom !!!
		}

		setEnabledPackageAttributeContainer(persistence);
	}

	private void setEnabledPackageAttributeContainer(boolean bool)
	{
		packageExtensionBlock.setEnabled(bool);
		packageExtensionBlock.getExtensionList().setEnabled(bool);
		packageExtensionBlock
			.getExtensionList()
			.getTableViewer()
			.getTable()
			.setEnabled(
			bool);
		packageExtensionBlock.updateControls();
	}

}
