/*
 * Created on May 5, 2004
 * author: VEZIER Antoine
 * Internship Master: Hamon Catherine
 * 
 */
 
package org.objectweb.eclipsespeedo.ui.preferences;
import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.jdt.internal.ui.JavaPluginImages;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.drivers.xml.ClasspathItem;
import org.objectweb.eclipsespeedo.drivers.xml.DriverDefinition;
import org.objectweb.eclipsespeedo.drivers.xml.DriverDefinitionProperty;
import org.objectweb.eclipsespeedo.utils.Util;



/**
 *
 */
public class EclipseSpeedoPreferencePage
	extends PreferencePage
	implements IWorkbenchPreferencePage, SelectionListener {
		
		
		private static final int DRIVER = 0;
		private static final int PROJECT = 1;
		private final Image fExceptionIcon =
			Util.createImage("icons/error.gif", getClass());
		private final Image fJarIcon =
			Util.createImage("icons/jar_l_obj.gif", getClass());
		Combo fDriverCombo;
		Composite content;
		Composite selectionBar;
		Composite body;
		int selectedDriverIndex;
		ArrayList definitions;
		ArrayList selectedControls;
		Table driverTable;
		Table projectTable;
		
	public void createControl(Composite parent) {
		super.createControl(parent);
	}
	
	protected Composite createContainer(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.makeColumnsEqualWidth = false;
		container.setLayout(layout);
		GridData gridData =
			new GridData(
				GridData.FILL_BOTH
					| GridData.GRAB_VERTICAL
					| GridData.GRAB_HORIZONTAL);
		return container;
	}

	protected Control createContents(Composite parent) {
		EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().refresh();
		definitions = EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().getDefinitions();

		//noDefaultAndApplyButton();
		this.content = createContainer(parent);

		createSelectionBar();
		createBody();

		return content;
	}
	private void createSelectionBar() {
		selectionBar = new Group(content, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.makeColumnsEqualWidth = false;
		selectionBar.setLayout(layout);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		selectionBar.setLayoutData(gridData);
		Label label = new Label(selectionBar, SWT.NONE);
		label.setText("Driver types:");
		gridData = new GridData();
		label.setLayoutData(gridData);
		fDriverCombo = new Combo(selectionBar, SWT.BORDER);
		Iterator defs = definitions.iterator();
		while (defs.hasNext()) {
			DriverDefinition element = (DriverDefinition) defs.next();
			fDriverCombo.add(element.getName());
		}
		gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		fDriverCombo.setLayoutData(gridData);
		fDriverCombo.select(selectedDriverIndex);
		fDriverCombo.addSelectionListener(this);
	}

	private void createBody() {
		content.layout(true);
		body = new Composite(content, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.makeColumnsEqualWidth = false;
		body.setLayout(layout);
		GridData gridData =	new GridData(GridData.FILL_BOTH | GridData.GRAB_VERTICAL);
		body.setLayoutData(gridData);

		TabFolder tabFolder = new TabFolder(body, SWT.NONE);
		tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_VERTICAL));

		Composite propertyTab =
			createTab(
				"Properties",
				"Define properties required for the server configuration",
				tabFolder);

		DriverDefinition element = (DriverDefinition) definitions.get(selectedDriverIndex);
		DriverPanel driverPanel = new DriverPanel();
		driverPanel.createContents(element, element.getName(), propertyTab);

		Composite driverlibraryTab =
			createTab(
				"Driver Libraries",
				"Set the libraries required to launch the server",
				tabFolder);
		LibraryPanel driverlibPanel = new LibraryPanel();
		driverlibPanel.createContents(element, element.getName(), driverlibraryTab, element.getEclipseprefconf().getDriverClassPath(), DRIVER);
		
		Composite projectlibraryTab =
			createTab(
				"Project common Libraries",
				"Set the libraries common librairies required to use the JDO driver in a project",
				tabFolder);
		LibraryPanel projectlibPanel = new LibraryPanel();
		projectlibPanel.createContents(element, element.getName(), projectlibraryTab, element.getEclipseprefconf().getProjClasspath(), PROJECT);
		
		tabFolder.setSelection(0);
		body.layout(true);
		body.setRedraw(true);
		content.redraw();
	}
	
	private Composite createTab(
		String tabDesc,
		String toolTip,
		TabFolder tabFolder) {
		TabItem containerTab = new TabItem(tabFolder, SWT.NONE);
		containerTab.setText(tabDesc);

				
		ScrolledComposite scroll = new ScrolledComposite(tabFolder, SWT.H_SCROLL | SWT.V_SCROLL | SWT.SHADOW_ETCHED_IN);

		scroll.setLayout(new GridLayout(1,true));
		scroll.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_VERTICAL | GridData.GRAB_HORIZONTAL));
		Composite composite = new Composite(scroll,SWT.NONE);
		scroll.setContent(composite);
		
		
		GridLayout gridLayout = new GridLayout(1,false);
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		composite.setLayout(gridLayout);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_VERTICAL | GridData.GRAB_HORIZONTAL));
		containerTab.setControl(scroll);
		containerTab.setToolTipText(toolTip);
		return composite;
	}
	public void init(IWorkbench workbench) {

		
	}

	public boolean performOk() {
		performApply();
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {

		super.performDefaults();
		EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().refresh();
		definitions =
		EclipseSpeedoPlugin
				.getDefault()
				.getDriverDefinitionManager()
				.getDefinitions();
		body.dispose();
		createBody();
		content.layout(true);
		body.setRedraw(true);
		content.redraw();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performApply()
	 */
	public void performApply() {
		int contCount = 0;
		if (selectedDriverIndex >= 0) {
			DriverDefinition def = (DriverDefinition) definitions.get(selectedDriverIndex);
			Iterator properties = def.getEclipsePreferencesProperties().iterator();
			while (properties.hasNext()) {
				DriverDefinitionProperty item = (DriverDefinitionProperty) properties.next();
				Control control = (Control) selectedControls.get(contCount++);
				switch (item.getType()) {
					case DriverDefinitionProperty.BOOLEAN :
						item.setDefaultValue(
							"" + ((Button) control).getSelection());
						break;
					case DriverDefinitionProperty.DIRECTORY :
					case DriverDefinitionProperty.FILE :
						item.setDefaultValue(
							((Text) control).getText().replace('\\', '/'));
						break;
					case DriverDefinitionProperty.STRING :
					default :
						item.setDefaultValue(((Text) control).getText());
						break;
				}
			}
			TableItem[] item = driverTable.getItems();
			ArrayList cp = new ArrayList();
			for (int i = 0; i < item.length; i++) {
				cp.add(item[i].getData());
			}
			def.getEclipseprefconf().setDriverClassPath(cp);
			
			item = projectTable.getItems();
			cp = new ArrayList();
			for (int i = 0; i < item.length; i++) {
				cp.add(item[i].getData());
			}
			def.getEclipseprefconf().setProjClasspath(cp);
			EclipseSpeedoPlugin.getDefault().getDriverDefinitionManager().update(def);
			body.dispose();
			createBody();
			content.layout(true);
			body.setRedraw(true);
			content.redraw();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.IPreferencePage#performCancel()
	 */
	public boolean performCancel() {
		return true;
	}
	
	
	/*
	 * @see SelectionListener#widgetSelected(SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		selectedDriverIndex = ((Combo) e.getSource()).getSelectionIndex();
		body.dispose();
		createBody();
		content.layout(true);
		body.setRedraw(true);
		content.redraw();
		body.redraw();
		content.update();
	}

	public void widgetDefaultSelected(SelectionEvent e) {
		widgetSelected(e);
	}


	class DriverPanel {

		protected Control createContents(
			DriverDefinition element,
			String title,
			Composite parent) {
			return createServerDefinitionPanel(element, parent);
		}

		protected Control createServerDefinitionPanel(
			DriverDefinition element,
			Composite composite) {
			Group defPanel = new Group(composite, SWT.NONE);
			defPanel.setText(element.getName());
			GridLayout layout = new GridLayout(3, false);
			defPanel.setLayout(layout);
			GridData gridData =
				new GridData(
					GridData.FILL_BOTH
						| GridData.GRAB_VERTICAL
						| GridData.GRAB_HORIZONTAL);
			gridData.widthHint = convertWidthInCharsToPixels(100);

			defPanel.setLayoutData(gridData);
			Iterator properties = element.getEclipsePreferencesProperties().iterator();
			selectedControls = new ArrayList();
			while (properties.hasNext()) {
				DriverDefinitionProperty item =
					(DriverDefinitionProperty) properties.next();
				Control control = null;
				switch (item.getType()) {
					case DriverDefinitionProperty.DIRECTORY :
						control = createLabeledPath(item.getLabel(),item.getDefaultValue(), defPanel);
						break;					
					case DriverDefinitionProperty.FILE :
						control =createLabeledFile(item.getLabel(), item.getDefaultValue(), defPanel);
						break;
					case DriverDefinitionProperty.STRING :
						control =createLabeledText(item.getLabel(), item.getDefaultValue(),  item.isModifiable(),defPanel);
						break;
					case DriverDefinitionProperty.BOOLEAN :
  						control = createLabeledCheck(item.getLabel(),	("true".equals(item.getDefaultValue())), defPanel);
						break;
					default :
						control = createLabeledText(item.getLabel(), item.getDefaultValue(), item.isModifiable(),defPanel);
						break;

				}
				selectedControls.add(control);
			}
			composite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
			composite.layout();
			              
			return defPanel;
		}
		protected Text createLabeledText(
			String title,
			String value,
			boolean modifiable,
			Composite defPanel) {
			GridData gridData;
			Label label = new Label(defPanel, SWT.WRAP);
			gridData = new GridData();
			label.setLayoutData(gridData);
			label.setText(title);
			Text fText;
			if (modifiable) fText = new Text(defPanel, SWT.SHADOW_IN | SWT.BORDER);
			else fText = new Text(defPanel, SWT.SHADOW_IN | SWT.BORDER | SWT.READ_ONLY);
			gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
			gridData.horizontalSpan = 2;
			fText.setLayoutData(gridData);
			fText.setText(value);
			return fText;
		}
		
		protected Button createLabeledCheck(
			String title,
			boolean value,
			Composite defPanel) {
			GridData gridData;
			Label label = new Label(defPanel, SWT.WRAP);
			gridData = new GridData();
			label.setLayoutData(gridData);
			label.setText(title);

			Button fButton = new Button(defPanel, SWT.CHECK);
			gridData =
				new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
			gridData.horizontalSpan = 2;
			fButton.setLayoutData(gridData);
			fButton.setSelection(value);
			return fButton;
		}
		protected Text createLabeledPath(
			String title,
			String value,
			Composite defPanel) {
			GridData gridData;
			Label label = new Label(defPanel, SWT.WRAP);
			gridData = new GridData();
			//gridData.horizontalSpan = 1;
			label.setLayoutData(gridData);
			label.setText(title);

			final Text fText = new Text(defPanel, SWT.SHADOW_IN | SWT.BORDER);
			gridData =
				new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
			gridData.horizontalSpan = 1;
			fText.setLayoutData(gridData);
			fText.setText(value);
			final String fpath = value;
			Button fButton = new Button(defPanel, SWT.PUSH);
			fButton.setText("...");
			fButton.setLayoutData(new GridData());
			fButton.addSelectionListener(new SelectionListener() {
				public void widgetSelected(SelectionEvent e) {
					DirectoryDialog dlg =
						new DirectoryDialog(getControl().getShell());
					dlg.setFilterPath(fText.getText());
					String res = dlg.open();
					if (res != null) {
						fText.setText(res);
					}
				}

				public void widgetDefaultSelected(SelectionEvent e) {
					widgetSelected(e);
				}

			});

			return fText;
		}

		protected Text createLabeledFile(
			String title,
			String value,
			Composite defPanel) {
			GridData gridData;
			Label label = new Label(defPanel, SWT.WRAP);
			gridData = new GridData();
			//gridData.horizontalSpan = 1;
			label.setLayoutData(gridData);
			label.setText(title);

			final Text fText = new Text(defPanel, SWT.SHADOW_IN | SWT.BORDER);
			gridData =
				new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
			gridData.horizontalSpan = 1;
			fText.setLayoutData(gridData);
			fText.setText(value);

			Button fButton = new Button(defPanel, SWT.PUSH);
			fButton.setText("...");
			fButton.setLayoutData(new GridData());
			fButton.addSelectionListener(new SelectionListener() {
				public void widgetSelected(SelectionEvent e) {
					FileDialog dlg = new FileDialog(getControl().getShell());
					dlg.setFileName(fText.getText());
					String res = dlg.open();
					if (res != null) {
						fText.setText(res);
					}
				}

				public void widgetDefaultSelected(SelectionEvent e) {
					widgetSelected(e);
				}

			});

			return fText;
		}
		
	}

	class LibraryPanel {
		
		protected Control createContents(
			DriverDefinition element,
			String title,
			Composite parent,
			ArrayList jarlist,
			int type) {
			return createLibraryPanel(element, parent, jarlist , type);
		}

		protected Control createLibraryPanel(
			DriverDefinition element,
			Composite composite,
			ArrayList jarlist,
			int type) {

			Iterator properties;
			if (type == DRIVER) driverTable = addLibraryPanel(element, composite, "Driver Libraries",jarlist);
			else if (type == PROJECT) projectTable = addLibraryPanel(element,composite, "Project Libraries", jarlist);
			composite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
			composite.layout();
			return null;
		}




		private Table addLibraryPanel(
			DriverDefinition element,
			Composite composite,
			String groupName,
			ArrayList libraries) {
			GridLayout layout;
			GridData gridData;
			Group libraryPanel = new Group(composite, SWT.NONE);
			libraryPanel.setText(groupName);
			layout = new GridLayout(3, false);
			libraryPanel.setLayout(layout);
			gridData =
				new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL);
			libraryPanel.setLayoutData(gridData);
			Table list =
				new Table(
					libraryPanel,
					SWT.SINGLE | SWT.V_SCROLL | SWT.H_SCROLL | SWT.BORDER);
			Iterator properties = libraries.iterator();
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalSpan = 3;
			gridData.widthHint = convertWidthInCharsToPixels(100);
			gridData.heightHint = convertHeightInCharsToPixels(8);
			list.setLayoutData(gridData);
			while (properties.hasNext()) {
				ClasspathItem item = (ClasspathItem) properties.next();
				TableItem tableItem = new TableItem(list, SWT.NONE);
				tableItem.setText(item.getClasspath());
				if (item.exists())
					tableItem.setImage(JavaPluginImages.DESC_OBJS_JAR.createImage());
				else
					tableItem.setImage(JavaPluginImages.DESC_OBJS_FIXABLE_ERROR.createImage());
				tableItem.setData(item);
			}
			createFileButton("Add Library", element, list, libraryPanel);
			createRemoveButton("Remove", element, list, libraryPanel);
			composite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
			composite.layout();
			return list;
		}

		private void createFileButton(
			String title,
			final DriverDefinition element,
			final Table table,
			Composite defPanel) {
			GridData gridData;

			Button fButton = new Button(defPanel, SWT.PUSH);
			fButton.setText(title);
			fButton.setLayoutData(new GridData());

			fButton.addSelectionListener(new SelectionListener() {
				public void widgetSelected(SelectionEvent e) {
					FileDialog dlg = new FileDialog(getControl().getShell());
					dlg.setFilterExtensions(new String[] { "*.jar", "*.zip" });
					String res = dlg.open();
					if (res != null) {
						ClasspathItem item = new ClasspathItem();
						item.setRawClasspathFor(element, res);
						TableItem tableItem = new TableItem(table, SWT.NONE);
						tableItem.setText(item.getClasspath());
						tableItem.setData(item);
						if (item.exists())
							tableItem.setImage(JavaPluginImages.DESC_OBJS_JAR.createImage());
						else {
							tableItem.setImage(JavaPluginImages.DESC_OBJS_ERROR.createImage());
						}
					}
				}

				public void widgetDefaultSelected(SelectionEvent e) {
					widgetSelected(e);
				}
			});
		}

		private void createRemoveButton(
			String title,
			final DriverDefinition element,
			final Table table,
			Composite defPanel) {
			GridData gridData;

			Button fButton = new Button(defPanel, SWT.PUSH);
			fButton.setText(title);
			fButton.setLayoutData(new GridData());
			fButton.addSelectionListener(new SelectionListener() {
				public void widgetSelected(SelectionEvent e) {
					if (table.getSelectionCount() > 0) {
						int item = table.getSelectionIndex();
						table.remove(item);
					}
				}

				public void widgetDefaultSelected(SelectionEvent e) {
					widgetSelected(e);
				}

			});

		}
	}

}