

package org.objectweb.eclipsespeedo.utils;

import java.io.File;
import java.io.FileInputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.viewsupport.ImageImageDescriptor;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.drivers.xml.ClasspathItem;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;


public class Util {


	/**
	 * String replace all is JDK 1.4
	 * We have added this so that Lomboz is JDK 1.3 compat.
	 * @param string
	 * @param string2
	 */
	private static Logger logger = EclipseSpeedoPlugin.loggerFactory.getLogger(Util.class.getName());
	
	public static String replaceAll(String string, String pattern, String value) {
		int start = string.indexOf(pattern);
		if (start < 0)
			return string;
		String newStr =
			string.substring(0, start)
				+ value
				+ string.substring(start + pattern.length(), string.length());
		return replaceAll(newStr, pattern, value);
	}
	
	


	//
	// Search the existence of the library in the project classpath
	//
	private static boolean isInPath(
		IJavaProject project,
		String lib,
		int type) {
		boolean found = false;
		IClasspathEntry entry = null;
		switch (type) {
			case ClasspathItem.VARIABLE :
				entry =
					JavaCore.newVariableEntry(
						new Path(lib.replace('\\', '/')),
						null,
						null);
				break;

			case ClasspathItem.ABSOLUTE :
				entry =
					JavaCore.newLibraryEntry(
						new Path(lib.replace('\\', '/')),
						null,
						null);
				break;

			default :
				return true;
		}

		try {
			IClasspathEntry[] entries = project.getRawClasspath();
			IClasspathEntry resolvedClasspathEntry = JavaCore.getResolvedClasspathEntry(entry);
			for (int i = 0; i < entries.length; i++) {
				
				IClasspathEntry currentClasspathEntry = JavaCore.getResolvedClasspathEntry(entries[i]);
				if (currentClasspathEntry!= null && currentClasspathEntry.equals(resolvedClasspathEntry)) {
					found = true;
					return found;
				}
			}
		} catch (JavaModelException e) {
			logger.log(BasicLevel.ERROR, e);
		}
		return found;
	}

	//
	// Find if there is a classpath entry that matches the lib
	// and remove it.
	//
	private static void cleanPath(IJavaProject project, String lib, int type) {

		IClasspathEntry entry = null;
		switch (type) {
			case ClasspathItem.VARIABLE :
				entry =
					JavaCore.newVariableEntry(
						new Path(lib.replace('\\', '/')),
						null,
						null);
				break;

			case ClasspathItem.ABSOLUTE :
				entry =
					JavaCore.newLibraryEntry(
						new Path(lib.replace('\\', '/')),
						null,
						null);
				break;

			default :
				return;
		}

		try {
			IClasspathEntry[] entries = project.getRawClasspath();
			int nEntries = entries.length;
			for (int i = 0; i < nEntries; i++) {
				if (JavaCore
					.getResolvedClasspathEntry(entries[i])
					.equals(JavaCore.getResolvedClasspathEntry(entry))) {
					int newEntriesSize = entries.length - 1;
					int count = 0;
					IClasspathEntry[] newEntries =
						new IClasspathEntry[newEntriesSize];
					for (int j = 0; j < entries.length; j++) {
						if (j != i)
							newEntries[count++] = entries[j];
					}
					project.setRawClasspath(newEntries, null);
					return;
				}
			}
		} catch (JavaModelException e) {
			logger.log(BasicLevel.ERROR, e);
		}
		return;
	}

	private static void addToRawLibraryPath(IJavaProject project, String lib)
		throws JavaModelException {
		IClasspathEntry[] entries = project.getRawClasspath();
		int nEntries = entries.length;
		IClasspathEntry[] newEntries = new IClasspathEntry[nEntries + 1];
		for (int i = 0; i < entries.length; i++) {
			newEntries[i] = entries[i];
		}

		newEntries[nEntries] =
			JavaCore.newLibraryEntry(new Path(lib), null, null);
		project.setRawClasspath(newEntries, null);
	}
	private static void addToRawVariablePath(IJavaProject project, String lib)
		throws JavaModelException {
		IClasspathEntry[] entries = project.getRawClasspath();
		int nEntries = entries.length;
		IClasspathEntry[] newEntries = new IClasspathEntry[nEntries + 1];
		for (int i = 0; i < entries.length; i++) {
			newEntries[i] = entries[i];
		}

		newEntries[nEntries] =
			JavaCore.newVariableEntry(new Path(lib), null, null);
		project.setRawClasspath(newEntries, null);
	}

	public static void addVarPath(IJavaProject project, String relSrcPath)
		throws JavaModelException {
		try {
			boolean found = false;
			IClasspathEntry[] entries = project.getRawClasspath();
			int nEntries = entries.length;
			IClasspathEntry[] newEntries = new IClasspathEntry[nEntries + 1];
			for (int i = 0; i < nEntries; i++) {
				newEntries[i] = entries[i];
				if ((entries[i].getContentKind()
					== IClasspathEntry.CPE_VARIABLE
					|| entries[i].getContentKind() == IClasspathEntry.CPE_LIBRARY)
					&& entries[i].getPath().toString().equals(relSrcPath)) {
					found = true;
				}
			}
			if (!found) {
				newEntries[nEntries] =
					JavaCore.newVariableEntry(new Path(relSrcPath), null, null);
				project.setRawClasspath(newEntries, null);
			}
		} catch (JavaModelException e) {
			logger.log(BasicLevel.ERROR, e);
		}
	}

	public static void createSourceFolder(
		IJavaProject project,
		String relSrcPath) {

		try {
			addSourceFolder(project, relSrcPath);
		} catch (Exception e) {
			logger.log(BasicLevel.ERROR, e);
		}
	}

	public static void addSourceFolder(IJavaProject project, String relSrcPath)
		throws CoreException, JavaModelException {
		IFolder jfolder = project.getProject().getFolder(relSrcPath);
		IPath jpath = jfolder.getFullPath();
		if (!jfolder.exists()) {
			jfolder.create(true, true, null);
		}

		IClasspathEntry[] entries = project.getRawClasspath();
		int nEntries = entries.length;
		boolean found = false;
		IClasspathEntry[] newEntries = new IClasspathEntry[nEntries + 1];
		for (int i = 0; i < nEntries; i++) {
			newEntries[i] = entries[i];
			if (entries[i].getPath().toString().endsWith(relSrcPath)) {
				found = true;
			}
		}
		if (!found) {
			newEntries[nEntries] = JavaCore.newSourceEntry(jpath);
			project.setRawClasspath(newEntries, null);
		}
	}

	public static boolean isJavaPackage(IResource resource)
		throws JavaModelException {
		if (resource.getType() != IResource.FOLDER)
			return false;
		IJavaElement je = (IJavaElement) JavaCore.create(resource);
		if (je == null)
			return false;
		return (je.getElementType() == IJavaElement.PACKAGE_FRAGMENT);
	}

	public static void addClasspathVariable(String var, String value) {
		IProgressMonitor monitor = null;
		try {
			IPath library = new Path(value);
			IPath existingVar = JavaCore.getClasspathVariable(var);
			JavaCore.newVariableEntry(new Path(var), null, null);
			if (existingVar == null) {
				JavaCore.newVariableEntry(new Path(var), null, null);
			}
			if (monitor == null) {
				monitor = new NullProgressMonitor();
			}

			monitor.beginTask("Setting " + var + " path", 1);
			try {
				IPath oldLibrary = JavaCore.getClasspathVariable(var);

				if (!library.equals(oldLibrary)) {
					JavaCore.setClasspathVariable(
						var,
						library,
						new SubProgressMonitor(monitor, 1));
				}
			} finally {
				monitor.done();
			}

		} catch (JavaModelException e) {
			logger.log(BasicLevel.ERROR, e);
		}

	}

	public static String[] createClassPath(IProject project, String appendCP)
		throws CoreException {
		String[] apcp = parseCP(appendCP);
		String[] cp =
			JavaRuntime.computeDefaultRuntimeClassPath(
				JavaCore.create(project));

		String[] all = new String[apcp.length + cp.length];
		for (int i = 0; i < apcp.length; i++) {
			all[i] = apcp[i];
		}
		for (int i = 0; i < cp.length; i++) {
			all[i + apcp.length] = cp[i];
		}

		return all;
	}
	
	public static String[] parseCP(String cp) {
		StringTokenizer tok = new StringTokenizer(cp, ";:");

		String[] cpa = new String[tok.countTokens()];
		int count = 0;
		while (tok.hasMoreTokens()) {
			cpa[count++] = tok.nextToken();
		}
		return cpa;
	}
	
	public static String mergeCP(String[] cpa) {

		String cp = "";
		int count = 0;
		for (int i = 0; i < cpa.length; i++) {
			cp += cpa[i];
			if( i < cpa.length-1)
				cp += File.pathSeparator;
			
		}
		return cp;
	}

	public static String asClassPath(IProject project) throws CoreException {
		String[] cp = createClassPath(project, "");
		StringBuffer buf = new StringBuffer(256);
		for (int i = 0; i < cp.length; i++) {
			String string = cp[i];
			buf.append(string);
			if (i != cp.length - 1)
				buf.append(File.pathSeparatorChar);
		}
		return buf.toString();
	}

	public static boolean fileNameMatches(
		String name,
		String ext,
		String[] filters) {
		if (name == null)
			return false;
		for (int i = 0; i < filters.length; i++) {
			if (filters[i].equals(name))
				return true;
			if (ext != null
				&& filters[i] != null
				&& filters[i].charAt(0) == '*') {
				if (filters[i].endsWith(ext))
					return true;
			}

		}
		return false;
	}


	public static ImageDescriptor createImageDescriptor(String file) {
		try {
			URL urlToImage =
				new URL(
					EclipseSpeedoPlugin.getDefault().getDescriptor().getInstallURL(),
					file);
			return ImageDescriptor.createFromURL(urlToImage);
		} catch (MalformedURLException e) {
			logger.log(BasicLevel.ERROR, e);
			return null;
		}

	}

	public static Image createImage(String fName, Class clazz) {
		try {
			logger.log(BasicLevel.DEBUG, "Entering Create Image. arg:"+EclipseSpeedoPlugin.getDefault().getInstalledURL()+"/"+fName);
			return (
				new Image(
					Display.getCurrent(),
					new FileInputStream(EclipseSpeedoPlugin.getDefault().getInstalledURL()+"/"+fName)));
		} catch (Exception e) {
			logger.log(BasicLevel.ERROR, e);
			return new Image(Display.getCurrent(), 1, 1);
		}
	}

	public static ImageDescriptor createImageDeseriptor(String fName, Class clazz) {
		try {
			return ImageImageDescriptor.createFromFile(Util.class,fName);
		} catch (Exception e) {
			logger.log(BasicLevel.ERROR, e);
			return null;
		}
	}

}