package org.objectweb.eclipsespeedo.wizards;

import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.ClassInstanceCreation;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.ImportDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.StringLiteral;
import org.eclipse.jdt.core.dom.TryStatement;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.ui.IJavaElementSearchConstants;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;


/**
 * The main creation wizard page
 */
public class MainCreationWizardPage extends WizardPage {
	private Text fileText;
	private IJavaElement Uselection;
	private IType classe;
	private int ASTClasse;
	private IProject project;
	private IJavaProject jp;
	private Button importJDOButton;
	private Button mainClass;
	private boolean hasJavaUtilImport = false;
	private boolean hasJavaIOImport = false;
	private boolean hasJDOOImport = false;
	private boolean hasProperties = false;
	private boolean hasMainMethod = false;

	/**
	 * The constructor
	 * 
	 * @param projectName
	 *            is projectName is null , that mean we want to create a new
	 *            file (new wizard case)
	 */
	public MainCreationWizardPage(IJavaElement jElSelected)	{
		super("wizardPage");
		setTitle(EclipseSpeedoPlugin.getResourceString("MainCreationWizardPage.title"));
		setDescription(EclipseSpeedoPlugin.getResourceString("MainCreationWizardPage.description"));
		this.Uselection = jElSelected;
		jp = jElSelected.getJavaProject();
		project = jp.getProject();
	}
	

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent)	{
		try	{
			GridData gd;
			Label label;
			Composite container = new Composite(parent, SWT.NULL);
			GridLayout layout = new GridLayout();
			container.setLayout(layout);
			layout.numColumns = 3;
			layout.verticalSpacing = 9;


			label = new Label(container, SWT.NULL);
			label.setText(EclipseSpeedoPlugin.getResourceString("MainCreationWizardPage.targetclass"));

			this.fileText =	new Text(container, SWT.BORDER | SWT.SINGLE | SWT.H_SCROLL | SWT.READ_ONLY);
			gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.widthHint = 150;
			this.fileText.setLayoutData(gd);

				 
			this.fileText.addModifyListener(new ModifyListener(){
				public void modifyText(ModifyEvent e){
					updateStatus();
				}
			});

			Button button = new Button(container, SWT.PUSH);
			button.setText("Browse...");
			gd = new GridData(GridData.FILL_HORIZONTAL);
			button.setLayoutData(gd);
			button.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					handleBrowse();
				}
			});
			
			// Separator
			label = new Label(container, SWT.SEPARATOR | SWT.HORIZONTAL);
			gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.horizontalSpan = 3;
			label.setLayoutData(gd);
			
			//	Imports
			gd = new GridData(GridData.FILL_HORIZONTAL);
			label = new Label(container, SWT.NULL);
			label.setText(EclipseSpeedoPlugin.getResourceString("MainCreationWizardPage.jdoimport"));
			label.setLayoutData(gd);
			gd = new GridData(GridData.FILL_HORIZONTAL);
			importJDOButton = new Button(container, SWT.CHECK);
			importJDOButton.addListener(SWT.Selection, new Listener(){
				public void handleEvent(Event e)
				{
					updateStatus();
				}
			});
			gd.horizontalSpan = 2;
			importJDOButton.setLayoutData(gd);
			importJDOButton.setSelection(true);

			//	do we add the main class and the properties
			gd = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
			gd.horizontalSpan = 1;
			label = new Label(container, SWT.NULL);
			label.setText(EclipseSpeedoPlugin.getResourceString("MainCreationWizardPage.mainclassproperties"));
			label.setLayoutData(gd);

			mainClass = new Button(container, SWT.CHECK);
			gd = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
			gd.horizontalSpan = 2;
			mainClass.setLayoutData(gd);
			mainClass.addListener(SWT.Selection, new Listener()	{
				public void handleEvent(Event e)
				{
					mainClassChanged(mainClass.getSelection());
				}
			});
			if (Uselection.getElementType() == IJavaElement.TYPE) {
				this.fileText.setText(Uselection.getElementName()+" in file "+((IType)Uselection).getCompilationUnit().getElementName());
				this.classe = ((IType)Uselection);
				updateAndCheckUnit(((IType)Uselection).getCompilationUnit(), this.classe);
			}
			mainClass.setSelection(true);

			updateStatus();
			setControl(container);
		}
		catch (Exception e)	{
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Select a package in the current project
	 * 
	 * @return IPackageFragment
	 * @throws JavaModelException
	 */
	public IPackageFragment selectType() throws JavaModelException {
		SelectionDialog packageDialog = createPackageDialog();
		if (packageDialog.open() == IDialogConstants.CANCEL_ID) return null;
		Object[] types = packageDialog.getResult();
		if (types == null || types.length == 0)
			return null;
		return (IPackageFragment) types[0];
	}

	/**
	 * Create a packageDialog specific for this project
	 * 
	 * @return @throws
	 *         JavaModelException
	 */
	public SelectionDialog createPackageDialog() throws JavaModelException {
		SelectionDialog packageDialog = JavaUI.createPackageDialog(getShell(), jp, 0, "");
		packageDialog.setTitle(EclipseSpeedoPlugin.getResourceString("MainCreationWizardPage.classSelection"));
		packageDialog.setMessage(EclipseSpeedoPlugin.getResourceString("MainCreationWizardPage.classSelectionmsg"));
		return packageDialog;
	}

	/**
	 * Check if there is a selected file
	 *  
	 */
	private void updateStatus()	{
		String container = getFileName();
		if (container.length() == 0	&& (mainClass.getSelection() || importJDOButton.getSelection())) {
			setPageComplete(false);
			return;
		}
		else setPageComplete(true);
	}


	/**
	 * @return String
	 */
	public String getFileName()	{
		if (fileText == null)
			return "";
		else
			return fileText.getText();
	}

	/**
	 * enable/disable the jdo properties
	 * 
	 * @param check
	 */
	private void mainClassChanged(boolean check) {
		updateStatus();
	}

	/**
	 * Allow to select a class We first select a package , and then a class.
	 *  
	 */
	private void handleBrowse()	{
		try	{
			// Select the package
			IPackageFragment type = selectType();

			if (type == null)	{
				return;
			}

			IPackageFragment pf = jp.findPackageFragment(type.getPath());

			if (pf == null)	{
				return;
			}
			ICompilationUnit[] a = pf.getCompilationUnits();

			if (a == null || a.length == 0) {
				return;
			}

			Object[] obj = new Object[pf.getCompilationUnits().length];
			for (int i = 0; i < pf.getCompilationUnits().length; i++) {
				obj[i] = a[i].getElementName();
			}

			/**
			 * Select a class in the package scope
			 */
			SelectionDialog typeDialog = JavaUI.createTypeDialog(EclipseSpeedoPlugin.getShell(), new ProgressMonitorDialog(getShell()), SearchEngine.createJavaSearchScope(pf.getChildren(), true), IJavaElementSearchConstants.CONSIDER_CLASSES, false, "*");

			typeDialog.setTitle(EclipseSpeedoPlugin.getResourceString("MainCreationWizardPage.classSelection"));
			typeDialog.setInitialSelections(obj);
			typeDialog.open();

			Object[] types = typeDialog.getResult();

			if (types == null) {
				return;
			}

			// we make sure that it is a type
			if (!(types[0] instanceof IType)){
				return;
			}

			//we get the name of the element
			IType selectedType = (IType) types[0];
			selectTarget(selectedType);
		}
		catch (Exception e)	{
			e.printStackTrace();
		}
	}

	private void selectTarget(IType clas) {
		String clasname = clas.getElementName();
		String fileName = clas.getCompilationUnit().getElementName();
		fileText.setText(clasname + " in file " +fileName);
		this.classe = clas;
		updateAndCheckUnit(clas.getCompilationUnit(), clas);
	}

	/**
	 * We do a pre-parsing of the unit, to get so existing values in the code
	 * We look for imports and for jdo properties
	 * 
	 * @param unit
	 */
	public void updateAndCheckUnit(ICompilationUnit unit, IType clas) {
		CompilationUnit cu = AST.parseCompilationUnit(unit, true);
		setDefault();
		TypeDeclaration tdecl = null;
		//imports
		List imports = cu.imports();
		for (int i = 0; i < imports.size(); i++){
			// look for javax.jdo
			if (((ImportDeclaration) imports.get(i)).getName().toString().equals("javax.jdo")) {
				hasJDOOImport = true;
				importJDOButton.setSelection(false);
				importJDOButton.setEnabled(false);
			}
			// look for java.util
			if (((ImportDeclaration) imports.get(i)).getName().toString().equals("java.util")) {
				hasJavaUtilImport = true;
			}
			// look for java.io
			if (((ImportDeclaration) imports.get(i)).getName().toString().equals("java.io")) {
				hasJavaIOImport = true;
			}
		}
		// We get types
		List typesList = cu.types();
		int p=0;
		for (Iterator ite=typesList.iterator();ite.hasNext();){
			Object o = ite.next();
			if ((o instanceof TypeDeclaration)&&((((TypeDeclaration) o).getName()).toString().equals(clas.getElementName()))){
				tdecl = (TypeDeclaration) o;
				this.ASTClasse = p;
				break;
			}
			p++;
		}
		
		if (tdecl == null){
			return;
		}
		
		MethodDeclaration[] methodDecl = tdecl.getMethods();			
		for (int j = 0; j < methodDecl.length; j++)	{
			MethodDeclaration tmp = (MethodDeclaration) methodDecl[j];
			if (tmp.getName().toString().equals("main")) {
				hasMainMethod = true;
				Block block = tmp.getBody();
				List statements = block.statements();
				for (int k = 0; k < statements.size(); k++)	{
					Statement statementTmp = (Statement) statements.get(k);
					if (statementTmp.getNodeType() ==  ASTNode.TRY_STATEMENT) {
						Block tryBlock = ((TryStatement) statementTmp).getBody();
						List trystatements = tryBlock.statements();
						for (int i = 0; i < trystatements.size(); i++)	{
							Statement curstatement = (Statement) trystatements.get(i);
							if (curstatement.getNodeType() ==  ASTNode.EXPRESSION_STATEMENT){
								ExpressionStatement exps = (ExpressionStatement) curstatement;
								Expression exp = exps.getExpression();
								if ((exp.getNodeType() == ASTNode.METHOD_INVOCATION)&&(((MethodInvocation)exp).getName().toString().equals("load"))) {
									MethodInvocation mi = (MethodInvocation) exp;
									List argList = mi.arguments();
									if (argList.size() == 1 && (argList.get(0) instanceof ClassInstanceCreation)) {
										ClassInstanceCreation cic = (ClassInstanceCreation) argList.get(0);
										if ((cic.arguments().get(0) instanceof StringLiteral) && 
										    (((StringLiteral) cic.arguments().get(0)).toString().equals("\"enhancer.properties\"")) &&
										    (mi.getExpression().resolveTypeBinding().getName().toString().equals("Properties"))){
											
											hasProperties = true;
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}


	private void setDefault() {
		this.hasJavaUtilImport = false;
		this.hasJavaIOImport = false;
		this.hasJDOOImport = false;
		this.hasMainMethod = false;
		this.hasProperties = false;
	}

	public IType getClasse(){
		return classe;
	}
	
	public int getASTClass(){
		return ASTClasse;
	}
	
	/**
	 * @return Button
	 */
	public boolean getimportJDOButton()	{
		return importJDOButton.getSelection();
	}

	public void setProject(IProject p) {
		project = p;
		jp = JavaCore.create(p);
	}

	/**
	 * @return
	 */
	public boolean hasJavaUtilImport() {
		return hasJavaUtilImport;
	}

	/**
	 * @return
	 */
	public boolean hasJavaIOImport() {
		return hasJavaIOImport;
	}
	
	/**
	 * @return
	 */
	public boolean hasJDOOImport() {
		return hasJDOOImport;
	}

	/**
	 * @return
	 */
	public boolean hasProperties() {
		return hasProperties;
	}

	/**
	 * @return
	 */
	public boolean hasMainMethod() {
		return hasMainMethod;
	}
}