/*
 * Created on Apr 2, 2004, 9:13:59 AM
 * author: VEZIER Antoine
 * Internship Master : Hamon Catherine
 *  
 */

package org.objectweb.eclipsespeedo.wizards;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.JavaPluginImages;
import org.eclipse.jdt.internal.ui.actions.WorkbenchRunnableAdapter;
import org.eclipse.jdt.internal.ui.util.ExceptionHandler;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;
import org.objectweb.eclipsespeedo.EclipseSpeedoPlugin;
import org.objectweb.eclipsespeedo.project.EclipseSpeedoProjectHelper;


/**
 * This is the Speedo new Project Creation Wizard
 */

public class SpeedoNewProjectWizard extends Wizard implements IExecutableExtension,INewWizard {
	
	private WizardNewProjectCreationPage mainPage;
	private ISelection selection;
	private SpeedoNewProjectWizardPage javaPage;
	private IConfigurationElement configElement;
	/**
	 * Constructor for SampleNewWizard.
	 */
	public SpeedoNewProjectWizard() {
		super();
		setNeedsProgressMonitor(true);
		setDefaultPageImageDescriptor(JavaPluginImages.DESC_WIZBAN_NEWJPRJ);
		setDialogSettings(EclipseSpeedoPlugin.getDefault().getDialogSettings());
		setWindowTitle(EclipseSpeedoPlugin.getResourceString("SpeedoNewProjectCreationWizard.title")); 
	}
	
	/**
	 * Adding the page to the wizard.
	 */
	public void addPages() {
		mainPage = new WizardNewProjectCreationPage("EclipseSpeedo.SpeedoNewProject");
		mainPage.setTitle(EclipseSpeedoPlugin.getResourceString("SpeedoNewProjectCreationWizardPage.title"));
		mainPage.setDescription(EclipseSpeedoPlugin.getResourceString("SpeedoNewProjectCreationWizardPage.description"));
		addPage(mainPage);
		javaPage = new SpeedoNewProjectWizardPage(mainPage);
		addPage(javaPage);
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 */
	public boolean performFinish() {
		IWorkspaceRunnable op = new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException, OperationCanceledException {
				try	{
					finishPage(monitor);
				}
				catch (InterruptedException e) {
					throw new OperationCanceledException(e.getMessage());
				}
			}
		};
		try	{
			getContainer().run(false, true, new WorkbenchRunnableAdapter(op));
		}
		catch (InvocationTargetException e)	{
			handleFinishException(getShell(), e);
			return false;
		}
		catch (InterruptedException e) {
			return false;
		}
		final IJavaProject javaProject = JavaCore.create(mainPage.getProjectHandle());
		Display.getCurrent().asyncExec(new Runnable() {
			public void run() {
				EclipseSpeedoProjectHelper.addEclipseSpeedoNature(javaProject);
			}
			

		});
		return true;
	}
	
	/**
	 * Stores the configuration element for the wizard. The config element will
	 * be used in <code> performFinish </code> to set the result perspective.
	 */
	public void setInitializationData(IConfigurationElement cfig, String propertyName, Object data)	{
		configElement = cfig;
	}
	
	protected void finishPage(IProgressMonitor monitor)	throws InterruptedException, CoreException {
		javaPage.performFinish(monitor); // use the full progress monitor
		BasicNewProjectResourceWizard.updatePerspective(configElement);
	}
	
	protected void handleFinishException(Shell shell, InvocationTargetException e) {
		String title = "NewProjectCreationWizard.op_error.title";
		String message = "NewProjectCreationWizard.op_error_create.message";

		e.printStackTrace();
		ExceptionHandler.handle(e, getShell(), title, message);
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
}