/*
 * Created on Apr 2, 2004, 9:13:59 AM
 * author: VEZIER Antoine
 * Internship Master : Hamon Catherine
 *  
 */

package org.objectweb.eclipsespeedo.wizards;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.wizards.ClassPathDetector;
import org.eclipse.jdt.ui.wizards.JavaCapabilityConfigurationPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.IWizardContainer;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;



public class SpeedoNewProjectWizardPage extends JavaCapabilityConfigurationPage {

	private ISelection selection;

	private WizardNewProjectCreationPage mainPage;

	private IPath currProjectLocation;
	protected IProject currProject;

	protected boolean canRemoveContent;

	public SpeedoNewProjectWizardPage(WizardNewProjectCreationPage Page) {
		super();
		mainPage = Page;
		currProjectLocation = null;
		currProject = null;
		canRemoveContent = false;
	}
	
	/**
	 * inherited method is redefined in order to handle project creation before 
	 * showing the page
	 * 
	 */
	public void setVisible(boolean visible)	{
		if (visible) {
			changeToNewProject();
		}
		else {
			IWizardContainer wizardContainer = getWizard().getContainer();
			if (wizardContainer.getCurrentPage() == getPreviousPage()) {
				removeProject();
			}
		}
		super.setVisible(visible);
	}

	private void changeToNewProject() {
		IProject newProjectHandle = mainPage.getProjectHandle();
		IPath newProjectLocation = mainPage.getLocationPath();

		// Tests if location of the future project is ok.
		if (mainPage.useDefaults()) {
			canRemoveContent =	!newProjectLocation.append(mainPage.getProjectName()).toFile().exists();
		}
		else {
			canRemoveContent = !newProjectLocation.toFile().exists();
		}

		// Do the curr* fields of SpeedoNewProjectWizardPage have to be initialized
		final boolean initialize =	!(newProjectHandle.equals(currProject)&& newProjectLocation.equals(currProjectLocation));
		
		// The IRunnableProgress to be runned by the wizardcontainer
		IRunnableWithProgress op = 
		new IRunnableWithProgress()	{
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				try	{
					updateProject(initialize, monitor);
				}
				catch (CoreException e)	{
					throw new InvocationTargetException(e);
				}
			}
		};

		try	{
			getContainer().run(false, true, op);
		}
		catch (InvocationTargetException e)	{
			e.printStackTrace();
		}
		catch (InterruptedException e) {
			e.printStackTrace();
			// cancel pressed
		}
	}
	
	/**
	 * Ensure that Project settings are Ok 
	 * and call intialization method for the wizardpage.
	 */
	protected void updateProject(boolean initialize, IProgressMonitor monitor) throws CoreException, InterruptedException {
		currProject = mainPage.getProjectHandle();
		currProjectLocation = mainPage.getLocationPath();
		boolean noProgressMonitor = !initialize && canRemoveContent;

		if (monitor == null || noProgressMonitor) {
			monitor = new NullProgressMonitor();
		}
		try	{
			monitor.beginTask("SpeedoNewProjectWizardPage.EarlyCreationOperation.desc", 2);

			createProject(currProject, currProjectLocation, new SubProgressMonitor(monitor, 1));
			if (initialize)	{
				IClasspathEntry[] entries = null;
				IPath outputLocation = null;

				if (currProjectLocation.toFile().exists()&& !Platform.getLocation().equals(currProjectLocation)) {
					// detect classpath
					if (!currProject.getFile(".classpath").exists()){
						// if .classpath exists noneed to look for files
						ClassPathDetector detector = new ClassPathDetector(currProject);
						entries = detector.getClasspath();
						outputLocation = detector.getOutputLocation();
					}
				}
				// JavaCore.create(currProject) returns the JavaProject corresponding to the IProject parameter
				// initialize the page
				init(JavaCore.create(currProject), outputLocation, entries, false);
			}
			monitor.worked(1);
		}
		finally	{
			monitor.done();
		}
	}

	/**
	 * Called from the wizard on finish.
	 */
	public void performFinish(IProgressMonitor monitor)	throws CoreException, InterruptedException {
		try	{
			monitor.beginTask("SpeedoNewProjectWizardPage.createproject.desc", 3);
			if (currProject == null)
			{
				updateProject(true, new SubProgressMonitor(monitor, 1));
			}
			configureJavaProject(new SubProgressMonitor(monitor, 2));
		}
		finally	{
			monitor.done();
			currProject = null;
		}
	}
	/**
	 * Called if project creation is cancelled
	 */
	private void removeProject() {
		if (currProject == null || !currProject.exists())	{
			return;
		}

		IRunnableWithProgress op = 
		new IRunnableWithProgress()	{
			public void run(IProgressMonitor monitor)
				throws InvocationTargetException, InterruptedException
			{
				boolean noProgressMonitor =
					Platform.getLocation().equals(currProjectLocation);
				if (monitor == null || noProgressMonitor)
				{
					monitor = new NullProgressMonitor();
				}
				monitor.beginTask("JDO NewProjectCreationWizardPage remove project", 3);

				try
				{
					currProject.delete(canRemoveContent, false, monitor);
				}
				catch (CoreException e)
				{
					throw new InvocationTargetException(e);
				}
				finally
				{
					monitor.done();
					currProject = null;
					canRemoveContent = false;
				}
			}
		};

		try	{
			getContainer().run(false, true, op);
		}
		catch (InvocationTargetException e)	{
			e.printStackTrace();
		}
		catch (InterruptedException e) {
			e.printStackTrace();
			// cancel pressed
		}
	}

	/**
	 * Called from the wizard on cancel.
	 */
	public void performCancel()	{
		removeProject();
	}

}