/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.api;

import org.objectweb.jorm.api.PException;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.ExpressionException;

import javax.jdo.JDOException;

/**
 * This helper permits to find the final nested exception of a given exception.
 *
 * @author S.Chassande-Barrioz
 */
public class ExceptionHelper {

    public static final Exception getNested(Exception e) {
        if (e == null)
            return null;
        Exception c = e;
        Exception i = null;
        boolean notLast = true;
        while (notLast) {
            if (c instanceof SpeedoException) {
                i = ((SpeedoException) c).getNestedException();
            } else if (c instanceof PException) {
                i = ((PException) c).getNestedException();
            } else if (c instanceof MedorException) {
                i = ((MedorException) c).getNestedException();
            } else if (c instanceof ExpressionException) {
                i = ((ExpressionException) c).getNestedException();
            } else if (c instanceof PersistenceException) {
                i = ((PersistenceException) c).getNestedException();
            } else if (c instanceof JDOException) {
                Throwable[] ts = ((JDOException) c).getNestedExceptions();
				if (ts != null && ts.length >0 && (ts[0] instanceof Exception)) {
					i = (Exception) ts[0];
				} else {
					notLast = false;
				}
            } else {
                notLast = false;
            }
            if (notLast && i != null) {
                c = i;
            } else {
                notLast = false;
            }
        }
        return c;
    }

}
