/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.api;

import javax.jdo.JDOFatalException;

/**
 * This exception is the result of a user accessing a field of a detached
 * instance, where the field was not copied to the detached instance.
 * 
 * @author Y.Bersihand
 */

public class JDODetachedFieldAccessException extends JDOFatalException {

	/**
	 * Constructs a new <code>JDODetachedFieldAccessException</code> without a
	 * detail message.
	 */
	public JDODetachedFieldAccessException() {
	}

	/**
	 * Constructs a new <code>JDODetachedFieldAccessException</code> with the
	 * specified detail message.
	 * 
	 * @param msg the detail message.
	 */
	public JDODetachedFieldAccessException(String msg) {
		super(msg);
	}

	/**
	 * Constructs a new <code>JDODetachedFieldAccessException</code> with the
	 * specified detail message and nested <code>Throwable</code>s.
	 * 
	 * @param msg
	 *            the detail message.
	 * @param nested
	 *            the nested <code>Throwable[]</code>.
	 */
	public JDODetachedFieldAccessException(String msg, Throwable[] nested) {
		super(msg, nested);
	}

	/**
	 * Constructs a new <code>JDODetachedFieldAccessException</code> with the
	 * specified detail message and nested <code>Throwable</code>s.
	 * 
	 * @param msg
	 *            the detail message.
	 * @param nested
	 *            the nested <code>Throwable</code>.
	 */
	public JDODetachedFieldAccessException(String msg, Throwable nested) {
		super(msg, nested);
	}

	/**
	 * * Constructs a new <code>JDODetachedFieldAccessException</code> with
	 * the specified detail message and failed object.
	 * 
	 * @param msg
	 *            the detail message.
	 * @param failed
	 *            the failed object.
	 */
	public JDODetachedFieldAccessException(String msg, Object failed) {
		super(msg, failed);
	}

	/**
	 * * Constructs a new <code>JDODetachedFieldAccessException</code> with
	 * the specified detail message, nested <code>Throwable</code>s, and
	 * failed object.
	 * 
	 * @param msg
	 *            the detail message.
	 * @param nested
	 *            the nested <code>Throwable[]</code>.
	 * @param failed
	 *            the failed object.
	 */
	public JDODetachedFieldAccessException(String msg, Throwable[] nested,
			Object failed) {
		super(msg, nested, failed);
	}

	/**
	 * * Constructs a new <code>JDODetachedFieldAccessException</code> with
	 * the specified detail message, nested <code>Throwable</code>s, and
	 * failed object.
	 * 
	 * @param msg
	 *            the detail message.
	 * @param nested
	 *            the nested <code>Throwable</code>.
	 * @param failed
	 *            the failed object.
	 */
	public JDODetachedFieldAccessException(String msg, Throwable nested,
			Object failed) {
		super(msg, nested, failed);
	}
}