/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.api;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * @author S.Chassande-Barrioz
 */
public class SpeedoException extends Exception {

    protected Exception nestedException;

    public SpeedoException() {
    }

    public SpeedoException(String s) {
        super(s);
    }

    public SpeedoException(Exception nested) {
        this.nestedException = nested;
    }

    public SpeedoException(String s, Exception nested) {
        super(s);
        this.nestedException = nested;
    }

    public Exception getNestedException() {
        return nestedException;
    }

    public void printStackTrace(PrintStream stream) {
        if (nestedException != null) {
            stream.println(getMessage());
            nestedException.printStackTrace(stream);
        } else
            super.printStackTrace(stream);
    }

    public void printStackTrace(PrintWriter writer) {
        if (nestedException != null) {
            writer.println(getMessage());
            nestedException.printStackTrace(writer);
        } else
            super.printStackTrace(writer);
    }

    public void printStackTrace() {
        printStackTrace(System.out);
    }
}
