/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass;

import org.objectweb.speedo.genclass.api.SpeedoGenClassProxy;
import org.objectweb.speedo.metadata.SpeedoVersion;
import org.objectweb.speedo.mim.api.LifeCycle;
import org.objectweb.speedo.mim.api.DetachedLifeCycle;
import org.objectweb.speedo.workingset.api.Transaction;
import org.objectweb.speedo.mim.api.SpeedoAccessor;
import org.objectweb.speedo.mim.api.SpeedoProxy;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.pm.api.ProxyManagerFactory;
import org.objectweb.speedo.genclass.api.SpeedoGenClassListener;
import org.objectweb.speedo.api.Debug;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.perseus.persistence.api.TransactionalPersistenceManager;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.RolledBackPersistenceException;
import org.objectweb.perseus.cache.api.UnFixProtocolException;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;

import javax.jdo.PersistenceManager;
import javax.jdo.JDOUnsupportedOptionException;
import javax.jdo.JDOException;
import javax.jdo.JDOUserException;
import javax.jdo.JDOFatalException;
import javax.jdo.spi.PersistenceCapable;
import javax.jdo.spi.StateManager;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;

/**
 * This class is a basic implementation of the SpeedoGenClassProxy interface.
 * It uses a delegate PBinding.
 *
 * @author S.Chassande-Barrioz
 */
public abstract class GenClass
        implements SpeedoGenClassProxy, Loggable, PBindingCtrl {

    /**
     * This fields indicates if the proxy is active (attached to at least a
     * Proxy Manager)
     */
    protected boolean jdoIsActive = false;

    /**
     * This is field is null when the proxy is not active
     */
    protected ProxyManagerFactory pmf = null;

    /**
     * This field is the Transactional persistence manager.
     */
    protected TransactionalPersistenceManager tpm = null;

    /**
     * is the PBinding which permits to store/load the data into/from the support
     */
    protected PBindingCtrl pbinding;

    /**
     * The logger used to trace this generic class
     */
    public Logger logger;

    /**
     * is the name of the mapper inside which the generic class is persistent
     */
    protected String mapperName;

    /**
     * is the name of the project name inside which the generic class is used.
     */
    protected String projectName;

    /**
     * is the type of the generic class
     */
    protected PType ptype;

    /**
     * The linked field associated to this gen class
     */
    protected String linkedField = null;

    /**
     * Is the PName which
     */
    protected Object pnameHints = null;

    /**
     * Id of the field that references this gen class instance
     */
    protected int gcid;

    /**
     * List of SpeedoGenClassListener
     */
    protected List listeners;

	protected long age;

	protected int fix = 0; 
	
	protected Object encodedPName;
	
    public void init(PType ptype,
                    String linkedField,
                    Object pnameHints,
					Logger logger) {
        this.pnameHints = pnameHints;
        this.ptype = ptype;
        this.linkedField = linkedField;
		this.logger = logger;
   }

	public void jdoSetPNameHints(Object hints) {
		pnameHints = hints;
	}

    public boolean elementIsReference() {
        return ptype.getNestedPType().getTypeCode() == PType.TYPECODE_REFERENCE;
    }

    public void jdoMakePersistent(ProxyManager pm, SpeedoProxy sp) {
        if (!sp.jdoIsActive()) {
            if (sp instanceof SpeedoGenClassProxy) {
				//Genclass contains genclass instances
                SpeedoGenClassProxy sgcp = (SpeedoGenClassProxy) sp;
                if (sgcp.jdoGetPType() == null) {
                    //Assign the PType to the generic class
                    sgcp.jdoSetPType(jdoGetPType().getNestedPType());
                }
                sgcp.jdoSetLinkedField(jdoGetGenClassId());
                sgcp.jdoSetPNameHints(getPName());
            }
			if (Debug.ON) {
				logger.log(BasicLevel.DEBUG, "jdoMakePersistent() pm=" + pm);
			}
			if (pm == null) {
				pm = (ProxyManager) jdoGetPersistenceManager();
			}
			if (pm == null) {
				throw new JDOUserException(
					"When a persistent object is used (read or write), " +
					"a PersistenceManager is needed");
			}
            pm.speedoMakePersistent(sp, true);
        }
    }

    public void jdoMakePersistent(ProxyManager pm, Iterator it) {
        if (it == null) {
            return;
        }
        while(it.hasNext()) {
            Object o = it.next();
            if (o instanceof SpeedoProxy) {
                jdoMakePersistent(pm, (SpeedoProxy) o);
            } else {
                // Optimisation: If the first element is not a SpeedoProxy, that
                // means that ALL collection elements are not SpeedoProxy
                // implementation
                return;
            }
        }
    }

    public void speedoSetGcId (int gcid) {
        this.gcid = gcid;
    }

    public void speedoAddGenClassListener (SpeedoGenClassListener l) {
        if (listeners == null) {
            listeners = new ArrayList();
			listeners.add(l);
        } else {
			for(int i=0; i<listeners.size(); i++) {
				if (listeners.get(i) == l) {
					return;
				}
			}
        	listeners.add(l);
		}
    }

    public void speedoAdd(Object elem, int gcFieldNumber) {
		throw new JDOException("Not supported in tuple class");
    }

	public boolean speedoAdd(Object elemToAdd, Object hints) {
		GenClassAccessor gca = (GenClassAccessor) jdoWriteIntention();
		return gca.speedoAdd(elemToAdd, hints);
	}

	public boolean speedoRemove(Object elemToRemove, Object hints) {
		GenClassAccessor gca = (GenClassAccessor) jdoWriteIntention();
		return gca.speedoRemove(elemToRemove, hints);
	}

    public void fireSpeedoElementAdded (Object elem) {
		if (Debug.ON) {
			logger.log(BasicLevel.DEBUG, "fireSpeedoElementAdded(): listner: "
				+ (listeners == null ? "no listener" : "" + listeners.size()));
		}
        if (listeners == null) {
            return;
        }
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            SpeedoGenClassListener l = (SpeedoGenClassListener)i.next();
            l.speedoElementAdded(elem, gcid);
        }
    }

    public void fireSpeedoElementRemoved (Object elem) {
		if (Debug.ON) {
			logger.log(BasicLevel.DEBUG, "fireSpeedoElementRemoved(): listner: "
				+ (listeners == null ? "no listener" : "" + listeners.size()));
		}
        if (listeners == null) {
            return;
        }
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            SpeedoGenClassListener l = (SpeedoGenClassListener)i.next();
            l.speedoElementRemoved(elem, gcid);
        }
    }

    // ABSTRACT METHODS //
    // -----------------//

    public abstract SpeedoAccessor createAccessor();

	public abstract SpeedoAccessor getReferenceAccessor();

	public abstract void setReferenceAccessor(SpeedoAccessor refAcc);

    // IMPLEMENTATION OF THE PBindingCtrl INTERFACE //
    // ---------------------------------------------//

    public void bind(PName name) throws PException {
        pbinding.bind(name);
    }

    public boolean exist(Object o) throws PException {
        return pbinding.exist(o);
    }

    public PName export(Object o) throws PException {
        return pbinding.export(o);
    }

    public PName export(Object o, Object o1) throws PException {
        return pbinding.export(o, o1);
    }

    public PClassMapping getPClassMapping() {
        return pbinding.getPClassMapping();
    }

    public PName getPName() {
        return pbinding.getPName();
    }

    public byte getStatus() {
        return pbinding.getStatus();
    }

    public void setPName(PName pName) {
        pbinding.setPName(pName);
    }

    public void setStatus(byte b) {
        pbinding.setStatus(b);
    }

    public void init(PClassMapping mapping) throws PException {
        pbinding.init(mapping);
    }

    public void read(Object o, PAccessor accessor) throws PException {
        pbinding.read(o, accessor);
    }

    public void read(Object o, PAccessor pAccessor, Object o1) throws PException {
        pbinding.read(o, pAccessor, o1);
    }

    public void unbind() throws PException {
        pbinding.unbind();
    }

    public void unexport(Object o) throws PException {
        pbinding.unexport(o);
    }

    public void write(Object o, PAccessor accessor) throws PException {
        if (logger != null)
            logger.log(BasicLevel.DEBUG, "Writing the gen class: " + linkedField);
        pbinding.write(o, accessor);
        GenClassAccessor gca = (GenClassAccessor) accessor;
        for(Iterator i = gca.elements.iterator();i.hasNext();) {
            GenClassElement gce = (GenClassElement) i.next();
            if (gce.getElemStatus() != PIndexedElem.ELEM_DELETED) {
                gce.setStatus(PIndexedElem.ELEM_UNMODIFIED);
            } else {
                // remove the element
                i.remove();
            }
        }
    }


    // IMPLEMENTATION OF THE PersistentCapable INTERFACE //
    //---------------------------------------------------//
    // Speedo implements partialy  the javax.jdo.spi package. Then only the
    // minimal method are implemented. The others throw a
    // JDOUnsupportedOptionException exception.

    public PersistenceManager jdoGetPersistenceManager() {
        return (jdoIsActive ? pmf.lookup() : null);
    }

    public void jdoReplaceFlags() {
        throw new JDOUnsupportedOptionException();
    }

    public PersistenceCapable jdoNewInstance(StateManager sm) {
        throw new JDOUnsupportedOptionException();
    }

    public PersistenceCapable jdoNewInstance(StateManager sm, Object oid) {
        throw new JDOUnsupportedOptionException();
    }

    public Object jdoNewObjectIdInstance() {
        throw new JDOUnsupportedOptionException();
    }

    public Object jdoGetObjectId() {
        return getPName();
    }

    public Object jdoGetTransactionalObjectId() {
        return getPName();
    }

    public void jdoReplaceField(int fieldNumber) {
        throw new JDOUnsupportedOptionException();
    }

    public void jdoReplaceFields(int[] fieldNumbers) {
        throw new JDOUnsupportedOptionException();
    }

    public void jdoProvideField(int fieldNumber) {
        throw new JDOUnsupportedOptionException();
    }

    public void jdoProvideFields(int[] fieldNumbers) {
        throw new JDOUnsupportedOptionException();
    }

    public void jdoCopyFields(Object pc, int[] fieldNumbers) {
        throw new JDOUnsupportedOptionException();
    }

    public void jdoMakeDirty(String fieldName) {
        // no fields => does nothing
    }

    public boolean jdoIsDirty() {
        SpeedoAccessor sa = getSpeedoAccessor();
        return sa != null
                && !LifeCycle.isTransient(sa.jdoGetStatus())
                && LifeCycle.isDirty(sa.jdoGetStatus());
    }

    public boolean jdoIsTransactional() {
        SpeedoAccessor sa = getSpeedoAccessor();
        return sa != null
                && !LifeCycle.isTransient(sa.jdoGetStatus())
                && LifeCycle.isTransactional(sa.jdoGetStatus());
    }

    public boolean jdoIsPersistent() {
        SpeedoAccessor sa = getSpeedoAccessor();
        return sa == null || LifeCycle.isPersistent(sa.jdoGetStatus());
    }

    public boolean jdoIsNew() {
        SpeedoAccessor sa = getSpeedoAccessor();
        return sa != null && LifeCycle.isNew(sa.jdoGetStatus());
    }

    public boolean jdoIsDeleted() {
        SpeedoAccessor sa = getSpeedoAccessor();
        return sa != null && LifeCycle.isDeleted(sa.jdoGetStatus());
    }

    public void jdoReplaceStateManager(StateManager sm) {
        // nothing to do, No State manager is used
    }

    public Object jdoNewObjectIdInstance(String s) {
        return null;
    }

    public void jdoCopyKeyFieldsToObjectId(Object o) {
        throw new JDOUnsupportedOptionException();
    }

    public void jdoCopyKeyFieldsToObjectId(PersistenceCapable.ObjectIdFieldSupplier objectIdFieldSupplier, Object o) {
        throw new JDOUnsupportedOptionException();
    }

    public void jdoCopyKeyFieldsFromObjectId(PersistenceCapable.ObjectIdFieldConsumer objectIdFieldConsumer, Object o) {
        throw new JDOUnsupportedOptionException();
    }


    // IMPLEMENTATION OF THE SpeedoGenClassProxy INTERFACE //
    //-----------------------------------------------------//

    /**
     * Assignes the PType of the generic class.
     * @param ptype of the generic class
     */
    public void jdoSetPType(PType ptype) {
        this.ptype = ptype;
    }

    /**
     * @return the PType of the generic class.
     */
    public PType jdoGetPType() {
        return ptype;
    }

    /**
     * assignes the PBinding to use to load/store the generic class
     * Call this method with null value means that the generic is no more
     * persistent.
     * @param pb is the PBinding.
     */
    public void jdoSetPBinding(PBinding pb) {
        pbinding = (PBindingCtrl) pb;
    }

    /**
     * @return the pbinding associated to the generic class. The value is if the
     * generic class is not persistent.
     */
    public PBinding jdoGetPBinding() {
        return pbinding;
    }

    /**
     * @return a string which represents the fully path of the persistent field
     * of the class with wich the generic class is persistent.
     * If the linked fiekf is the 'f1' provided by the class 'com.foo.Bar'
     * then the returned string will be 'com.foo.Bar/f1'
     */
    public String jdoGetGenClassId() {
        return linkedField;
    }

    /**
     * Assignes the linked field name for this generic class.
     * @param lf is tring which represents the fully path of the persistent
     * field of the class with wich the generic class is persistent.
     * If the linked fiekf is the 'f1' provided by the class 'com.foo.Bar'
     * then the expected string is 'com.foo.Bar/f1'
     */
    public void jdoSetLinkedField(String lf) {
        linkedField = lf;
    }

    /**
     * Assignes the new value of the generic class (Collection, Map or Array
     * or ...).
     */
    public void setElements(Object o) {
		((GenClassAccessor) jdoWriteIntention()).setElements(o);
    }

    // IMPLEMENTATION OF THE SpeedoProxy INTERFACE //
    //---------------------------------------------//

	public void copyAccessor(SpeedoAccessor src, SpeedoAccessor dest) {
		((GenClassAccessor) dest).jdoLoadFieldsFromAccessor(src);
	}

    public Object jdoGetPNameHints() {
        return pnameHints;
    }

    public String jdoGetProjectName() {
        return projectName;
    }


    public boolean jdoIsActive() {
        return jdoIsActive;
    }

    public void jdoIsActive(boolean newvalue) {
        jdoIsActive = newvalue;
    }

	public byte jdoGetVersioningStrategy(){
		return SpeedoVersion.NO_VERSION;
	}
	
    public SpeedoAccessor getSpeedoAccessor() {
        if (!jdoIsActive) {
            return getReferenceAccessor();
        }
        ProxyManager pm = pmf.lookup();
        if (pm == null) {
            throw new JDOUserException("When a persistent object is used (read or write), a PersistenceManager is needed");
        }
        Transaction tx = (Transaction) pm.currentTransaction();
        return (SpeedoAccessor) tx.lookup(getPName());
    }

    public Collection getSpeedoAccessors() {
		return null;
    }

    public void removeSpeedoAccessor(SpeedoAccessor sa) {
    }

    public ProxyManagerFactory jdoGetProxyManagerFactory() {
        return pmf;
    }

    public void jdoSetProxyManagerFactory(ProxyManagerFactory pmf) {
        this.pmf = pmf;
    }

    public TransactionalPersistenceManager jdoGetTransactionalPersistenceManager() {
        return tpm;
    }

    public void jdoSetTransactionalPersistenceManager(TransactionalPersistenceManager tpm) {
        this.tpm = tpm;
    }

    public SpeedoAccessor jdoReadIntention() {
		Transaction tx = (Transaction) pmf.lookup().currentTransaction();
        try {
            return (SpeedoAccessor) tpm.readIntention(tx, this);
        } catch (RolledBackPersistenceException e) {
            if (tx.isActive()) {
                tx.rollback();
            }
            throw new JDOFatalException("The current transaction has been rolledback, please retry", e);
        } catch (PersistenceException e) {
            throw new JDOException("Impossible to notify a read intention", e);
        }
    }

    public SpeedoAccessor jdoWriteIntention() {
		Transaction tx = (Transaction) pmf.lookup().currentTransaction();
        try {
            return (SpeedoAccessor) tpm.writeIntention(tx, this);
        } catch (RolledBackPersistenceException e) {
            if (tx.isActive()) {
                tx.rollback();
            }
            throw new JDOFatalException("The current transaction has been rolledback, please retry",e);
        } catch (PersistenceException e) {
            throw new JDOException("Impossible to notify a write intention", e);
        }
    }

    public Object getJormConfig() {
        return null;
    }

    public String jdoGetJdoFileName() {
        return null;
    }

    public void jdoDeletePersistent(ProxyManager pm, SpeedoAccessor sa) {
		((GenClassAccessor) sa).jdoDeletePersistent(pm);
    }

	public boolean fgIsPostload(String fgName) {
		// TODO Auto-generated method stub
		return false;
	}

	public Collection fgGetFieldsToLoad(String fgName) {
		// TODO Auto-generated method stub
		return null;
	}

	public Collection fgGetFieldsToLoad(String fgName, boolean onlyDirectRef) {
		// TODO Auto-generated method stub
		return null;
	}
	public Collection fgGetNestedFetchGroups(String fgName) {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean fgIsDefined(String fgName) {
		// TODO Auto-generated method stub
		return false;
	}

	public void setEncodedPName(Object encodedPName) {
		this.encodedPName = encodedPName;
	}

	public Object getEncodedPName() {
		return encodedPName;
	}

	public SpeedoProxy jdoDetachCopy(ProxyManager pm, Map map, Object clone, Collection fgHints) {
		if(clone == null){
			try{
				String proxyType = this.getClass().getName();
				//instanciate the SpeedoGenClassProxy clone
				clone = (SpeedoGenClassProxy) java.lang.Class.forName(proxyType).newInstance();
				SpeedoGenClassProxy spClone = (SpeedoGenClassProxy)clone;
				(spClone).setEncodedPName(getPName().encodeString());
				(spClone).setCeAge(0);
				(spClone).jdoSetProxyManagerFactory(null);
				(spClone).jdoSetTransactionalPersistenceManager(null);
				(spClone).jdoIsActive(false);
				//instanciate the clone of the fields
				GenClassAccessor fieldsClone = (GenClassAccessor) createAccessor();
				(spClone).setReferenceAccessor(fieldsClone);
				fieldsClone.jdoSetSpeedoProxy(spClone);
				//put the association between the speedoproxy and its clone into the map
				if(map != null)
					map.put(this, clone);
				synchronized(fgHints){
					//	call the method on the accessor
					((GenClassAccessor)getReferenceAccessor()).jdoDetachCopy(pm, map, fieldsClone, fgHints);
				}
				//mark the state as detached
				fieldsClone.setDetachedStatus(DetachedLifeCycle.DETACHED_CLEAN);
			}
			catch(Exception e){
				throw new JDOUserException("Problem while detaching an object from a collection: \n" + e);
			}
		}
		return (SpeedoProxy)clone;
	}

	public void jdoAttachCopy(ProxyManager pm, Map map, Object clone, SpeedoAccessor sa, boolean makeTransactional){
		if(map != null){
			if( (GenClass) map.get(clone) == this)
				return;
			map.put(clone, this);
			((GenClassAccessor) sa).jdoAttachCopy(pm, map, ((SpeedoProxy) clone).getReferenceAccessor(), makeTransactional);
		}
	}
	
	public void jdoRefresh(ProxyManager pm, Map map, Collection fgHints){
		if(map != null && !map.containsKey((SpeedoGenClassProxy)this)){
			map.put(this.getPName(), (SpeedoGenClassProxy)this);
			synchronized(fgHints){
				//	call the method on the accessor
				((GenClassAccessor)getReferenceAccessor()).jdoRefresh(pm, map, fgHints);
			}
		}
	}
	
	public boolean jdoIsDetachable() {
		// TODO Auto-generated method stub
		return false;
	}

		
    // IMPLEMENTATION OF THE CacheEntry INTERFACE //
    //-------------------------------------------//

    public Object getCeObject() {
        return this;
    }

    public Object getCeIdentifier() {
        return getPName();
    }


    // IMPLEMENTATION OF THE FixableCacheEntry INTERFACE //
    //---------------------------------------------------//

    public void fixCe() {
        fix++;
    }

    public void unfixCe() throws UnFixProtocolException {
		fix--;
    }

    public int getCeFixCount() {
        return fix;
    }

    // IMPLEMENTATION OF THE ReplaceableCacheEntry INTERFACE //
    //-------------------------------------------------------//

    public long getCeAge() {
        return age;
    }

    public void setCeAge(long l) {
        age = l;
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE Loggable INTERFACE
    // ------------------------------------------------------------------------

    public void setLogger(Logger _logger) {
        logger = _logger;
    }

    public void setLoggerFactory(LoggerFactory loggerfactory) {
        logger = loggerfactory.getLogger(this.getClass().getName());
    }

    public Logger getLogger() {
        return logger;
    }

    public LoggerFactory getLoggerFactory() {
        return null;
    }
}
