/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.jdo.JDOException;

import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.speedo.api.Debug;
import org.objectweb.speedo.genclass.GenClassAccessor;
import org.objectweb.speedo.genclass.GenClassElement;
import org.objectweb.speedo.genclass.PIndexedElemIterator;
import org.objectweb.speedo.genclass.api.SpeedoGenClassProxy;
import org.objectweb.speedo.mim.api.LifeCycle;
import org.objectweb.speedo.mim.api.SpeedoAccessor;
import org.objectweb.speedo.mim.api.SpeedoProxy;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.workingset.api.Transaction;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author	S.Chassande-Barrioz
 */
public class CollectionAccessor
        extends GenClassAccessor
        implements Collection {

	/**
	 * is the internal collection used when the gen class is not active.
	 * When the proxy is active the 'elements' field is used.
	 */
	protected Collection collection = null;


    /**
     * Instanciates and initializes a new collection with an initial size.
     */
    public CollectionAccessor(SpeedoGenClassProxy jdoProxy) {
        super(jdoProxy);
		collection = (Collection) jdoProxy.createGenClass();
    }

    public String toString() {
        return "CollectionAccessor: id=" + jdoProxy.jdoGetGenClassId();
    }

	protected boolean duplicatAllowed() {
		return true;
	}

	public GenClassElement speedoRemove2(Object o) {
		tmpelem.setElement(o);   // tmp type = etype.
		Iterator it = elements.iterator();
		while (it.hasNext()) {
			CollectionElem e = (CollectionElem) it.next();
			if (e.equals(tmpelem) && e.getElemStatus() != PIndexedElem.ELEM_DELETED) {
				e.setStatus(PIndexedElem.ELEM_DELETED);
				if (Debug.ON && getLogger() != null) {
					logger.log(BasicLevel.DEBUG, "speedoRemove2(): removed");
				}
				return e;
			}
		}
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "speedoRemove2(): not found");
		}
		return null;
	}

	public void jdoDeletePersistent(ProxyManager pm) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "jdoDeletePersistent");
		}
		clear();
	}

	public void setElements(Object o) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "setElements()");
		}
		clear();
		if (o != null) {
			addAll(((Collection) o));
		}
	}

	public void jdoLoadFieldsFromAccessor(SpeedoAccessor sa) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "jdoLoadFieldsFromAccessor()");
		}
		CollectionAccessor ca = (CollectionAccessor) sa;
		collection.clear();
		collection.addAll(ca.collection);
		supportDelta = ca.supportDelta;
		elements.clear();
		elements.addAll(ca.elements);
		jdoSetStatus(ca.jdoGetStatus());
	}

	public void jdoDetachCopy(ProxyManager pm, Map map, SpeedoAccessor fieldsClone, Collection fgHints){
		((CollectionAccessor)fieldsClone).jdoLoadFieldsFromAccessor(this);
		//copy the elements in the collection: collection is no more available when the proxy is not active 
		((CollectionAccessor)fieldsClone).collection.clear();
		//get an interator on the PIndexed elements
		Iterator it = ((GenClassAccessor) fieldsClone).elements.iterator();
		while(it.hasNext()){
			//get the PIndexElement
			CollectionElem iElem = (CollectionElem) it.next();
			//get the speedo proxy associated
			SpeedoProxy sp = (SpeedoProxy) iElem.getElement(pm);
			SpeedoProxy clone = null;
			if(map != null)
				clone = (SpeedoProxy) map.get(sp);  
			synchronized(fgHints){
				//get the clone of the speedo proxy	
				clone = (SpeedoProxy)  pm.speedoDetachCopy(sp, map, fgHints);
			}
			//add it in the collection of elements reachable when the proxy is no more active
			((CollectionAccessor)fieldsClone).collection.add(clone);
		}
		((GenClassAccessor) fieldsClone).jdoSetStatus(LifeCycle.TRANSIENT);
	}
	
	public void jdoAttachCopy(ProxyManager pm, Map map, SpeedoAccessor fieldsClone, boolean makeTransactional) {
		CollectionAccessor ca = (CollectionAccessor) fieldsClone;
		//get an iterator on the PIndexed elements of the collection
		Iterator it = ca.collection.iterator();
		while(it.hasNext()){
			//get the detached speedo proxy
			SpeedoProxy spDetached = (SpeedoProxy) it.next();
			SpeedoProxy sp = (SpeedoProxy) map.get((Object)spDetached);
			if(sp == null){
				map.put((Object) spDetached, sp);
				sp = (SpeedoProxy) pm.speedoAttachCopy(spDetached, makeTransactional, map);
			}
			/*
			//add only the dirty elements
			if(spDetached.getReferenceAccessor().getDetachedStatus() == DetachedLifeCycle.DETACHED_DIRTY){
				//put the element of the collection attribute in the elements attribute (the opposite of the jdoDetachCopy method)
				CollectionElem cElem = (CollectionElem) speedoAdd(sp, false);
				elements.add(cElem);
			}*/
		}
	}
	
	public void jdoRefresh(ProxyManager pm, Map map, Collection fgHints){
		//get an interator on the PIndexed elements
		Iterator it = elements.iterator();
		while(it.hasNext()){
			//get the PIndexElement
			CollectionElem iElem = (CollectionElem) it.next();
			//get the speedo proxy associated
			SpeedoProxy sp = (SpeedoProxy) iElem.getElement(pm);
			if(map != null && !map.containsKey(sp.getPName())){
				synchronized(fgHints){
					//refresh
					pm.speedoRefresh(sp, map, fgHints);
				}
			}
		}
	}
	
    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE Collection INTERFACE
    // ------------------------------------------------------------------------

    public boolean add(Object o) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "add()");
		}
		GenClassElement gce = speedoAdd(o, false);

        //Send the event about the new element adding
		if (gce != null) {
			Object _o = o;
			if (_o instanceof PName) {
				_o = gce.getElement(jdoProxy.jdoGetPersistenceManager());
			}
            if (_o instanceof SpeedoProxy) {
				jdoProxy.fireSpeedoElementAdded(_o);
			}
        }
        return gce != null;
    }

    public boolean addAll(Collection c) {
        Iterator it = c.iterator();
        boolean changed = false;
        while (it.hasNext()) {
            changed |= add(it.next());
        }
        return changed;
    }

    public void clear() {
        if (elements.size() > 0) {
			if (Debug.ON && getLogger() != null) {
				logger.log(BasicLevel.DEBUG, "clear()");
			}
            Collection c = new ArrayList(this);
            Iterator i = c.iterator();
            while (i.hasNext()) {
                remove(i.next());
            }
            elements.clear(); // efficient, but does not work with relations
            supportDelta = false;
        } else {
			if (Debug.ON && getLogger() != null) {
				logger.log(BasicLevel.DEBUG, "clear(): empty collection");
			}
		}
    }

    public boolean contains(Object o) {
        tmpelem.setElement(o);
        Iterator it = elements.iterator();
        while (it.hasNext()) {
            PIndexedElem e = (PIndexedElem) it.next();
            if (e.equals(tmpelem) && e.getElemStatus() != PIndexedElem.ELEM_DELETED) {
				if (Debug.ON && getLogger() != null) {
					logger.log(BasicLevel.DEBUG, "contains(): yes");
				}
                return true;
            }
        }
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "contains(): no");
		}
        return false;
    }

    public boolean containsAll(Collection c) {
        Iterator it = c.iterator();
        while (it.hasNext()) {
            if (!contains(it.next())) {
                return false;
            }
        }
        return true;
    }

    public boolean equals(Object o) {
        return (o instanceof Collection) &&
                ((Collection) o).size() == size() &&
                containsAll((Collection) o);
    }

    public boolean isEmpty() {
        if (elements.isEmpty()) {
            return true;
        }
        Iterator it = elements.iterator();
        // The set is empty if it contains only deleted indexed elements
        while (it.hasNext()) {
            if (((PIndexedElem) it.next()).getElemStatus() != PIndexedElem.ELEM_DELETED) {
                return false;
            }
        }
        return true;
    }

    public Iterator iterator() {
        return new PIndexedElemIterator(elements, this,
                jdoProxy.jdoGetPersistenceManager(), ((Loggable) jdoProxy).getLogger());
    }

    public boolean remove(Object o) {
        GenClassElement gce = speedoRemove2(o);
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "remove: gce=" + gce);
		}
        if (gce != null) {
			Object _o = gce.getElement(jdoProxy.jdoGetPersistenceManager());
            if (_o != null) {
            	jdoProxy.fireSpeedoElementRemoved(_o);
			}
        }
        return gce != null;
    }

    public boolean removeAll(Collection c) {
        Iterator it = c.iterator();
        boolean removed = true;
        while (it.hasNext()) {
            removed &= remove(it.next());
        }
        return removed;
    }

    public boolean retainAll(Collection c) {
        throw new UnsupportedOperationException();
    }

    public int size() {
        Iterator it = elements.iterator();
        int size = 0;
        while (it.hasNext()) {
            if (((PIndexedElem) it.next()).getElemStatus() != PIndexedElem.ELEM_DELETED) {
                size++;
            }
        }
        return size;
    }

    public Object[] toArray() {
        ArrayList al = new ArrayList(elements.size());
        Iterator it = elements.iterator();
        while (it.hasNext()) {
            GenClassElement gce = (GenClassElement) it.next();
            if (gce.getElemStatus() != PIndexedElem.ELEM_DELETED) {
                al.add(gce.getElement());
            }
        }
        return al.toArray();
    }

    public Object[] toArray(Object[] a) {
        ArrayList al = new ArrayList(elements.size());
        Iterator it = elements.iterator();
        while (it.hasNext()) {
            GenClassElement gce = (GenClassElement) it.next();
            if (gce.getElemStatus() != PIndexedElem.ELEM_DELETED) {
                al.add(gce.getElement());
            }
        }
        return al.toArray(a);
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE PGenClassAccessor INTERFACE
    // ------------------------------------------------------------------------

    public PIndexedElem createPIndexedElem(GenClassAccessor gca) {
        return new CollectionElem(gca);
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE SpeedoGenClassCoherence INTERFACE
    // ------------------------------------------------------------------------

    public boolean speedoAdd(Object o, Object hints) {
        return speedoAdd(o, false) != null;
    }

    protected GenClassElement speedoAdd(Object o, boolean allowDuplicates) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "speedoAdd(, " + allowDuplicates + ")");
		}
		if (o instanceof SpeedoProxy) {
			jdoProxy.jdoMakePersistent(null, (SpeedoProxy) o);
		}
		synchronized (this) {
			if (!allowDuplicates && contains(o)) {
				if (Debug.ON && getLogger() != null) {
					logger.log(BasicLevel.DEBUG, "speedoAdd(, " + allowDuplicates + "): already exist");
				}
				return null;
			}
			if (Debug.ON && getLogger() != null) {
				logger.log(BasicLevel.DEBUG, "speedoAdd(, " + allowDuplicates + "): adding");
			}
			// Put it in the "elements" list
			GenClassElement element = (GenClassElement) createPIndexedElem();
			element.setElement(o);
            return (elements.add(element) ? element : null);
        }
    }

    public boolean speedoRemove(Object o, Object hints) {
		if (Debug.ON && getLogger() != null) {
			logger.log(BasicLevel.DEBUG, "speedoRemove()");
		}
		return speedoRemove2(o) != null;
    }

}
