/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.collection;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.speedo.genclass.GenClassElement;
import org.objectweb.speedo.genclass.GenClassAccessor;
import org.objectweb.speedo.mim.api.SpeedoProxy;
import org.objectweb.speedo.mim.api.SpeedoAccessor;

import javax.jdo.PersistenceManager;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;

/**
 * @author	P. D?chamboux
 */
public class CollectionElem implements PIndexedElem, GenClassElement {

    /**
     * The error message thrown when an accessor method associated to a
     * primitive element type is called. Indeed this implementation supports
     * only object elements.
     */
    public final static String ERROR_MESSAGE_BAD_FIELD_NAME =
            "Wrong index field name";

    /**
     * The error message thrown when an accessor method associated to a
     * primitive element type is called. Indeed this implementation supports
     * only object elements.
     */
    public final static String ERROR_MESSAGE_NO_NULL_INDEX =
            "Null value not supported for index field";

    /**
     * This constant is the name of the index field. This value must be use with
     * the methods associated to the management of the index.
     */
    public final static String INDEX_FIELD_NAME = "idx";

    /**
     * This field is the element (or its PName for a reference).
     */
    protected Object element;

    /**
     * This field can be the index. The index type is known at instanciation
     * time.
     */
    protected Object index;

    /**
     * This field represents the satus of the PIndexedEleme. The possible value
     * are ELEM_CREATED, ELEM_DELETED, ELEM_MODIFIED, ELEM_UNMODIFIED
     */
    protected byte status = ELEM_CREATED;

    protected GenClassAccessor gca;

    public CollectionElem(GenClassAccessor gca) {
        this.gca = gca;
    }

    // IMPLEMENTATION OF THE GenClassElement INTERFACE //
    // ------------------------------------------------//

    public Object getIndex() {
        return index;
    }

    public void setIndex(Object index) {
        this.index = index;
    }

    /**
     * @return the element of the gen class. It is a user object.
     */
    public synchronized Object getElement() {
		return getElement(null);
    }

    /**
     * Assignes the element of the gen class. It is a user object.
     * @param element to add
     */
    public void setElement(Object element) {
        this.element = element;
    }

    /**
     * @param pm is the persistence manager which permits to resolve the PName
     * into a java reference.
     * @return the element of the gen class. The element is a reference
     * (SpeedoProxy).
     */
    public Object getElement(PersistenceManager pm) {
		Object res = element;
        if (res instanceof PName) {
			synchronized(this) {
				if (element instanceof PName) {
					if (pm == null) {
						pm = gca.jdoProxy.jdoGetPersistenceManager();
					}
            		element = pm.getObjectById(element, false);
				}
				res = element;
			}
        }
        return res;
    }

    public SpeedoAccessor getSpeedoAccessor() {
        return gca;
    }

	public void unSwizzle() {
		if (element instanceof SpeedoProxy) {
			synchronized(this) {
				if (element instanceof SpeedoProxy) {
					element = ((SpeedoProxy) element).getPName();
				}
			}
		}
	}

    // IMPLEMENTATION OF THE PIndexedElem INTERFACE //
    // ---------------------------------------------//

    public byte getElemStatus() {
        return status;
    }

    public String pieGetStringElem() throws PExceptionTyping {
        return (String) element;
    }

    public Date pieGetDateElem() throws PExceptionTyping {
        return (Date) element;
    }

    public char[] pieGetCharArrayElem() throws PExceptionTyping {
        return (char[]) element;
    }

    public byte[] pieGetByteArrayElem() throws PExceptionTyping {
        return (byte[]) element;
    }

    public Serializable pieGetSerializedElem() throws PExceptionTyping {
        return (Serializable) element;
    }

    public PName pieGetRefElem() {
        PName pn = (PName) (element instanceof SpeedoProxy
            ? ((SpeedoProxy) element).getPName()
            : element);
        try {
            pn = ((PNamingContext) gca.jdoGetSpeedoProxy().getPClassMapping()
                    .getPNameCoder()).export(null, pn);
        } catch (Exception e) {
        }
        return pn;
    }

    public boolean pieGetBooleanElem() throws PExceptionTyping {
        return ((Boolean) element).booleanValue();
    }

    public Boolean pieGetObooleanElem() throws PExceptionTyping {
        return (Boolean) element;
    }

    public byte pieGetByteElem() throws PExceptionTyping {
        return ((Byte) element).byteValue();
    }

    public Byte pieGetObyteElem() throws PExceptionTyping {
        return (Byte) element;
    }

    public char pieGetCharElem() throws PExceptionTyping {
        return ((Character) element).charValue();
    }

    public Character pieGetOcharElem() throws PExceptionTyping {
        return (Character) element;
    }

    public short pieGetShortElem() throws PExceptionTyping {
        return ((Short) element).shortValue();
    }

    public Short pieGetOshortElem() throws PExceptionTyping {
        return (Short) element;
    }

    public int pieGetIntElem() throws PExceptionTyping {
        return ((Integer) element).intValue();
    }

    public Integer pieGetOintElem() throws PExceptionTyping {
        return (Integer) element;
    }

    public long pieGetLongElem() throws PExceptionTyping {
        return ((Long) element).longValue();
    }

    public Long pieGetOlongElem() throws PExceptionTyping {
        return (Long) element;
    }

    public float pieGetFloatElem() throws PExceptionTyping {
        return ((Float) element).floatValue();
    }

    public Float pieGetOfloatElem() throws PExceptionTyping {
        return (Float) element;
    }

    public double pieGetDoubleElem() throws PExceptionTyping {
        return ((Double) element).doubleValue();
    }

    public Double pieGetOdoubleElem() throws PExceptionTyping {
        return (Double) element;
    }

    public BigDecimal pieGetBigDecimalElem() throws PException {
        return (BigDecimal) element;
    }

	public BigInteger pieGetBigIntegerElem() throws PException {
		return (BigInteger) element;
	}

    public void pieSetStringElem(String value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetDateElem(Date value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetCharArrayElem(char[] value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetByteArrayElem(byte[] value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetSerializedElem(Serializable value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetRefElem(PName value) throws PExceptionTyping {
        try {
            element = value.resolve(null);
        } catch (Exception e) {
        }
    }

    public void pieSetBooleanElem(boolean value) throws PExceptionTyping {
        element = new Boolean(value);
    }

    public void pieSetObooleanElem(Boolean value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetByteElem(byte value) throws PExceptionTyping {
        element = new Byte(value);
    }

    public void pieSetObyteElem(Byte value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetCharElem(char value) throws PExceptionTyping {
        element = new Character(value);
    }

    public void pieSetOcharElem(Character value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetShortElem(short value) throws PExceptionTyping {
        element = new Short(value);
    }

    public void pieSetOshortElem(Short value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetIntElem(int value) throws PExceptionTyping {
        element = new Integer(value);
    }

    public void pieSetOintElem(Integer value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetLongElem(long value) throws PExceptionTyping {
        element = new Long(value);
    }

    public void pieSetOlongElem(Long value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetFloatElem(float value) throws PExceptionTyping {
        element = new Float(value);
    }

    public void pieSetOfloatElem(Float value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetDoubleElem(double value) throws PExceptionTyping {
        element = new Double(value);
    }

    public void pieSetOdoubleElem(Double value) throws PExceptionTyping {
        element = value;
    }

    public void pieSetBigDecimalElem(BigDecimal bigDecimal) throws PException {
        element = bigDecimal;
    }

	public void pieSetBigIntegerElem(BigInteger value) throws PException {
		element = value;
	}
    // ---------------------------------------------------------------------------
    // Index accessor
    // ---------------------------------------------------------------------------

    public void pieSetByteIndexField(String fn, byte value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        index = new Byte(value);
    }

    public void pieSetObyteIndexField(String fn, Byte value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        if (value == null)
            throw new PExceptionTyping(ERROR_MESSAGE_NO_NULL_INDEX);
        index = value;
    }

    public void pieSetCharIndexField(String fn, char value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        index = new Character(value);
    }

    public void pieSetOcharIndexField(String fn, Character value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        if (value == null)
            throw new PExceptionTyping(ERROR_MESSAGE_NO_NULL_INDEX);
        index = value;
    }

    public void pieSetShortIndexField(String fn, short value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        index = new Short(value);
    }

    public void pieSetOshortIndexField(String fn, Short value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        if (value == null)
            throw new PExceptionTyping(ERROR_MESSAGE_NO_NULL_INDEX);
        index = value;
    }

    public void pieSetIntIndexField(String fn, int value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        index = new Integer(value);
    }

    public void pieSetOintIndexField(String fn, Integer value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        if (value == null)
            throw new PExceptionTyping(ERROR_MESSAGE_NO_NULL_INDEX);
        index = value;
    }

    public void pieSetLongIndexField(String fn, long value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        index = new Long(value);
    }

    public void pieSetOlongIndexField(String fn, Long value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        if (value == null)
            throw new PExceptionTyping(ERROR_MESSAGE_NO_NULL_INDEX);
        index = value;
    }

    public void pieSetStringIndexField(String fn, String value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        if (value == null)
            throw new PExceptionTyping(ERROR_MESSAGE_NO_NULL_INDEX);
        index = value;
    }

    public void pieSetDateIndexField(String fn, Date value) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        if (value == null)
            throw new PExceptionTyping(ERROR_MESSAGE_NO_NULL_INDEX);
        index = value;
    }

    public short pieGetShortIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return ((Short) index).shortValue();
    }

    public Short pieGetOshortIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return (Short) index;
    }

    public long pieGetLongIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return ((Long) index).longValue();
    }

    public Long pieGetOlongIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return (Long) index;
    }

    public int pieGetIntIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return ((Integer) index).intValue();
    }

    public Integer pieGetOintIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return (Integer) index;
    }

    public String pieGetStringIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return (String) index;
    }

    public Date pieGetDateIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return (Date) index;
    }

    public byte pieGetByteIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return ((Byte) index).byteValue();
    }

    public Byte pieGetObyteIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return (Byte) index;
    }

    public char pieGetCharIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return ((Character) index).charValue();
    }

    public Character pieGetOcharIndexField(String fn) throws PExceptionTyping {
        if (!INDEX_FIELD_NAME.equals(fn))
            throw new PExceptionTyping(ERROR_MESSAGE_BAD_FIELD_NAME);
        return (Character) index;
    }

    // ---------------------------------------------------------------------------
    // Other methods
    // ---------------------------------------------------------------------------

    public void setStatus(byte s) {
        status = s;
    }

    public boolean equals(Object o) {
		if (!(o instanceof CollectionElem)) {
			return false;
		}
		//Store the element value in a local variable in order to avoid
		// syncrhonization
		Object _element = element;
		Object ceElem =  ((CollectionElem) o).getElement();
		if (_element == null) {
			return ceElem == null;
		}
		if (ceElem == null) {
			return false;
		}
		if (_element instanceof PName) {
			if (ceElem instanceof PName) {
				return _element.equals(ceElem);
			} else if (ceElem instanceof SpeedoProxy) {
				return _element.equals(((SpeedoProxy) ceElem).getPName());
			} else {
				return false;
			}
		} else if (_element instanceof SpeedoProxy) {
			if (ceElem instanceof PName) {
				return ((SpeedoProxy) _element).getPName().equals(ceElem);
			} else if (ceElem instanceof SpeedoProxy) {
				return _element.equals(ceElem);
			} else {
				return false;
			}
		} else {
			return  _element.equals(ceElem);
		}
    }
}
