/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.collection;

import org.objectweb.speedo.mim.api.SpeedoAccessor;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.genclass.GenClass;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * This class is an implementation of the java.util.Collection interface.
 *
 * @author	Eric.Bruneton, S.Chassande-Barrioz
 */
public class CollectionImpl extends GenClass implements Collection {

    protected final static int DEFAULT_SIZE = -1;

	CollectionAccessor accessor;

    /**
     * Instanciates and initializes a new collection with an initial size.
     */
    public CollectionImpl() {
        super();
		accessor = (CollectionAccessor) createAccessor();
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE Collection INTERFACE
    // ------------------------------------------------------------------------

    public boolean add(Object o) {
        boolean result;
        if (!jdoIsActive) {
            result = accessor.collection.add(o);
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoWriteIntention();
            result = ca.add(o);
        }
        return result;
    }

    public boolean addAll(Collection c) {
        boolean result;
        if (!jdoIsActive) {
            result = accessor.collection.addAll(c);
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoWriteIntention();
            result = ca.addAll(c);
        }
        return result;
    }

    public void clear() {
        if (!jdoIsActive) {
            accessor.collection.clear();
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoWriteIntention();
            ca.clear();
        }
    }

    public boolean contains(Object o) {
        if (!jdoIsActive) {
            return accessor.collection.contains(o);
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoReadIntention();
            return ca.contains(o);
        }
    }

    public boolean containsAll(Collection c) {
        if (!jdoIsActive) {
            return accessor.collection.containsAll(c);
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoReadIntention();
            return ca.containsAll(c);
        }
    }

    public boolean equals(Object o) {
        if (!jdoIsActive) {
            return accessor.collection.equals(o);
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoReadIntention();
            return ca.equals(o);
        }
    }

    public boolean isEmpty() {
        if (!jdoIsActive) {
            return accessor.collection.isEmpty();
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoReadIntention();
            return ca.isEmpty();
        }
    }

    public Iterator iterator() {
        if (!jdoIsActive) {
            return accessor.collection.iterator();
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoReadIntention();
            return ca.iterator();
        }
    }

    public boolean remove(Object o) {
        boolean result;
        if (!jdoIsActive) {
            result = accessor.collection.remove(o);
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoWriteIntention();
            result = ca.remove(o);
        }
        return result;
    }

    public boolean removeAll(Collection c) {
        boolean result;
        if (!jdoIsActive) {
            result = accessor.collection.removeAll(c);
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoWriteIntention();
            result = ca.removeAll(c);
        }
        return result;
    }

    public boolean retainAll(Collection c) {
        throw new UnsupportedOperationException();
    }

    public int size() {
        if (!jdoIsActive) {
            return accessor.collection.size();
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoReadIntention();
            return ca.size();
        }
    }

    public Object[] toArray() {
        if (!jdoIsActive) {
            return accessor.collection.toArray();
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoReadIntention();
            return ca.toArray();
        }
    }

    public Object[] toArray(Object[] a) {
        if (!jdoIsActive) {
            return accessor.collection.toArray(a);
        } else {
			CollectionAccessor ca = (CollectionAccessor) jdoReadIntention();
            return ca.toArray(a);
        }
    }

    // IMPLEMENTATION OF THE SpeedoProxy INTERFACE //
    //---------------------------------------------//

    public void jdoMakePersistent(ProxyManager pm, SpeedoAccessor sa) {
        if (accessor.collection != null) {
            jdoMakePersistent(pm,accessor.collection.iterator());
        }
    }

    // OTHER METHODS //
    // --------------//

    public SpeedoAccessor createAccessor() {
        return new CollectionAccessor(this);
    }

    public Object createGenClass() {
        return new ArrayList();
    }

	public SpeedoAccessor getReferenceAccessor() {
		return accessor;
	}
	public void setReferenceAccessor(SpeedoAccessor refAcc) {
		accessor = (CollectionAccessor) refAcc;
	}



}
