/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.map;

import org.objectweb.speedo.genclass.api.SpeedoGenClassProxy;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.mim.api.SpeedoAccessor;

import java.util.TreeMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Collection;
import java.util.Comparator;
import java.util.SortedMap;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class TreeMapImpl extends TreeMap
        implements SpeedoGenClassProxy {

	public TreeMapImpl() {
	    super();
		accessor = (MapAccessor) createAccessor();
	}

	public String toString() {
	    return "";
	}

	public abstract void jdoMakePersistent(ProxyManager pm, Iterator it);

	MapAccessor accessor;

	public SpeedoAccessor getReferenceAccessor() {
		return accessor;
	}
	public void setReferenceAccessor(SpeedoAccessor refAcc) {
		accessor = (MapAccessor) refAcc;
	}

	public Object createGenClass() {
	    return new TreeMap();
	}

	public SpeedoAccessor createAccessor() {
	    return new MapAccessor(this);
	}

	public void jdoMakePersistent(ProxyManager pm, SpeedoAccessor sa) {
	    if (size() > 0) {
	        jdoMakePersistent(pm, values().iterator());
	    }
	}

	// IMPLEMENTS THE HashSet CLASS //
	//------------------------------//

	public int size() {
	    if (!jdoIsActive()) {
	        return accessor.size();
	    } else {
			MapAccessor ma = (MapAccessor) jdoReadIntention();
	        return ma.size();
	    }
	}

	public boolean isEmpty() {
	    if (!jdoIsActive()) {
	        return accessor.isEmpty();
	    } else {
			MapAccessor ma = (MapAccessor) jdoReadIntention();
	        return ma.isEmpty();
	    }
	}

	public boolean containsValue(Object o) {
	    if (!jdoIsActive()) {
	        return accessor.containsValue(o);
	    } else {
			MapAccessor ma = (MapAccessor) jdoReadIntention();
	        return ma.containsValue(o);
	    }
	}

	public synchronized boolean containsKey(Object o) {
	    if (!jdoIsActive()) {
	        return accessor.containsKey(o);
	    } else {
			MapAccessor ma = (MapAccessor) jdoReadIntention();
	        return ma.containsKey(o);
	    }
	}

	public synchronized Object get(Object o) {
	    if (!jdoIsActive()) {
	        return accessor.get(o);
	    } else {
			MapAccessor ma = (MapAccessor) jdoReadIntention();
	        return ma.get(o);
	    }
	}

	public synchronized Object put(Object o, Object o1) {
	    if (!jdoIsActive()) {
	        return accessor.put(o, o1);
	    } else {
			MapAccessor ma = (MapAccessor) jdoWriteIntention();
	        return ma.put(o, o1);
	    }
	}

	public synchronized Object remove(Object o) {
	    if (!jdoIsActive()) {
	        return accessor.remove(o);
	    } else {
			MapAccessor ma = (MapAccessor) jdoWriteIntention();
	        return ma.remove(o);
	    }
	}

	public synchronized void putAll(Map map) {
	    if (!jdoIsActive()) {
	        accessor.putAll(map);
	    } else {
			MapAccessor ma = (MapAccessor) jdoWriteIntention();
	        ma.putAll(map);
	    }
	}

	public synchronized void clear() {
	    if (!jdoIsActive()) {
	        accessor.clear();
	    } else {
			MapAccessor ma = (MapAccessor) jdoWriteIntention();
	        ma.clear();
	    }
	}

	public Set keySet() {
	    if (!jdoIsActive()) {
	        return accessor.keySet();
	    } else {
			MapAccessor ma = (MapAccessor) jdoReadIntention();
	        return ma.keySet();
	    }
	}

	public Set entrySet() {
	    if (!jdoIsActive()) {
	        return accessor.entrySet();
	    } else {
			MapAccessor ma = (MapAccessor) jdoReadIntention();
	        return ma.entrySet();
	    }
	}

	public Collection values() {
	    if (!jdoIsActive()) {
	        return accessor.values();
	    } else {
			MapAccessor ma = (MapAccessor) jdoReadIntention();
	        return ma.values();
	    }
	}

	public Comparator comparator() {
		throw new UnsupportedOperationException();
	}

	public Object firstKey() {
		throw new UnsupportedOperationException();
	}

	public Object lastKey() {
		throw new UnsupportedOperationException();
	}

	public Object clone() {
		throw new UnsupportedOperationException();
	}

	public SortedMap subMap(Object fromKey, Object toKey) {
		throw new UnsupportedOperationException();
	}

	public SortedMap headMap(Object toKey) {
		throw new UnsupportedOperationException();
	}

	public SortedMap tailMap(Object fromKey) {
		throw new UnsupportedOperationException();
	}

}
