/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.jca;

import org.objectweb.speedo.pm.api.ProxyManager;
import javax.transaction.xa.Xid;

/**
 * It represents a XA context. An XAContext is linked to a XA transaction
 * represented by a XID. In a transaction the same ProxyManager is used.
 *
 * @author S.Chassande-Barrioz
 */
public class XAContext {
	/**
	 * Is a possible status of the XAContext.
	 * It represents the initial state when the XAContext is not yet linked to a
	 * transaction. Then the PM must not be used in this state.
	 */
	public final static byte UNKNOWN = -1;

	/**
	 * Is a possible status of the XAContext.
	 * The start() method has been called in order to demarcate the begin of the
	 * PM use.
	 */
	public final static byte STARTED = 1;

	/**
	 * Is a possible status of the XAContext.
	 * The end() method has been called in order to demarcate the end of the
	 * PM use.
	 */
	public final static byte ENDED = 2;

	/**
	 * Is a possible status of the XAContext.
	 * The prepare() method has been called in order to launch the first step
	 * of the two phase commit. The PM must not be used in this state.
	 */
	public final static byte PREPARED = 3;

	/**
	 * The XID of the transaction
	 */
	public Xid xid = null;

	/**
	 * Is the ProxyManager to use in the XAContext (~transaction)
	 */
	public ProxyManager pm = null;

	/**
	 * is the status of the XAContext. The possible values are UNKNOWN, STARTED,
	 * ENDED or PREPARED.
	 */
	public byte status = UNKNOWN;

	/**
	 * This boolean indicates if the used PM has been registered as a
	 * java.transaction.Synchronization on the transaction.
	 */
	public boolean synchroRegistred = false;

	public XAContext(Xid xid) {
		this.xid = xid;
	}
}
