/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.mapper.api;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * defines the attributes (Fractal attribute Controller) of the JormFactory
 * component.
 *
 * @see  org.objectweb.speedo.mapper.api.JormFactory
 * @author S.Chassande-Barrioz
 */
public interface JormFactoryAttributes extends AttributeController {

	/**
	 * nothing is done. Then the mapping structures already exist at the Speedo
	 * starting
	 */
	byte DO_NOTHING = 0;

	/**
	 * The mapping structures are created if they are not already present
	 */
	byte CREATE_IF_REQUIRED = 1;

	/**
	 * The mapping structures are removed (if they already exist), and created.
	 */
	byte FORCE_CREATE = 2;

	/**
	 * The data present in the mapping structure are removed. Of course if the
	 * mapping structure do not exist, they are created.
	 */
	byte DELETE_DATA = 3;

	/**
	 * @return the default mapping structure rule applied during the map
	 * operation of the persistent classes.
	 */
	byte getMappingStructureRule();

	/**
	 *
	 * @param rule is the default mapping structure rule to apply during the map
	 * operation of the persistent classes.
	 */
	void setMappingStructureRule(byte rule);

	/**
	 * Indicates if during the class registration, the persistent classes have
	 * to be mapped.
	 * True : the class are mapped at class registration time
	 * False: nothing is done.
	 */
	boolean getMappingOnClassRegistration();

	/**
	 *
	 * @param val is a boolean value Indicating if during the class
	 * registration, the persistent classes have to be mapped.
	 * True : the class are mapped at class registration time
	 * False: nothing is done.
	 */
	void setMappingOnClassRegistration(boolean val);
}
