/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.mapper.lib;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PMapperListener;
import org.objectweb.jorm.api.PMapCluster;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;

import javax.jdo.JDOException;
import java.util.Collection;

/**
 * This class is an implementation of the PMapper interface delegating all
 * method to another PMapper, except for the lookup method. In this case the
 * lookup method is done by the use of the JormFactory. The aim is to always
 * have the PClassMapping instance. But only the JormFactory is able to build
 * the PClassMapping instance of a Persistent class.
 *
 * @author S.Chassande-Barrioz
 */
public class DelegatePMapper implements PMapper {

    private JormFactory jf;
    private PMapper mapper;
    private ClassLoader classLoader;

    public DelegatePMapper(PMapper mapper) {
        this.mapper = mapper;
    }

	public JormFactory getJormFactory() {
		return jf;
	}

	public void setJormFactory(JormFactory jf) {
		this.jf = jf;
	}

	public PMapper getMapper() {
		return mapper;
	}

	public void setMapper(PMapper mapper) {
		this.mapper = mapper;
	}

	public ClassLoader getClassLoader() {
		return classLoader;
	}

	public void setClassLoader(ClassLoader classLoader) {
		this.classLoader = classLoader;
	}
    // IMPLEMENTATION OF THE PMapper INTERFACE //

    public PClassMapping lookup(String s) {
        try {
            return jf.getPClassMapping(s, classLoader);
        } catch (PException e) {
            throw new JDOException(
                    "Impossible to initialize the persistent class '"  + s
                    + "': ", ExceptionHelper.getNested(e));
        }
    }

    public void setPMapper(PMapper pm) {
        mapper.setPMapper(pm);
    }

    public void setLogger(Logger l) {
        mapper.setLogger(l);
    }

    public void setPrefetchCache(PrefetchCache prefetchCache) throws PException {
        mapper.setPrefetchCache(prefetchCache);
    }
    public PrefetchCache getPrefetchCache() {
        return mapper.getPrefetchCache();
    }
    public void closeConnection(Object o) throws PException {
        mapper.closeConnection(o);
    }
    public String cn2mn(String s) {
        return mapper.cn2mn(s);
    }
    public Object getConnection() throws PException {
        return mapper.getConnection();
    }
    public Object getConnection(Object o) throws PException {
        return mapper.getConnection(o);
    }
	public Object getConnection(Object connectionContext, Object user) throws PException {
		return mapper.getConnection(connectionContext, user);
	}
    public String getMapperName() {
        return mapper.getMapperName();
    }
    public PMappingStructuresManager getPMappingStructuresManager() {
        return mapper.getPMappingStructuresManager();
    }
    public void map(PClassMapping pClassMapping) throws PException {
        mapper.map(pClassMapping);
    }
    public void map(Object o, PClassMapping pClassMapping) throws PException {
        mapper.map(o, pClassMapping);
    }
    public void map(Object o, PClassMapping pClassMapping, boolean b) throws PException {
        mapper.map(o, pClassMapping, b);
    }
    public void setConnectionFactory(Object o) throws PException {
        mapper.setConnectionFactory(o);
    }
    public Object getConnectionFactory() {
        return mapper.getConnectionFactory();
    }
    public void setMapperName(String s) {
        mapper.setMapperName(s);
    }
    public Manager getMetaInfoManager() {
        return mapper.getMetaInfoManager();
    }
    public PTypeSpace getPTypeSpace() {
        return mapper.getPTypeSpace();
    }
    public void start() throws PException {
        mapper.start();
    }
    public void stop() throws PException {
        mapper.stop();
    }
    public void unmap(String s) throws PException {
        mapper.unmap(s);
    }
    public void addMapperEventListener(PMapperListener pMapperListener) {
        mapper.addMapperEventListener(pMapperListener);
    }
    public void removeMapperEventListener(PMapperListener pMapperListener) {
        mapper.removeMapperEventListener(pMapperListener);
    }
    public PClassMapping createGenClassMapping() throws PException {
        return mapper.createGenClassMapping();
    }
    public PMapCluster getPMapCluster(String s) throws PException {
        return mapper.getPMapCluster(s);
    }
    public Collection getPMapClusters() {
        return mapper.getPMapClusters();
    }
    public void addDependency(String s, String s1) throws PException {
        mapper.addDependency(s, s1);
    }
    public void classDefined(String s) throws PException {
        mapper.classDefined(s);
    }

	public void declareClass(String jcname) {
		mapper.declareClass(jcname);
	}
}

