/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.metadata;

import java.util.ArrayList;
import java.util.List;

/**
 * Description of all elements which can contain an extension.
 * @author	S.Chassande-Barrioz
 */
public abstract class SpeedoElement {
    /**
     * The extension corresponding to the element.
     */
    public List jdoExtension = new ArrayList();

    public String getExtensionValueByKey(String key) {
	    SpeedoExtension se = getExtensionByKey(key);
	    return (se == null ? null : se.value);
    }

    public SpeedoExtension getExtensionByKey(String key) {
        if (jdoExtension == null || key == null) {
            return null;
        }
        for(int i=0; i<jdoExtension.size(); i++) {
            SpeedoExtension se = (SpeedoExtension) jdoExtension.get(i);
            if (key.equals(se.key)) {
                return se;
            }
        }
        return null;
    }
    public SpeedoExtension getExtension(String vendor, String key) {
        if (jdoExtension == null || vendor == null || key == null) {
            return null;
        }
        for(int i=0; i<jdoExtension.size(); i++) {
            SpeedoExtension se = (SpeedoExtension) jdoExtension.get(i);
            if (vendor.equals(se.vendorName) && key.equals(se.key)) {
                return se;
            }
        }
        return null;
    }

    public void addExtension(SpeedoExtension se) {
        jdoExtension.add(se);
    }
}
