/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.metadata;

import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.util.monolog.api.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * This class describes all persistence capable classes.
 * @author	S.Chassande-Barrioz
 */
public class SpeedoXMLDescriptor {
    /**
     * List of packages descriptions which contains persistence capable classes.
     */
    public Map jdoPackage = new HashMap();

	public SpeedoMetaInfo smi;

    public String xmlFile = null;

	public boolean requireEnhancement = true;

	public SpeedoXMLDescriptor(SpeedoMetaInfo smi) {
		this.smi = smi;
	}

    /**
     * Transform this object into a String.
     * @return the string corresponding to the object.
     */
    public String toString() {
        Enumeration e = Collections.enumeration(jdoPackage.values());
        String s = "descriptor ";
        while (e.hasMoreElements()) {
            s = s + "  " + e.nextElement().toString();
        }
        return s;
    }

    /**
     * Adds a package descriptor to the global descriptor.
     * Throws an exception if there are persistent capable fields defined twice.
     * @param pac package descriptor to add.
     * @param failsOnError if an error provoques an exception or a warning message.
     * @param logger logger where to put warning message.
     * @exception SpeedoException If a persistent field of the package was already
     * described in the global descriptor.
     */
    public void add(Object pac, boolean failsOnError, Logger logger) throws SpeedoException {
        SpeedoPackage p = (SpeedoPackage) pac;
        //the package already exixts in the descriptor.
        if (jdoPackage.containsKey(p.name)) {
            //we add all its classes
            Enumeration e = Collections.enumeration(p.jdoClass.values());
            SpeedoPackage pref = (SpeedoPackage) jdoPackage.get(p.name);
            while (e.hasMoreElements()) {
                pref.add(e.nextElement(), failsOnError, logger);
            }
        }
        //we add the entire package
        else {
            p.jdoXMLDescriptor = this;
            jdoPackage.put(p.name, pac);
        }
    }

    public List getSpeedoClasses() {
        List scs = new ArrayList();
        Iterator it = jdoPackage.values().iterator();
        while(it.hasNext()) {
            scs.addAll(((SpeedoPackage) it.next()).jdoClass.values());
        }
        return scs;
    }

    public SpeedoClass getSpeedoClass(String fqn, boolean other) {
	    //System.out.println("XML.getSpeedoClass(" + fqn + ", " + other + "): XML=" + xmlFile);
	    SpeedoClass sc = null;
	    if (other) {
		    sc = smi.getSpeedoClass(fqn, this);
	    }
	    if (sc != null) {
		    //System.out.println("XML.getSpeedoClass(): return " + sc);
		    return sc;
	    }
	    SpeedoPackage sp = (SpeedoPackage) jdoPackage.get(NamingRules.packageName(fqn));
	    if (sp != null) {
	        sc = (SpeedoClass) sp.jdoClass.get(NamingRules.className(fqn));
	    }
	    //System.out.println("XML.getSpeedoClass(): return " + sc);
	    return sc;
    }
}
