/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.mim.api;

import java.util.Collection;
/**
 * @author Y.Bersihand
 */
public interface FetchPlan {
	
	public static final String DEFAULT = "default";
	public static final String ALL = "all";
	public static final String VALUES = "values";
	public static final String NONE = "none";
	
	/**
	 * Add the fetchgroup to the set of active fetch groups
	 */
	FetchPlan addGroup(String fetchGroupName);
	
	/**
	 *  Remove the fetch group from the set active fetch groups. 
	 */
	FetchPlan removeGroup(String fetchGroupName); 
	
	/** 
	 * Remove all active groups and activate the default fetch group. 
	 */ 
	FetchPlan resetGroups(); 
	
	/**
	 *  Return the names of all active fetch groups. 
	 */ 
	Collection getGroups(); 
	
	/** 
	 * Set a collection of groups. 
	 */ 
	FetchPlan setGroups(Collection fetchGroupNames); 
	
	/** 
	 * Set the fetch size for large result set support. Use 0 to unset. 
	 */ 
	FetchPlan setFetchSize(int fetchSize); 
	
	/** 
	 * Return the fetch size, or 0 if not set. 
	 */ 
	int getFetchSize();

}
