/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.mim.api;

import org.objectweb.perseus.persistence.api.State;
import org.objectweb.jorm.api.PAccessor;
import java.io.Serializable;
import java.util.Collection;

/**
 * This interface a state of an user instance.
 *
 * @author S.Chassande-Barrioz
 */
public interface SpeedoAccessor extends PAccessor, State, Serializable {

    /**
     * Changes the status of this object
     *
     * @see org.objectweb.speedo.mim.api.LifeCycle
     * @param action the action that may change the status
     */
    void jdoChangeStatus(byte action);

    /**
     * Gets the current status of this object
     *
     * @see org.objectweb.speedo.mim.api.LifeCycle
     * @return the current state in the life cycle
     */
    byte jdoGetStatus();

    /**
     * Forces the new value of the status
     *
     * @see org.objectweb.speedo.mim.api.LifeCycle
     * @param newValue the new status of this object
     */
    void jdoSetStatus(byte newValue);

	/**
	 * Version management is performed via the use of timestamps.
	 */
	long getTimeStamp();
	
	void setTimeStamp(long ts);

	/**
	   * Gets the current status of this detached object
	   *
	   * @return the current state in the life cycle
	   */
	  byte getDetachedStatus();

	  /**
	   * Forces the new value of the detached status
	   *
	   * @param newValue the new status of this object
	   */
	  void setDetachedStatus(byte newValue);
	  
    /**
     * @return The SpeedoProxy attached to this state representation.
     */
    SpeedoProxy jdoGetSpeedoProxy();

    /**
     * Assignes the SpeedoProxy attached to this state representation.
     */
    void jdoSetSpeedoProxy(SpeedoProxy sp);

    /**
     * Prepare to write (relations consistency management)
     */
    void prepareWrite();

    /**
     * The context has been closed (ex: transaction end).
     */
    void workingSetClosed();

	/**
	 * indicates if the object has been flushed on the support in a writing
	 * before the end of the working set (commit | rollback | close).
	 */
	boolean jdoHasBeenFlush();

	/**
	 * Assignates a boolean value to indicate if the object has been flushed
	 * on the support in a writing before the end of the working
	 * set (commit | rollback | close).
	 * @param val is the boolean which must be assigned.
	 */
	void jdoSetFlushed(boolean val);  
	
}

