/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.naming.lib;

import org.objectweb.asm.Type;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.CommonClassMapping;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.NameRef;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.jorm.JormMIMappingBuilder;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.naming.api.MIBuilderHelper;
import org.objectweb.speedo.naming.api.NamingManager;
import org.objectweb.speedo.naming.api.UserId;
import org.objectweb.speedo.naming.api.UserIdFactory;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Properties;

import javax.jdo.JDOUserException;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class UserIdCompositeNamingManager
	extends NamingManagerHelper
	implements NamingManager {

	private final static String COMPOSITE_USER_ID = "cuid";

	protected String getName() {
		return COMPOSITE_USER_ID;
	}

    // IMPLEMENTATION OF THE METHOD FROM THE NamingManager INTERFACE //
    //---------------------------------------------------------------//
    public boolean canManage(SpeedoClass sc) {
        if (sc.identityType == SpeedoIdentity.USER_ID) {
            ArrayList idFields = new ArrayList();
            Iterator fieldsIt = sc.jdoField.values().iterator();
            while (fieldsIt.hasNext()) {
                SpeedoField sp = (SpeedoField) fieldsIt.next();
                if (sp.primaryKey) {
                    idFields.add(sp);
                }
            }
            String objectidClass = sc.objectidClass;
            if (sc.generateObjectId()) {
                objectidClass = NamingRules.generatedObjectIdName(sc.getFQName());
            }
            return objectidClass != null && objectidClass.length() > 0;
        }
        return false;
    }

    public Object encode(PName pn) throws PException {
        if (pn instanceof UserIdFactory) {
            return ((UserIdFactory) pn).getNewUserId();
        }
        return null;
    }

    public PName decode(PNameCoder pnc,
                        Object oid,
                        Class clazz,
                        JormFactory jf) throws PException {
        if (oid instanceof UserId) {
            //the oid is a UserId
            if (pnc != null) {
                //Use the pnc found by the clazz
                return pnc.decodeAbstract(oid, null);
            } else if (((UserId) oid).jdoGetPersistentClassName() != null) {
                //No pnc but the class name is inside the UserId
                ClassLoader cl = oid.getClass().getClassLoader();
                if (cl == null) {
                    cl = jf.getClass().getClassLoader();
                }
                if (cl == null) {
                    cl = ClassLoader.getSystemClassLoader();
                }
                return jf.getPNamingContext(
                        ((UserId) oid).jdoGetPersistentClassName(), cl)
                        .decodeAbstract(oid, null);
            }
        } else if (pnc != null
                && (oid instanceof String)
                && (pnc.getNull() instanceof UserIdFactory)) {
            //The oid is a String which must be the parameter of the constructor
            Class idclass = ((UserIdFactory) pnc.getNull())
                    .getNewUserId().getClass();
            Object o = null;
            try {
                o = idclass.getConstructor(new Class[]{String.class})
                        .newInstance(new Object[]{(String) oid});
            } catch (Exception e) {
                throw new JDOUserException("No constructor with a String " +
                        "parameter available on the identifier class: "
                        + idclass.getName());
            }
            //recall with the UserId
            return decode(pnc, o, clazz, jf);
        }

        return null;
    }

    public boolean canProvidePBinder(Object hints, ClassLoader classLoader) {
		if(!super.canProvidePBinder(hints, classLoader)) {
			return false;
		}
		String cn = getBinderClassNameFromHints(hints, COMPOSITE_USER_ID);
		if (cn == null) {
			return false;
		}
        try {
            classLoader.loadClass(cn);
            return true;
        } catch (ClassNotFoundException e) {
            return false;
        }
    }

    public boolean canProvidePNamingContext(Object hints, ClassLoader classLoader) {
		if(!super.canProvidePNamingContext(hints, classLoader)) {
			return false;
		}
		String cn = getPNCClassNameFromHints(hints, COMPOSITE_USER_ID);
		if (cn == null) {
			return false;
		}
		try {
			classLoader.loadClass(getPNCClassNameFromHints(hints, COMPOSITE_USER_ID));
			return true;
		} catch (ClassNotFoundException e) {
			return false;
		}
    }

    public void fillNameDef(MIBuilderHelper mibh,
                            Manager manager,
                            NameDef nd,
                            SpeedoClass tsc,
                            SpeedoClass ssc,
                            MetaObject mo,
                            Reference ref,
                            CommonClassMapping hcm,
                            JormMIMappingBuilder mb,
                            boolean isIdentifier,
                            boolean isInGenClass,
                            boolean createField,
                            Collection createdMOs) throws SpeedoException, PException {
		boolean debug = logger.isLoggable(BasicLevel.DEBUG);
		if (debug) {
			logger.log(BasicLevel.DEBUG, "CompositeUserId: Filling the name def: "
				+ "\n\tsource class=" + (ssc == null ? null  : ssc.getFQName())
				+ "\n\ttarget class=" + (tsc == null ? null  : tsc.getFQName())
				+ "\n\tmo=" + mo
				+ "\n\treference=" + ref
				+ "\n\tisIdentifier=" + isIdentifier
				+ "\n\tisInGenClass=" + isInGenClass
				+ "\n\tcreateField=" + createField);
		}
		SpeedoClass clazzWithIdField = tsc;
		while(clazzWithIdField.superClassName != null) {
			clazzWithIdField = clazzWithIdField.getSpeedoClassFromContext(
				clazzWithIdField.superClassName);
		}
        ArrayList idFields = new ArrayList();
        Iterator fieldsIt = clazzWithIdField.jdoField.values().iterator();
        while (fieldsIt.hasNext()) {
            SpeedoField sf = (SpeedoField) fieldsIt.next();
            if (sf.primaryKey) {
                idFields.add(sf);
            }
        }
        String objectidClass = clazzWithIdField.objectidClass;
        if (objectidClass == null && tsc.generateObjectId()) {
            objectidClass = NamingRules.generatedObjectIdName(clazzWithIdField.getFQName());
        }
        //The user has specified an object id ==> use it for the namedef
        CompositeName cn = manager.getCompositeName(objectidClass);
        boolean cnnotdefined = (cn == null);
        if (cnnotdefined) {
			if (debug) {
				logger.log(BasicLevel.DEBUG, "\tCreate the composite name " + objectidClass);
			}
            cn = manager.createCompositeName(objectidClass);
            createdMOs.add(cn);
        }
        NameRef nr = nd.createNameRef(cn);
        for (int i = 0; i < idFields.size(); i++) {
            SpeedoField sf = (SpeedoField) idFields.get(i);
            int size = PType.NOSIZE;
            SpeedoExtension se = sf.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.SIZE);
            if (se != null) {
                try {
                    size = Integer.parseInt(se.value);
                } catch (NumberFormatException e) {
                    //The error will occur during the parsing of the primitive
                    // field in the referenced class
                }
            }
            int scale = PType.NOSIZE;
            se = sf.getExtension(SpeedoProperties.VENDOR_NAME, SpeedoProperties.SCALE);
            if (se != null) {
                try {
                    scale = Integer.parseInt(se.value);
                } catch (NumberFormatException e) {
                    //The error will occur during the parsing of the primitive
                    // field in the referenced class
                }
            }
            if (cnnotdefined) {
                cn.createCompositeNameField(sf.name,
                        mibh.getPrimitivePType(Type.getType(sf.desc)),
                        size, scale);
            }
            String fn = mibh.getNameDefFieldPrefix(
                    ref, isIdentifier, isInGenClass) + sf.name;
			if (debug) {
				logger.log(BasicLevel.DEBUG, "\tField name " + fn);
			}
            if (createField) {
                fn = mibh.createNameDefField(mo, fn,
                        mibh.getPrimitivePType(Type.getType(sf.desc)), size, scale).getName();
            }
            nr.addProjection(sf.name, fn);
        }
        if (createField && (mb != null)) {
            mb.createNameDefMapping(hcm, nd, ssc, isIdentifier, isInGenClass);
        }
    }

    public String getPNameHints(SpeedoClass sc, NameDef nd) {
        return "this.jdoReferenceState";
    }

	public String getGCPNameHints(SpeedoClass sc, NameDef nd) {
		return "proxy.getPName()";
	}

	/**
	 * Build a property value compsed such as this pattern
	 * userid,binder_class_name,pnc_class_name,class_name
	 * @param nd
	 * @param targetClass
	 * @param sourceMO
	 * @param key
	 * @param result
	 */
	public void getJormNamingConfig(NameDef nd,
									SpeedoClass targetClass,
									MetaObject sourceMO,
									String key,
									Properties result) {
		StringBuffer sb = new StringBuffer();
		sb.append(COMPOSITE_USER_ID);
		sb.append(HINTS_SEP);
		String binderClassName = NamingRules.binderName(
			nd.getNameRef().getCompositeName().getFQName());
		sb.append(binderClassName);
		sb.append(HINTS_SEP);
		String className = targetClass.getFQName();
		SpeedoClass ancestor = targetClass.getAncestor();
		if (sourceMO instanceof GenClassRef //For a Genclass ref or identifier
			|| (ancestor == null //No inheritance
				&& targetClass.jormclass.getSubClasses().isEmpty())) {
			sb.append(binderClassName);
		} else {
			if (ancestor == null) {
				ancestor = targetClass;
			}
			sb.append(NamingRules.kfpncName(ancestor.getFQName()));
			className = ancestor.getFQName();
		}
		sb.append(HINTS_SEP);
		sb.append(className);
		result.setProperty(key, sb.toString());
    }
/*
    public NamingManager.NamingField[] getNamingfields(SpeedoClass sc) {
		ArrayList nfs = new ArrayList();
		for(Iterator it = sc.jdoField.values().iterator(); it.hasNext();) {
			SpeedoField sf = (SpeedoField) it.next();
			if (sf.primaryKey) {

				try {
					Class c = Util.getClass(Type.getType(sf.desc),
										Thread.currentThread().getContextClassLoader());
//TODO: calculate the ptype from the class
					PType pt = null;
					nfs.add(new NamingField(sf.name, c, sf.name, pt));
				} catch (Exception e) {
					logger.log(BasicLevel.
				}
			}
		}
        return nfs;
    }
*/

}
