/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.pm.api;


import java.util.Collection;

/**
 * This interfaces defines a ProxyManager switch. It binds a thread to a
 * list of ProxyManager. In a same context two proxy managers cannot be managed
 * by the same PersistentManagerFactory.
 *
 * @author S.Chassande-Barrioz
 */
public interface ProxyManagerSwitch {

    /**
     * @param pmf is persistent manager factory which manages the returned
     * proxy manager.
     * @return the ProxyManager managed by the given persistence manager factory
     * and bound to current the context, or the null value if there is no
     * ProxyManager.
     */
    ProxyManager lookup(ProxyManagerFactory pmf);

    /**
     * It assignes a ProxyManager to the current context.
     * @param pm is the ProxyManager
     */
    void bind(ProxyManager pm);

    /**
     * It clears the list of ProxyManager for the current context.
     */
    void clear();

    /**
     * It clears a ProxyManager for the current context.
	 * @return true if the specified pm has been unbound from the context/thread
     */
    boolean unbind(ProxyManager pm);

	/**
	 * It clears the ProxyManager associated to a ProxyManagerFactory for the
	 * current context
	 * @return true if a pm associated to the specified PMF, has been unbound
	 * from the context/thread
	 */
    boolean unbind(ProxyManagerFactory pmf);

    /**
     * @return all ProxyManager instances bound with the current context
     */
    Collection entries();
}
