/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.query.api;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.pm.api.ProxyManager;

import java.util.Collection;
import java.util.Map;

/**
 * Represents a compiled JDO query.
 *
 * @author S.Chassande-Barrioz
 */
public interface CompiledQuery {

    short UNDEFINED = 0;
    short DEFINED = 1;
    short COMPILED = 2;

	/**
	 * @return the query status. The possible values are UNDEFINED, DEFINED 
	 * or COMPILED.
	 */
	short getStatus();

	/**
	 * @return the definition of the query if it is defined, null value
	 * otherwise.
	 */
	QueryDefinition getDefinition();

    /**
	 * Compiles the query if it is not already done.
	 */
	void compile() throws SpeedoException, MedorException, ExpressionException;

    /**
	 * Executes the compiled query.
	 * @param parameters is the JDO parameters of query (does not contains
	 * internal parameter). The order of parameter must match the parameter
	 * declaration.
	 * @param pm is the proxy manager representing the execution context. It
	 * also used to load the instance in the result collection.
	 * @return a collection of persistent instance. The collection can be empty
	 * but is never null.
	 */
	Collection execute(Object[] parameters, ProxyManager pm)
            throws SpeedoException, MedorException, ExpressionException;

    Collection execute(Map parameters, ProxyManager  pm)
            throws SpeedoException, MedorException, ExpressionException;

}
