/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.query.api;

import javax.jdo.Query;

/**
 *
 * @author S.Chassande-Barrioz
 */
public interface JDOQuery extends Query {

	/**
	 * The application may want to skip some number of results that may have
	 * been previously returned, and additionally may want to limit the number
	 * of instances returned from a query. The default range for query execution
	 * if this method is not called are (0, Long.MAX_VALUE).
	 * @param first
	 * @param last
	 */
	void setRange(long first, long last);

	/**
	 * If the application knows that there can be exactly zero or one instance
	 * returned from a query, the result of the query is most conveniently
	 * returned as an instance (possibily null) instead of a Collection.
	 * @param unique
	 */
	void setUnique(boolean unique);

	/**
	 * The application wmight want to get results from a query that are not
	 * instance of the candidate class. The results might be fields of
	 * persistent instances, instances of classes other than the candidate
	 * class, or aggregates of fields.
	 *
	 * @param result
	 */
	void setResult(String result);

	/**
	 * The application may have a user-defined class that best represents the
	 * results of a query. In this case, the application can specify that
	 * instances of this class should be returned.
	 *
	 * @param resultClass
	 */
	void setResultClass(Class resultClass);
}
