/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.wizard.ant.batch;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.MatchingTask;
import org.apache.tools.ant.types.DTDLocation;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;
import org.objectweb.jorm.api.PException;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.generation.api.SpeedoCompilerParameter;
import org.objectweb.speedo.wizard.SpeedoWizardCompiler;

import java.io.File;
import java.util.ArrayList;

public class Antjdo2jorm extends Task {

    private Path classpath = null;
    private Path jormclasspath = null;

    // Provides the absolute path for .jdo files
    private Antjdo2jorm.Description jdo = null;

    // Provides the absolute path for the JORM files
    private Antjdo2jorm.Description jorm = null;

    // Provides the absolute path for the .class files
    private File binaryDir = null;

    // Provides the absolute path for the generated .pd files
    private File outputDir = null;

    // Makes sense when java files must be compiled
    // not used here.
    /*
    private File src = null;
    */

    private SpeedoCompilerParameter scp = null;
    private SpeedoWizardCompiler wc = null;

    public void init() throws BuildException {
        super.init();
        wc = new SpeedoWizardCompiler();
        scp = wc.getSpeedoCompilerParameter();
    }

    /**
	 * @deprecated
	 */ 
	public void setConffile(File jormconfFile) {
    }

    public void setLogPropFile(File logPropFile) {
        scp.logPropFile = logPropFile.getAbsolutePath();
    }

    public void setProjectname(String projectName) {
        scp.projectName = projectName;
    }

    public void setMappername(String mapperName) {
        scp.mapperName = mapperName;
    }

    /*
    public void setSrc(File src) {
        this.src = src;
    }
    */

    public void setKeepsrc(boolean keepsrc) {
        scp.keepsrc = keepsrc;
    }


    /**
     * Create a DTD location record. This stores the location of a DTD. The
     * DTD is identified by its public Id. The location may either be a file
     * location or a resource location.
     *
     * @return the DTD location object to be configured by Ant
     */
    public DTDLocation createDTD() {
        DTDLocation dtdLocation = new DTDLocation();
        scp.dtdLocations.add(dtdLocation);
        return dtdLocation;
    }

    // Provides the absolute path for the generated .pd files
    public void setOutputdir(File outputDir) {
        scp.pdoutput = outputDir.getAbsolutePath();
    }

    public void setBinarydir(File binaryDir) {
        this.binaryDir = binaryDir;
        scp.output = binaryDir.getAbsolutePath();
    }

/*
    public void setInput(File intput) {
        scp.input = intput.getAbsolutePath();
    }
*/
    public void setClasspath(Path cp) {
        if (classpath == null)
            classpath = cp;
        else
            classpath.append(cp);
    }

    public Path getClasspath() {
        return classpath;
    }

    public Path createClasspath() {
        if (classpath == null)
            classpath = new Path(getProject());
        return classpath.createPath();
    }

    public void setClasspathRef(Reference r) {
        createClasspath().setRefid(r);
    }

    public Antjdo2jorm.Description createJdopath() {
        Antjdo2jorm.Description d = new Antjdo2jorm.Description();
        this.jdo = d;
        d.setProject(getProject());
        return d;
    }

    public Antjdo2jorm.Description createJormpath() {
        Antjdo2jorm.Description d = new Antjdo2jorm.Description();
        this.jorm = d;
        d.setProject(getProject());
        return d;
    }

    public void setJormClasspath(Path cp) {
        if (jormclasspath == null)
            jormclasspath = cp;
        else
            jormclasspath.append(cp);
    }

    public Path getJormClasspath() {
        return jormclasspath;
    }

    public Path createJormClasspath() {
        if (jormclasspath == null)
            jormclasspath = new Path(getProject());
        return jormclasspath.createPath();
    }

    public void setJormClasspathRef(Reference r) {
        createJormClasspath().setRefid(r);
    }

    /**
     * Main method of the task executed by ant.
     * Parses xml parameters, loads AntSpeedoExec with its own loader
     */
    public void execute() throws BuildException {

       if (jdo == null) {
        log("Please, specify the root path for the jdo files.", Project.MSG_INFO);
           return;
        //    jdo = createJdopath();
        //    jdo.setDir(src);
        //    //jdo.setIncludes("**/*.jdo");
        }
        // Root path for jdo descriptor files.
        scp.jdoDir = jdo.dir.getAbsolutePath();
        scp.jdo = new MyArrayList(
                jdo.getDirectoryScanner(jdo.dir).getIncludedFiles());
        log("jdoDir " + scp.jdoDir, Project.MSG_INFO);
        // Root path for existing jorm descriptor files.
        if (jorm != null) {
            scp.jormDir = jorm.dir.getAbsolutePath();
            scp.jorm = new MyArrayList(
                jorm.getDirectoryScanner(jorm.dir).getIncludedFiles());
            log("jormDir " + scp.jormDir, Project.MSG_INFO);
        }
        if (scp.dtdLocations.size() == 0) {
            DTDLocation dl = new DTDLocation();
            dl.setPublicId("-//ObjectWeb Consortium//DTD JORM DEFINITIONS 2.0//EN");
            dl.setLocation("jorm2.dtd");
            scp.dtdLocations.add(dl);
            dl = new DTDLocation();
            dl.setPublicId("-//Sun Microsystems, Inc.//DTD Java Data Objects Metadata 1.0//EN");
            dl.setLocation("jdo.dtd");
            scp.dtdLocations.add(dl);
        }
        if (jormclasspath == null) {
            jormclasspath = classpath;
        }
        scp.jormclasspath = new MyArrayList(jormclasspath.list());
        scp.classpath = classpath;
        log("scp.jormclasspath " + scp.jormclasspath, Project.MSG_DEBUG);

        if (scp.input == null || scp.input.length()==0) {
            scp.input = scp.output;
        }
        try {
            wc.init();
            wc.process();
        } catch (SpeedoException e) {
            e.printStackTrace();
            throw new BuildException(getNestedException(e));
        }
    }

    private Exception getNestedException(Exception e) {
        if (e instanceof SpeedoException
                && ((SpeedoException) e).getNestedException() != null) {
            return getNestedException(((SpeedoException) e).getNestedException());
        } else if (e instanceof PException
                && ((PException) e).getNestedException() != null) {
            return getNestedException(((PException) e).getNestedException());
        } else
            return e;
    }

    /**
     * Task used to parse nested <code>jdopath</code> or <code>jormpath</code> elements.
     */
    public class Description extends MatchingTask {

        public File dir = null;

        public void setDir(File dir) {
            this.dir = dir;
        }

        public DirectoryScanner getDirectoryScanner(File p) {
            return super.getDirectoryScanner(p);
        }
    }

}

class MyArrayList extends ArrayList {
    public MyArrayList(Object[] os) {
        super(os.length);
        for (int i = 0; i < os.length; i++)
            add(os[i]);
    }
}
