/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.runtime.basic;

import javax.naming.spi.InitialContextFactory;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.Name;
import javax.naming.NamingEnumeration;
import javax.naming.NameParser;
import java.util.Hashtable;
import java.util.Map;
import java.util.HashMap;

/**
 * @author S.Chassande-Barrioz
 */
public class SpeedoInitialNamingFactory implements InitialContextFactory {
    static Context ctx = new SpeedoContext(new HashMap());

    public Context getInitialContext(Hashtable environment)
        throws NamingException {
        return ctx;
    }
}
class SpeedoContext implements Context {
    Map ctx;

    public SpeedoContext(Map ctx) {
        this.ctx = ctx;
    }

    public Object lookup(Name name) throws NamingException {
        Object o = ctx.get(name.get(0));
        if (o instanceof Context) {
            return ((Context) o).lookup(name.getSuffix(1));
        } else {
            return o;
        }
    }

    public Object lookup(String name) throws NamingException {
        return ctx.get(name);
    }

    public void bind(Name name, Object obj) throws NamingException {
        if (name.size() > 1) {
            ((Context)ctx.get(name.get(0))).rebind(name.getSuffix(1), obj);
        } else {
            bind(name.get(0), obj);
        }
    }

    public void bind(String name, Object obj) throws NamingException {
        if (ctx.containsKey(name)) {
            throw new NamingException(name + " already bound");
        }
        ctx.put(name, obj);
    }

    public void rebind(Name name, Object obj) throws NamingException {
        if (name.size() > 1) {
            ((Context)ctx.get(name.get(0))).rebind(name.getSuffix(1), obj);
        } else {
            ctx.put(name.get(0), obj);
        }
    }

    public void rebind(String name, Object obj) throws NamingException {
        ctx.put(name, obj);
    }

    public void unbind(Name name) throws NamingException {
        if (name.size() > 1) {
            ((Context)ctx.get(name.get(0))).unbind(name.getSuffix(1));
        } else {
            unbind(name.get(0));
        }
    }

    public void unbind(String name) throws NamingException {
        ctx.remove(name);
    }

    public void rename(Name oldName, Name newName) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public void rename(String oldName, String newName) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public NamingEnumeration list(Name name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public NamingEnumeration list(String name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public NamingEnumeration listBindings(Name name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public NamingEnumeration listBindings(String name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public void destroySubcontext(Name name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public void destroySubcontext(String name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public Context createSubcontext(Name name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public Context createSubcontext(String name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public Object lookupLink(Name name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public Object lookupLink(String name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public NameParser getNameParser(Name name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public NameParser getNameParser(String name) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public Name composeName(Name name, Name prefix) throws NamingException {
        throw new UnsupportedOperationException();
    }

    public String composeName(String name, String prefix)
        throws NamingException {
        throw new UnsupportedOperationException();
    }

    public Object addToEnvironment(String propName, Object propVal)
        throws NamingException {
        throw new UnsupportedOperationException();
    }

    public Object removeFromEnvironment(String propName)
        throws NamingException {
        throw new UnsupportedOperationException();
    }

    public Hashtable getEnvironment() throws NamingException {
        throw new UnsupportedOperationException();
    }

    public void close() throws NamingException {
    }

    public String getNameInNamespace() throws NamingException {
        throw new UnsupportedOperationException();
    }
}

