/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.basic;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.jdo.JDOException;
import javax.jdo.JDOUserException;
import javax.jdo.PersistenceManager;
import javax.jdo.JDOHelper;

import junit.framework.Assert;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.pobjects.basic.BasicA;
import org.objectweb.speedo.pobjects.basic.BasicType;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestBasicA extends SpeedoTestHelper {

	public TestBasicA(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.basic.TestBasicA";
	}

	public void testgetPMF() {
		assertTrue("Not the same PMF",
			pmf == JDOHelper.getPersistenceManagerFactory(getPMFProperties()));
	}
	public void testCreateInTxAndRemoveInTx2PM() {
		logger.log(BasicLevel.DEBUG, "testCreateInTxAndRemoveInTx2PM");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateInTxAndRemoveInTx2PM");
		ba.writeF2(2);
		Locale l = new Locale("en"); //Examples: "en", "de_DE", "_GB", "en_US_WIN", "de__POSIX", "fr__MAC"
		ba.setLocale(l);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(ba);
		Object id = pm.getObjectId(ba);
		Assert.assertNotNull("Null object identifier", id);
		assertEquals(1, ba.getCheckJdoStore());
		assertEquals(1, ba.getCheckJdoLoad());
		assertEquals(1, ba.getCheckJdoClear());		
		assertEquals(1, ba.getCheckJdoDelete());
		assertEquals("Bad locale language value", l.getLanguage(), ba.getLocale().getLanguage());
		assertEquals("Bad locale country value", l.getCountry(), ba.getLocale().getCountry());
		assertEquals("Bad locale variant value", l.getVariant(), ba.getLocale().getVariant());
		pm.currentTransaction().commit();
		assertEquals(2, ba.getCheckJdoStore());
		pm.close();

		ba = null;

		pm = pmf.getPersistenceManager();
		pm.evictAll();
		pm.currentTransaction().begin();
		ba = (BasicA) pm.getObjectById(id, true);
		Assert.assertNotNull("Object not found", ba);
		Assert.assertEquals("Bad f1 value", "testCreateInTxAndRemoveInTx2PM", ba.readF1());
		Assert.assertEquals("Bad f1 value", 2, ba.readF2());
		assertEquals("Bad locale language value", l.getLanguage(), ba.getLocale().getLanguage());
		assertEquals("Bad locale country value", l.getCountry(), ba.getLocale().getCountry());
		assertEquals("Bad locale variant value", l.getVariant(), ba.getLocale().getVariant());
		assertEquals(2, ba.getCheckJdoClear());
		assertEquals(1, ba.getCheckJdoDelete());
		pm.deletePersistent(ba);
		assertEquals(2, ba.getCheckJdoDelete());
		pm.currentTransaction().commit();

		try {
			Object o = pm.getObjectById(id, true);
			if (o != null)
				fail("Removed object always availlable: " + o);
		} catch (JDOUserException e) {
			//OK
		} catch (Exception e) {
			e.printStackTrace();
			fail("Bad exception thrown when an object does not exist: " + e);
		}

		pm.close();
	}

	public void testCreateAndRemoveInTx2PM() {
		logger.log(BasicLevel.DEBUG, "testCreateAndRemoveInTx2PM");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateAndRemoveInTx2PM");
		ba.writeF2(2);
		Locale l = new Locale("de_DE"); //Examples: "en", "de_DE", "_GB", "en_US_WIN", "de__POSIX", "fr__MAC"
		ba.setLocale(l);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.makePersistent(ba);
		Object id = pm.getObjectId(ba);
		Assert.assertNotNull("Null object identifier", id);
		pm.close();

		ba = null;

		pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		ba = (BasicA) pm.getObjectById(id, true);
		Assert.assertNotNull("Object not found", ba);
		Assert.assertEquals("Bad f1 value", "testCreateAndRemoveInTx2PM", ba.readF1());
		Assert.assertEquals("Bad f1 value", 2, ba.readF2());
		assertEquals("Bad locale language value", l.getLanguage(), ba.getLocale().getLanguage());
		assertEquals("Bad locale country value", l.getCountry(), ba.getLocale().getCountry());
		assertEquals("Bad locale variant value", l.getVariant(), ba.getLocale().getVariant());
		
		pm.deletePersistent(ba);
		pm.currentTransaction().commit();
		pm.close();
	}


	public void testCreateAndRemoveInTx1PM() {
		logger.log(BasicLevel.DEBUG, "testCreateAndRemoveInTx1PM");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateAndRemoveInTx1PM");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.makePersistent(ba);
		pm.currentTransaction().begin();
		pm.deletePersistent(ba);
		pm.currentTransaction().commit();
		pm.close();
	}

	public void testCreateAndRemoveInSameTx1PM() {
		logger.log(BasicLevel.DEBUG, "testCreateAndRemoveInSameTx1PM");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateAndRemoveInSameTx1PM");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(ba);
		pm.deletePersistent(ba);
		try {
			pm.currentTransaction().commit();
		} catch (JDOException e) {
			logger.log(BasicLevel.ERROR, "testCreateAndRemoveInSameTx1PM", ExceptionHelper.getNested(e));
			fail("Jorm does not support export following by an unexport");
		}
		pm.close();
	}

	public void testCreateInTxAndRemoveInTx1PM() {
		logger.log(BasicLevel.DEBUG, "testCreateInTxAndRemove1PM");
		BasicA ba = new BasicA();
		ba.writeF1("testCreateInTxAndRemove1PM");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(ba);
        pm.currentTransaction().commit();		
		pm.currentTransaction().begin();
		pm.deletePersistent(ba);
        pm.currentTransaction().commit();
        try {
            pm.close();
        } catch (JDOException e) {
            logger.log(BasicLevel.ERROR, "testCreateInTxAndRemove1PM", ExceptionHelper.getNested(e));
            fail("Speedo does not support the use of a PM after a transaction commit");
        }
	}

    public void testTransientAfterTx() {
        logger.log(BasicLevel.DEBUG, "testTransientAfterTx");
        BasicA ba = new BasicA();
        ba.writeF1("testTransient");
        ba.writeF2(2);
        Locale l = new Locale("_GB"); //Examples: "en", "de_DE", "_GB", "en_US_WIN", "de__POSIX", "fr__MAC"
        ba.setLocale(l);
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();
        pm.makePersistent(ba);
        pm.currentTransaction().commit();
        Object oid = pm.getObjectId(ba);
        try {
            pm.makeTransient(ba);
            pm.close();
            Assert.assertEquals("Bad f1 value", "testTransient", ba.readF1());
            assertEquals("Bad locale language value", l.getLanguage(), ba.getLocale().getLanguage());
            assertEquals("Bad locale country value", l.getCountry(), ba.getLocale().getCountry());
            assertEquals("Bad locale variant value", l.getVariant(), ba.getLocale().getVariant());
            ba  = null;
        } catch (JDOException e) {
            logger.log(BasicLevel.ERROR, "testTransient", ExceptionHelper.getNested(e));
            fail("Speedo does not support the makeTransient operation after a Tx");
        } finally {
            pm = pmf.getPersistenceManager();
            ba = (BasicA) pm.getObjectById(oid, false);
            pm.currentTransaction().begin();
            pm.deletePersistent(ba);
            pm.currentTransaction().commit();
            pm.close();
        }
    }

    public void testTransientNoTx() {
        logger.log(BasicLevel.DEBUG, "testTransientNoTx");
        BasicA ba = new BasicA();
        ba.writeF1("testTransient");
        ba.writeF2(2);
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();
        pm.makePersistent(ba);
        pm.currentTransaction().commit();
        Object oid = pm.getObjectId(ba);
        pm.close();
        ba = null;
        pm = pmf.getPersistenceManager();
        try {
            ba = (BasicA) pm.getObjectById(oid, false);
            pm.makeTransient(ba);
            pm.close();
            Assert.assertEquals("Bad f1 value", "testTransient", ba.readF1());
            ba  = null;
        } catch (JDOException e) {
            logger.log(BasicLevel.ERROR, "testTransientNoTx", ExceptionHelper.getNested(e));
            fail("Speedo does not support the makeTransient operation (notx)");
        } finally {
            pm = pmf.getPersistenceManager();
            ba = (BasicA) pm.getObjectById(oid, false);
            pm.currentTransaction().begin();
            pm.deletePersistent(ba);
            pm.currentTransaction().commit();
            pm.close();
        }
    }

	public void testImbricatedPM() {
		logger.log(BasicLevel.DEBUG, "testImbricatedPM");
		BasicA ba = new BasicA();
		ba.writeF1("testImbricatedPM");
		ba.writeF2(2);
		PersistenceManager pm1 = pmf.getPersistenceManager();
		PersistenceManager pm2 = pmf.getPersistenceManager();
        Assert.assertTrue("Same persistence manager", pm1!=pm2);
		pm2.makePersistent(ba);
        pm2.close();
		//touch the pm
		pm1.getUserObject();
        ba.readF1();
        ba.readF2();
        pm1.currentTransaction().begin();
		pm1.deletePersistent(ba);
		pm1.currentTransaction().commit();
        try {
            pm1.close();
        } catch (JDOException e) {
            logger.log(BasicLevel.ERROR, "testImbricatedPM", ExceptionHelper.getNested(e));
            fail("Speedo does not support the use of a PM after a transaction commit");
        }
	}

	public void testRollBackTxSamePM() {
		logger.log(BasicLevel.DEBUG, "testRollBackTxSamePM");
		BasicA ba = new BasicA();
		ba.writeF1("testRollBack1");
		ba.writeF2(2);
		PersistenceManager pm = pmf.getPersistenceManager();

        pm.currentTransaction().begin();
		pm.makePersistent(ba);
        pm.currentTransaction().commit();

        pm.currentTransaction().begin();
		ba.writeF2(3);
        Assert.assertEquals("Bad f2 value before roolback", 3, ba.readF2());
        pm.currentTransaction().rollback();
        Assert.assertEquals("Bad f2 value after roolback", 2, ba.readF2());

        pm.currentTransaction().begin();
        pm.deletePersistent(ba);
        pm.currentTransaction().commit();
        pm.close();
	}

    public void testRollBack2() {
        logger.log(BasicLevel.DEBUG, "testRollBack2");
        BasicA ba = new BasicA();
        ba.writeF1("testRollBack1");
        ba.writeF2(2);
        PersistenceManager pm = pmf.getPersistenceManager();

        pm.currentTransaction().begin();
        pm.makePersistent(ba);
        pm.currentTransaction().commit();

        pm.currentTransaction().begin();
        ba.writeF2(3);
        Assert.assertEquals("Bad f2 value before roolback", 3, ba.readF2());
        pm.currentTransaction().rollback();
        Assert.assertEquals("Bad f2 value after roolback", 2, ba.readF2());
        ba.writeF2(4);
        Assert.assertEquals("Bad f2 value after roolback", 4, ba.readF2());
        pm.close();

        pm = pmf.getPersistenceManager();
        Assert.assertEquals("Bad f2 value after roolback", 4, ba.readF2());
        
        pm.currentTransaction().begin();
        pm.deletePersistent(ba);
        pm.currentTransaction().commit();
        pm.close();
    }

    public void testBasicType(long longval) {
        logger.log(BasicLevel.DEBUG, "testBasicType" + longval);
        BasicType bt = new BasicType();
	    bt.assign(longval);
        PersistenceManager pm = pmf.getPersistenceManager();

        pm.currentTransaction().begin();
        pm.makePersistent(bt);
	    Object oid = pm.getObjectId(bt);
        pm.currentTransaction().commit();

	    bt = null;
	    pm.evictAll();

        pm.currentTransaction().begin();
	    bt = (BasicType) pm.getObjectById(oid, false);
	    Assert.assertNotNull("bt is null", bt);
		bt.check(longval);
	    pm.deletePersistent(bt);
	    pm.currentTransaction().commit();
        pm.close();
    }
    public void testBasicType35() {
	    testBasicType(35);
    }
    public void _testBasicType0() {
	    testBasicType(0);
    }

	public void testSerialization() {
		logger.log(BasicLevel.DEBUG, "testSerialization");
		BasicA ba = new BasicA();
		ba.writeF1("testSerialization");
		ba.writeF2(5);
		Locale l = new Locale("en_US_WIN"); //Examples: "en", "de_DE", "_GB", "en_US_WIN", "de__POSIX", "fr__MAC"
		ba.setLocale(l);
		PersistenceManager pm = pmf.getPersistenceManager();

		pm.currentTransaction().begin();
		pm.makePersistent(ba);
		Object oid = pm.getObjectId(ba);
		pm.currentTransaction().commit();
		pm.close();
		File f = new File(new File(new File("output"),"test"), "testSerialization");
		if (f.exists()) {
			f.delete();
		}
		try {
			ObjectOutputStream oos = new ObjectOutputStream(
			    new FileOutputStream(f));
			oos.writeObject(ba);
			oos.close();

			ba = null;
			ObjectInputStream ois = new ObjectInputStream(
			    new FileInputStream(f));
			ba = (BasicA) ois.readObject();
			ois.close();
			ba.readF1_F2();
			assertEquals("Bad locale language value", l.getLanguage(), ba.getLocale().getLanguage());
			assertEquals("Bad locale country value", l.getCountry(), ba.getLocale().getCountry());
			assertEquals("Bad locale variant value", l.getVariant(), ba.getLocale().getVariant());
			
		} catch (Exception e) {
			logger.log(BasicLevel.ERROR, "testSerialization fails: ", e);
			fail(e.getMessage());
		} finally {
			pm = pmf.getPersistenceManager();
			pm.currentTransaction().begin();
			pm.deletePersistent(pm.getObjectById(oid, false));
			pm.currentTransaction().commit();
			pm.close();
		}
	}

	public void testStaticFinder() {
		logger.log(BasicLevel.DEBUG, "testStaticFinder");
		ArrayList al = new ArrayList();
		for(int i =0; i<50; i++) {
			BasicA ba = new BasicA();
			ba.writeF1("testStaticFinder" + i);
			ba.writeF2(i);
			al.add(ba);
		}
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.makePersistentAll(al);
		pm.close();
		List l = BasicA.getAllBasicA(pmf);
		pm = pmf.getPersistenceManager();
		try {
			assertSameCollection("bad query result", al, l);
		} finally {
			pm.currentTransaction().begin();
			pm.deletePersistentAll(al);
			pm.currentTransaction().commit();
			pm.close();
		}
	}

	public void testNonNullField() {
		logger.log(BasicLevel.DEBUG, "testNonNullField");
		BasicA ba = new BasicA();
		ba.writeF1("testNonNullField");
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(ba);
		pm.currentTransaction().commit();
		pm.currentTransaction().begin();
		ba.setNonullField(null);
		try {
			pm.currentTransaction().commit();
			fail("no exception thrown when a null value try to be store for a persistent field marked as non null");
		} catch (JDOUserException e) {
		} catch (Exception e) {
			fail("Bad exception thrown when a null value try to be store for a persistent field marked as non null: " + e.getClass());
		} finally {
			pm.currentTransaction().begin();
			pm.deletePersistent(ba);
			pm.currentTransaction().commit();
			pm.close();
		}
	}
}
