/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.concurrency;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pobjects.basic.BasicA;

import javax.jdo.PersistenceManager;

import junit.framework.Assert;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestSeveralPMUsers extends SpeedoTestHelper {

    public TestSeveralPMUsers(String s) {
        super(s);
    }

    protected String getLoggerName() {
        return LOG_NAME + ".rt.concurrency.TestSeveralPMUsers";
    }

    public void test2thread1PMNoTx() {
        final PersistenceManager pm = pmf.getPersistenceManager();
        BasicA ba = new BasicA();
        ba.writeF1("test2thread1PMNoTx.thread1");
        ba.writeF2(2);
        pm.makePersistent(ba);
        final Object oid = pm.getObjectId(ba);
        Thread t = new Thread(
                new Runnable() {
                    public void run() {
                        try {
                            BasicA _ba = ((BasicA) pm.getObjectById(oid, false));
                            _ba.readF1_F2();
                            _ba.writeF1("test2thread1PMNoTx.thread2");
                            Assert.assertTrue(true);
                        } catch (Exception e) {
                            e.printStackTrace();
                            fail(e.getMessage());
                        }
                    }
                });
        t.start();
        try {
            t.join();
        } catch (InterruptedException e) {
        }
        Assert.assertTrue("The thread2 is not finished", !t.isAlive());
        pm.close();

    }


    public void test2thread1PMTx() {
        final PersistenceManager pm = pmf.getPersistenceManager();
        BasicA ba = new BasicA();
        ba.writeF1("test2thread1PM.thread1");
        ba.writeF2(1);
        pm.currentTransaction().begin();
        pm.makePersistent(ba);
        final Object oid = pm.getObjectId(ba);
        Thread t = new Thread(
                new Runnable() {
                    public void run() {
                        try {
                            BasicA _ba = ((BasicA) pm.getObjectById(oid, false));
                            _ba.readF1_F2();
                            _ba.writeF1("test2thread1PM.thread2");
                            _ba.writeF2(2);
                            pm.currentTransaction().commit();
                            Assert.assertTrue(true);
                        } catch (Exception e) {
                            e.printStackTrace();
                            fail(e.getMessage());
                        }
                    }
                });
        t.start();
        try {
            t.join();
        } catch (InterruptedException e) {
        }
        Assert.assertTrue("The thread2 is not finished", !t.isAlive());
        pm.currentTransaction().begin();
        Assert.assertEquals("Bad f2 value", 2, ba.readF2());
        ba.writeF2(3);
        pm.currentTransaction().commit();
        pm.close();
    }
}
