/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.runtime.detach;


import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.pobjects.detach.Car;
import org.objectweb.speedo.pobjects.detach.Coach;
import org.objectweb.speedo.pobjects.detach.FormulaOne;
import org.objectweb.speedo.pobjects.detach.Player;
import org.objectweb.speedo.pobjects.detach.Team;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author Y.Bersihand
 */
public class TestAttach extends SpeedoTestHelper {

	public TestAttach(String s) {
		super(s);
		//deleteDatabase();
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.detach.TestAttach";
	}
	
	/**
	 * Test the attach method: make an object persistent, detach it then attach it.
	 */
	public void testAttachCopy() {
		logger.log(BasicLevel.DEBUG, "***************testAttachCopy*****************");
		Team t = new Team("Bordeaux",null,null);
		Collection players = new ArrayList();
		Player p1 = new Player("p1", t, 25);
		players.add(p1);
		Player p2 = new Player("p2", t, 32);
		players.add(p2);
		t.setPlayers(players);
		Coach c = new Coach("c1", 5, t);
		t.setCoach(c);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "make persistent the team " + t.toString());
		pm.makePersistent(t);
		pm.currentTransaction().commit();
		//detach the team t
		Team copyOfT = (Team) ((ProxyManager)pm).detachCopy((Object)t);
		//print the team out
		logger.log(BasicLevel.DEBUG, copyOfT.toString());
		pm.currentTransaction().begin();
		//attach the team t
		Team attachedTeam = (Team) ((ProxyManager)pm).attachCopy((Object)copyOfT,false);
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG,"The attached version of the team is as follows:\n " + attachedTeam.toString());
		pm.close();
	}
		
	/**
	 * Test the attach method with update: make an object persistent, detach it , modify it and then attach it.
	 */
	public void testAttachModifiedCopy() {
		logger.log(BasicLevel.DEBUG, "***************testAttachModifiedCopy*****************");
		Team t = new Team("Paris",null,null);
		Collection players = new ArrayList();
		Player p1 = new Player("p3", t, 20);
		players.add(p1);
		Player p2 = new Player("p4", t, 30);
		players.add(p2);
		t.setPlayers(players);
		Coach c = new Coach("c2", 10, t);
		t.setCoach(c);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "make persistent the team " + t.toString());
		pm.makePersistent(t);
		pm.currentTransaction().commit();
		//detach the team t
		Team copyOfT = (Team) ((ProxyManager)pm).detachCopy((Object)t);
		//modify the team
		copyOfT.getCoach().setExperience(99);
		//		print the team out
		logger.log(BasicLevel.DEBUG, "Copy of team " + copyOfT.toString());
		pm.currentTransaction().begin();
		//attach the team t
		Team attachedTeam = (Team) ((ProxyManager)pm).attachCopy((Object)copyOfT,false);
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG,"The attached version of the team is as follows:\n " + attachedTeam.toString());
		pm.close();
	}

	/**
	 * Test the attach method with update on a collection: make an object persistent, detach it , modify it and then attach it.
	 */
	public void testAttachModifiedCollectionCopy() {
		logger.log(BasicLevel.DEBUG, "***************testAttachModifiedCollectionCopy*****************");
		Team t = new Team("Nantes",null,null);
		Collection players = new ArrayList();
		Player p1 = new Player("p5", t, 20);
		players.add(p1);
		Player p2 = new Player("p6", t, 30);
		players.add(p2);
		t.setPlayers(players);
		Coach c = new Coach("c3", 10, t);
		t.setCoach(c);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "make persistent the team " + t.toString());
		pm.makePersistent(t);
		pm.currentTransaction().commit();
		//detach the team t
		Team copyOfT = (Team) ((ProxyManager)pm).detachCopy((Object)t);
		pm.close();
		//modify the coach
		copyOfT.getCoach().setExperience(99);
		//and modify the collection
		Iterator it = copyOfT.getPlayers().iterator();
		while(it.hasNext()){
			Player p = (Player) it.next();
			p.setAge(99);
		}
		//		print the team out
		logger.log(BasicLevel.DEBUG, "Copy of team " + copyOfT.toString());
		PersistenceManager pm2 = pmf.getPersistenceManager();
		pm2.currentTransaction().begin();
		//attach the team t
		Team attachedTeam = (Team) ((ProxyManager)pm2).attachCopy((Object)copyOfT,false);
		pm2.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG,"The attached version of the team is as follows:\n " + attachedTeam.toString());
		pm2.close();
	}

	/**
	 * Test the attach method with a new object: the object is made persistent.
	 */
	public void testAttachNewObject() {
		logger.log(BasicLevel.DEBUG, "***************testAttachNewObject*****************");
		Team t = new Team("Bastia",null,null);
		Collection players = new ArrayList();
		Player p1 = new Player("p7", t, 20);
		players.add(p1);
		Player p2 = new Player("p8", t, 30);
		players.add(p2);
		t.setPlayers(players);
		Coach c = new Coach("c4", 10, t);
		t.setCoach(c);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "attach the team " + t.toString());
		Team attachedTeam = (Team) ((ProxyManager)pm).attachCopy((Object)t,false);
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG,"The attached version of the team is as follows:\n " + attachedTeam.toString());
		pm.close();
	}

	/**
	 * Test the attach method with an invalid copy: make an object persistent, modify it, detach it , rollback the transaction and then attach it.
	 */
	public void testAttachInvalidCopy() {
		logger.log(BasicLevel.DEBUG, "***************testAttachInvalidCopy*****************");
		Team t = new Team("Auxerre",null,null);
		Collection players = new ArrayList();
		Player p1 = new Player("p9", t, 20);
		players.add(p1);
		Player p2 = new Player("p10", t, 30);
		players.add(p2);
		t.setPlayers(players);
		Coach c = new Coach("c5", 10, t);
		t.setCoach(c);
		try{	
			PersistenceManager pm = pmf.getPersistenceManager();
			pm.currentTransaction().begin();
			logger.log(BasicLevel.DEBUG, "make persistent the team " + t.toString());
			System.out.println("MAKEPERSISTENT BEGINNING");
			pm.makePersistent(t);
			System.out.println("MAKEPERSISTENT END");
			pm.currentTransaction().commit();
			pm.currentTransaction().begin();
			System.out.println("UPDATE BEGINNING");
			//modify the team
			t.getCoach().setExperience(99);
			Iterator it  = t.getPlayers().iterator();
			//modify the players
			while(it.hasNext()){
				Player p = (Player) it.next();
				p.setAge(99);
			}
			System.out.println("DETACH BEGINNING");
			//detach the team t
			Team copyOfT = (Team) ((ProxyManager)pm).detachCopy((Object)t);
			System.out.println("DETACH END");
			System.out.println("UPDATE END");
			//rollback the tx : it invalids the copy
			pm.currentTransaction().rollback();
			//		print the team out
			logger.log(BasicLevel.DEBUG, "Copy of team " + copyOfT.toString());
			pm.currentTransaction().begin();
			System.out.println("ATTACH BEGIN");
			//attach the "invalid" team 
			Team attachedTeam = (Team) ((ProxyManager)pm).attachCopy((Object)copyOfT,false);
			System.out.println("ATTACH END");
			pm.currentTransaction().commit();
			logger.log(BasicLevel.DEBUG,"The attached version of the team is as follows:\n " + attachedTeam.toString());
			pm.close();
		}
		catch(Exception e){
			if(e instanceof JDOException){
				logger.log(BasicLevel.DEBUG, "Correct exception type has been caught : " + e);
			}
			else{
				logger.log(BasicLevel.DEBUG, "Problem occurred : " + e);
			}
		}
	}
	
	
	/**
	 * Test the attach method: make an inherited object persistent, detach it then attach it.
	 */
	public void testAttachInherited() {
		logger.log(BasicLevel.DEBUG, "***************testAttachInherited*****************");
		Car c = new Car("r5", 4, "red");
		FormulaOne f = new FormulaOne("williams", 4, "green", 262);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "make persistent the car " + c.toString() );
		logger.log(BasicLevel.DEBUG, "make persistent the formula one " + f.toString());
		pm.makePersistent(c);
		pm.makePersistent(f);
		pm.currentTransaction().commit();
		//detach the car c
		Car copyOfC = (Car) ((ProxyManager)pm).detachCopy((Object)c);
		//print the car out
		logger.log(BasicLevel.DEBUG, copyOfC.toString());
		//detach the formula one f
		FormulaOne copyOfF = (FormulaOne) ((ProxyManager)pm).detachCopy((Object)f);
		//print the formula one out
		logger.log(BasicLevel.DEBUG, copyOfF.toString());
		pm.currentTransaction().begin();
		//attach the copied car 
		Car attachedCar = (Car) ((ProxyManager)pm).attachCopy((Object)copyOfC,false);
		//attach the copied formula one
		FormulaOne attachedF = (FormulaOne) ((ProxyManager)pm).attachCopy((Object)copyOfF,false);
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG,"The attached version of the car is as follows:\n " + attachedCar.toString());
		logger.log(BasicLevel.DEBUG,"The attached version of the formula one is as follows:\n " + attachedF.toString());
		pm.close();
	}
	
	/**
	 * Test the attach method with update: make an inherited object persistent, detach it , modify it and then attach it.
	 */
	public void testAttachModifiedInherited() {
		logger.log(BasicLevel.DEBUG, "***************testAttachModifiedInherited*****************");
		FormulaOne f = new FormulaOne("renault", 4, "yellow", 262);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "make persistent the formula one " + f.toString());
		pm.makePersistent(f);
		pm.currentTransaction().commit();
		//detach
		FormulaOne copyOfF = (FormulaOne) ((ProxyManager)pm).detachCopy((Object)f);
		//modify 
		copyOfF.setNbOfWheels(2);
		copyOfF.setSpeedMax(320);
		//		print out
		logger.log(BasicLevel.DEBUG, "Copy of formula one " + copyOfF.toString());
		pm.currentTransaction().begin();
		//attach 
		FormulaOne attachedF = (FormulaOne) ((ProxyManager)pm).attachCopy((Object)copyOfF,false);
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG,"The attached version of the formula one is as follows:\n " + attachedF.toString());
		pm.close();
	}
	
	
	/**
	 * Test the attach method with a new inherited object: the object is made persistent.
	 */
	public void testAttachNewInherited() {
		logger.log(BasicLevel.DEBUG, "***************testAttachNewInherited*****************");
		FormulaOne f = new FormulaOne("sauber", 4, "red", 262);
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "attach the f1 " + f.toString());
		FormulaOne attachedF = (FormulaOne) ((ProxyManager)pm).attachCopy((Object)f,false);
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG,"The attached version of the team is as follows:\n " + attachedF.toString());
		pm.close();
	}
	
	
	/**
	 * Test the attach method with an invalid copy: make an inherited object persistent, modify it, detach it , rollback the transaction and then attach it.
	 */
	public void testAttachInvalidInherited() {
		logger.log(BasicLevel.DEBUG, "***************testAttachInvalidInherited*****************");
		FormulaOne f = new FormulaOne("ferrari", 4, "red", 262);
		try{	
			PersistenceManager pm = pmf.getPersistenceManager();
			pm.currentTransaction().begin();
			logger.log(BasicLevel.DEBUG, "make persistent the f1 " + f.toString());
			pm.makePersistent(f);
			pm.currentTransaction().commit();
			pm.currentTransaction().begin();
			//modify
			 f.setNbOfWheels(1);
			 //detach the team t
			FormulaOne copyOfF = (FormulaOne) ((ProxyManager)pm).detachCopy((Object)f);
			//rollback the tx : it invalids the copy
			pm.currentTransaction().rollback();
			//	print out
			logger.log(BasicLevel.DEBUG, "Copy of f1 " + copyOfF.toString());
			logger.log(BasicLevel.DEBUG, "rollback of the tx: the attach shouldn't work.");
			pm.currentTransaction().begin();
			//attach the "invalid" f1 
			FormulaOne attachedF = (FormulaOne) ((ProxyManager)pm).attachCopy((Object)copyOfF,false);
			pm.currentTransaction().commit();
			logger.log(BasicLevel.DEBUG,"The attached version of the team is as follows:\n " + attachedF.toString());
			pm.close();
		}
		catch(Exception e){
			logger.log(BasicLevel.DEBUG, "Problem occurred : " + e);
		}
	}
		
	/*----------------- PRIVATE METHODS----------------------------*/
	/**
	 * Delete all the entries from the table tableName
	 */
	private void deleteQuery(String tableName){
		try{
			 PMapper mapper =  getMapper(getPMF());
			Connection connection = (Connection) mapper.getConnection();
			connection.createStatement().execute("DELETE FROM "+ tableName);
			mapper.closeConnection(connection);
		}
		catch (Exception e){
			logger.log(BasicLevel.DEBUG, e.toString());
		}
	}
	
	private void deleteDatabase(){
		deleteQuery("SE_COACH");
		deleteQuery("SE_TEAM");
		deleteQuery("SE_PLAYER");
		deleteQuery("SE_VEHICLE");
	}
}
