/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.runtime.detach;


import java.math.BigDecimal;

import javax.jdo.PersistenceManager;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.pobjects.detach.Share;
import org.objectweb.speedo.pobjects.detach.SharePrice;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Initially, java.math.* was considered as a reference.
 * See visitFieldInsn() method of ClassAccessorModifier class.
 * The test:
 * 	detaches a Share
 * 	modify a BigDecimal value of a SharePrice of the prices
 *  attaches the Share
 * @author Y.Bersihand
 */
public class TestAttachBigDecimal extends SpeedoTestHelper {

	public TestAttachBigDecimal(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.detach.TestAttachBigDecimal";
	}
	
	/**
	 * Test the detach method
	 */
	public void testDetachModifyAttachCollection() {
		logger.log(BasicLevel.DEBUG, "*****************testDetachModifyAttachCollection************");
		//1st create 2 shares and prices
		Share share1 = new Share(1);
		Share share2 = new Share(2);
		
		SharePrice sp11 = new SharePrice(11, 2004, 11, new BigDecimal(11000));
		SharePrice sp12 = new SharePrice(12, 2004, 11, new BigDecimal(12000));
		SharePrice sp13 = new SharePrice(13, 2004, 11, new BigDecimal(13000));
		
		share1.addPrice(sp11);
		share1.addPrice(sp12);
		share1.addPrice(sp13);
		
		SharePrice sp21 = new SharePrice(21, 2005, 11, new BigDecimal(21000));
		SharePrice sp22 = new SharePrice(22, 2005, 11, new BigDecimal(22000));
		SharePrice sp23 = new SharePrice(23, 2005, 11, new BigDecimal(23000));
		
		share1.addPrice(sp21);
		share1.addPrice(sp22);
		share1.addPrice(sp23);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(share1);
		pm.makePersistent(share2);
		pm.currentTransaction().commit();
		
		Share copyOfShare = (Share) ((ProxyManager)pm).detachCopy((Object)share1);
		logger.log(BasicLevel.DEBUG, "share detached");
		SharePrice price = copyOfShare.getPrice(12,2004,11);
		if (price != null) {
		    price.setPrice(new BigDecimal(99000));
		    //price.setMonth(999);
		}
		logger.log(BasicLevel.DEBUG, "share price updated");
		pm.currentTransaction().begin();
		Share attachedShare = (Share) ((ProxyManager)pm).attachCopy((Object)copyOfShare,false);
		pm.currentTransaction().commit();
		logger.log(BasicLevel.DEBUG, "share attached");
	}
	
}
