/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.runtime.detach;


import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collection;

import javax.jdo.JDOUserException;
import javax.jdo.PersistenceManager;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pm.api.ProxyManager;
import org.objectweb.speedo.pobjects.detach.Address;
import org.objectweb.speedo.pobjects.detach.Car;
import org.objectweb.speedo.pobjects.detach.Coach;
import org.objectweb.speedo.pobjects.detach.FormulaOne;
import org.objectweb.speedo.pobjects.detach.Person;
import org.objectweb.speedo.pobjects.detach.Player;
import org.objectweb.speedo.pobjects.detach.Team;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author Y.Bersihand
 */
public class TestSwiss extends SpeedoTestHelper {

	public TestSwiss(String s) {
		super(s);
		deleteDatabase();
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.detach.TestSwiss";
	}
	
	/**
	 * Test the detach method
	 */
	public void testDetachManyTimes() {
		//Object personId = createPerson();
		long personId = createPersonLong();
		Person detached1 = getPersonById(personId);
		logger.log(BasicLevel.DEBUG, "detached1 address: " + detached1.getAddress().getType());
		Person detached2 = getPersonById(personId);
		logger.log(BasicLevel.DEBUG, "detached2 address: " + detached2.getAddress().getType());
	}
	
	/*----------------- PRIVATE METHODS----------------------------*/
	private Object createPerson(){
		logger.log(BasicLevel.DEBUG, "***************createPerson Object*****************");
		Address address = new Address("mail");
		Person person = new Person(address);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(person);
		pm.currentTransaction().commit();
		return pm.getObjectId(person);
	}
	
	private long createPersonLong(){
		logger.log(BasicLevel.DEBUG, "***************createPersonLong*****************");
		Address address = new Address("mail");
		Person person = new Person(address);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(person);
		pm.currentTransaction().commit();
		return person.getId();
	}
	
	private Person getPersonById(Object personId) {
		logger.log(BasicLevel.DEBUG, "***************getPersonById*****************");
	    Person result = null;
	    PersistenceManager pm = pmf.getPersistenceManager();
	    try {
	      //Object oid = pm.newObjectIdInstance(Person.class, "" + personId);
	      Person p = (Person) pm.getObjectById(personId, false);
	      if (p != null) {
	        p.getAddress();
	        result = (Person) ((ProxyManager) pm).detachCopy(p);
	      }
	    } catch (Exception e) {
	      logger.log(BasicLevel.DEBUG, "Person with Key=" + personId + " not found.");
	    } finally {
	      pm.close();
	    }
	    return result;
	 }
	
	private Person getPersonById(long personId) {
		logger.log(BasicLevel.DEBUG, "***************getPersonById*****************");
	    Person result = null;
	    PersistenceManager pm = pmf.getPersistenceManager();
	    try {
	      Object oid = pm.newObjectIdInstance(Person.class, "" + personId);
	      Person p = (Person) pm.getObjectById(oid, false);
	      if (p != null) {
	        p.getAddress();
	        result = (Person) ((ProxyManager) pm).detachCopy(p);
	      }
	    } catch (Exception e) {
	      logger.log(BasicLevel.DEBUG, "Person with Key=" + personId + " not found.");
	    } finally {
	      pm.close();
	    }
	    return result;
	 }
	/**
	 * Delete all the entries from the table tableName
	 */
	private void deleteQuery(String tableName){
		try{
			 PMapper mapper =  getMapper(getPMF());
			Connection connection = (Connection) mapper.getConnection();
			connection.createStatement().execute("DELETE FROM "+ tableName);
			mapper.closeConnection(connection);
		}
		catch (Exception e){
			logger.log(BasicLevel.DEBUG, e.toString());
		}
	}
	
	private void deleteDatabase(){
		deleteQuery("ESM_ADDRESSES");
		deleteQuery("ESM_PERSONS");
	}
}
