/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.runtime.query;

import org.objectweb.speedo.pobjects.ref.Department;
import org.objectweb.speedo.pobjects.ref.Employee;
import org.objectweb.speedo.pobjects.collection.AMMB;
import org.objectweb.speedo.pobjects.collection.BMMB;
import org.objectweb.speedo.pobjects.collection.Ref2AMMB;
import org.objectweb.speedo.pobjects.collection.Ref2Ref2AMMB;
import org.objectweb.speedo.SpeedoTestHelper;

import javax.jdo.PersistenceManager;
import java.util.Collection;
import java.util.ArrayList;

/**
 * @author S.Chassande-Barrioz
 */
public class POBuilder extends SpeedoTestHelper {

    public final static String depName = "RD";
    public final static String[] depNames = { "MAPS", "BIZZ", "TECK"};
    public final static String[] names = {"John", "Paul", "Marie", "Sandie"};
    public final static float[] salaries = {4000, 3000, 2999, 3001};
    public final static int[] nameOrder = {0, 2, 1, 3};
    public final static int[] salariesOrder = {0, 3, 1, 2};
    public final static int NB_XMMB = 4;

    public POBuilder(String name) {
        super(name);
    }

    protected String getLoggerName() {
        return LOG_NAME + ".rt.query";
    }

    public void testCreationOfPersistentObject() {
        PersistenceManager pm = pmf.getPersistenceManager();
        Department d = new Department(depName);
        pm.makePersistent(d);
        for(int i=0; i<depNames.length; i++) {
            pm.makePersistent(new Department(depNames[i]));
        }

        //Creation of Employee
        for(int i=0; i<names.length; i++) {
            pm.makePersistent(new Employee(names[i], d, salaries[i]));
        }

        //Creation of XMNB objects for testing collection operator
        Ref2AMMB[] r1 = new Ref2AMMB[NB_XMMB];
        Ref2Ref2AMMB[] r2 = new Ref2Ref2AMMB[NB_XMMB];
        AMMB[] as = new AMMB[NB_XMMB];
        BMMB[] bs = new BMMB[NB_XMMB];
        for(int i=0; i<as.length; i++) {
            as[i] = new AMMB(i);
            bs[i] = new BMMB(i);
            r1[i] = new Ref2AMMB(i*10, as[i]);
            r2[i] = new Ref2Ref2AMMB(i*100, r1[i]);
        }
        Collection as0 = new ArrayList(2);
        as0.add(bs[0]);
        as0.add(bs[1]);
        as[0].setBs(as0);
        Collection as1 = new ArrayList(3);
        as1.add(bs[0]);
        as1.add(bs[1]);
        as1.add(bs[2]);
        as[1].setBs(as1);
        Collection as2 = new ArrayList(2);
        as2.add(bs[1]);
        as2.add(bs[2]);
        as[2].setBs(as2);

        Collection bs0 = new ArrayList(2);
        bs0.add(as[0]);
        bs0.add(as[1]);
        bs[0].setAs(bs0);
        Collection bs1 = new ArrayList(3);
        bs1.add(as[0]);
        bs1.add(as[1]);
        bs1.add(as[2]);
        bs[1].setAs(bs1);
        Collection bs2 = new ArrayList(2);
        bs2.add(as[1]);
        bs2.add(as[2]);
        bs[2].setAs(bs2);
        pm.makePersistentAll(r2);
        pm.makePersistentAll(bs);
        pm.close();
    }
}
