/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.tck;

import java.util.Iterator;

import javax.jdo.Extent;
import javax.jdo.JDOHelper;
import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.Transaction;

import org.objectweb.speedo.pobjects.tck.StateTransitionObj;
import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.util.monolog.api.BasicLevel;







	


public class JdoLifecycleTck extends SpeedoTestHelper {

	public static final int TRANSIENT                   = 0;
	public static final int PERSISTENT_NEW              = 1;
	public static final int PERSISTENT_CLEAN            = 2;
	public static final int PERSISTENT_DIRTY            = 3;
	public static final int HOLLOW                      = 4;
	public static final int TRANSIENT_CLEAN             = 5;
	public static final int TRANSIENT_DIRTY             = 6;
	public static final int PERSISTENT_NEW_DELETED      = 7;
	public static final int PERSISTENT_DELETED          = 8;
	public static final int PERSISTENT_NONTRANSACTIONAL = 9;
	public static final int NUM_STATES = 10;
	public static final int ILLEGAL_STATE = 10;

	public static final String[] states = {
			"transient",
			"persistent-new",
			"persistent-clean",
			"persistent-dirty",
			"hollow",
			"transient-clean",
			"transient-dirty",
			"persistent-new-deleted",
			"persistent-deleted",
			"persistent-nontransactional",
			"illegal"
	};
	private static final int IS_PERSISTENT       = 0;
	private static final int IS_TRANSACTIONAL    = 1;
	private static final int IS_DIRTY            = 2;
	private static final int IS_NEW              = 3;
	private static final int IS_DELETED          = 4;
	private static final int NUM_STATUSES        = 5;

	/*
	 * This table indicates the values returned by the status interrogation
	 * methods for each state. This is used to determine the current lifecycle
	 * state of an object.
	 */
	private static final boolean state_statuses[][] = {
//      IS_PERSISTENT   IS_TRANSACTIONAL    IS_DIRTY    IS_NEW      IS_DELETED
// transient
			{   false,          false,              false,      false,      false},

// persistent-new
			{   true,           true,               true,       true,       false},

// persistent-clean
			{   true,           true,               false,      false,      false},

// persistent-dirty
			{   true,           true,               true,       false,      false},

// hollow
			{   true,           false,              false,      false,      false},

// transient-clean
			{   false,          true,               false,      false,      false},

// transient-dirty
			{   false,          true,               true,       false,      false},

// persistent-new-deleted
			{   true,           true,               true,       true,       true},

// persistent-deleted
			{   true,           true,               true,       false,      true},

// persistent-nontransactional
			{   true,           false,              false,      false,      false}
	};


// These values MUST correspond to index in sorted array below.
	private static final int OPTION_APPLICATIONIDENTITY       = 0;
	private static final int OPTION_ARRAY                     = 1;
	private static final int OPTION_ARRAYLIST                 = 2;
	private static final int OPTION_CHANGEAPPLICATIONIDENTITY = 3;
	private static final int OPTION_DATASTOREIDENTITY         = 4;
	private static final int OPTION_HASHMAP                   = 5;
	private static final int OPTION_HASHTABLE                 = 6;
	private static final int OPTION_LINKEDLIST                = 7;
	private static final int OPTION_LIST                      = 8;
	private static final int OPTION_MAP                       = 9;
	private static final int OPTION_NONDURABLEIDENTITY        = 10;
	private static final int OPTION_NONTRANSACTIONALREAD      = 11;
	private static final int OPTION_NONTRANSACTIONALWRITE     = 12;
	private static final int OPTION_NULLCOLLECTION            = 13;
	private static final int OPTION_OPTIMISTIC                = 14;
	private static final int OPTION_RETAINVALUES              = 15;
	private static final int OPTION_TRANSIENTTRANSACTIONAL    = 16;
	private static final int OPTION_TREEMAP                   = 17;
	private static final int OPTION_TREESET                   = 18;
	private static final int OPTION_VECTOR                    = 19;

	private static boolean[] options = new boolean[OPTION_VECTOR + 1];
	private static Boolean syncObj  = new Boolean(false);
	private static boolean syncFlag = false;

	private static String[] optionNames = { // MUST be in "natural sorted order"
			"-ApplicationIdentity",
			"-Array",
			"-ArrayList",
			"-ChangeApplicationIdentity",
			"-DatastoreIdentity",
			"-HashMap",
			"-Hashtable",
			"-LinkedList",
			"-List",
			"-Map",
			"-NonDurableIdentity",
			"-NontransactionalRead",
			"-NontransactionalWrite",
			"-NullCollection",
			"-Optimistic",
			"-RetainValues",
			"-TransientTransactional",
			"-TreeMap",
			"-TreeSet",
			"-Vector"
	};

	
	
	
	
    private PersistenceManager          pm;
    private Transaction                 transaction;
    private int                         scenario;
    private int                         operation;
    private int                         current_state;
    private int                         expected_state;
    private int                         new_state;
    private String                      pmfName;
    private String                      contextFactory;
    private String                      providerURL;
    private String                      connectionUserName;
    private String                      connectionPassword;
    private String                      connectionURL;

private static final String PMFNAME = "pmfName";
private static final String JNDICONTEXTFACTORY = "PMFProperties";

/**
 * Operations that cause state changes
 */
private static final int MAKEPERSISTENT          = 0;
private static final int DELETEPERSISTENT        = 1;
private static final int MAKETRANSACTIONAL       = 2;
private static final int MAKENONTRANSACTIONAL    = 3;
private static final int MAKETRANSIENT           = 4;
private static final int COMMITNORETAINVALUES    = 5;
private static final int COMMITRETAINVALUES      = 6;
private static final int ROLLBACKNORETAINVALUES  = 7;
private static final int ROLLBACKRETAINVALUES    = 8;
private static final int REFRESHDATASTORE        = 9;
private static final int REFRESHOPTIMISTIC       = 10;
private static final int EVICT                   = 11;
private static final int READOUTSIDETX           = 12;
private static final int READOPTIMISTIC          = 13;
private static final int READDATASTORE           = 14;
private static final int WRITEOUTSIDETX          = 15;
private static final int WRITEINSIDETX           = 16;

private static final String[] operations = {
    "makePersistent",
    "deletePersistent",
    "makeTransactional",
    "makeNontransactional",
    "makeTransient",
    "commit, retainValues=false",
    "commit, retainValues=true",
    "rollback, retainValues=false",
    "rollback, retainValues=true",
    "refresh with active datastore tx",
    "refresh with active optimistic tx",
    "evict",
    "read field outside tx",
    "read field with active optimistic tx",
    "read field with active datastore tx",
    "write field outside tx",
    "write field with active tx"
};
private static final int NUM_OPERATIONS = 17;

private static final boolean[] closes_transaction =
{ false, false, false, false, false, true, true, true, true, false,
false, false, false, false, false, false, false };

/**
 * Illegal state transitions
 */
private static final int UNCHANGED                   = -1;
private static final int ERROR                       = -2;
private static final int IMPOSSIBLE                  = -3;
private static final int NOT_APPLICABLE              = -4;

/**
 * State transitions
 */

public static final int[][] transitions = { // [operation] [current state] = new state
// makePersistent
    { PERSISTENT_NEW, UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED, PERSISTENT_NEW,
      PERSISTENT_NEW, UNCHANGED, UNCHANGED, UNCHANGED},

// deletePersistent
    { ERROR, PERSISTENT_NEW_DELETED, PERSISTENT_DELETED, PERSISTENT_DELETED,
      PERSISTENT_DELETED, ERROR, ERROR, UNCHANGED, UNCHANGED, PERSISTENT_DELETED},

// makeTransactional
    { TRANSIENT_CLEAN, UNCHANGED, UNCHANGED, UNCHANGED, PERSISTENT_CLEAN,
      UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED, PERSISTENT_CLEAN},
    
// makeNontransactional
    { ERROR, ERROR, PERSISTENT_NONTRANSACTIONAL, ERROR, UNCHANGED,
      TRANSIENT, ERROR, ERROR, ERROR, UNCHANGED},

// makeTransient
    { UNCHANGED, ERROR, TRANSIENT, ERROR, TRANSIENT,
      UNCHANGED, UNCHANGED, ERROR, ERROR, TRANSIENT},

// commit, retainValues = false
    { UNCHANGED, HOLLOW, HOLLOW, HOLLOW, UNCHANGED, UNCHANGED,
      TRANSIENT_CLEAN, TRANSIENT, TRANSIENT, UNCHANGED},

// commit, retainValues = true
    { UNCHANGED, PERSISTENT_NONTRANSACTIONAL, PERSISTENT_NONTRANSACTIONAL,
      PERSISTENT_NONTRANSACTIONAL, UNCHANGED, UNCHANGED, TRANSIENT_CLEAN,
      TRANSIENT, TRANSIENT, UNCHANGED},

// rollback, retainValues = false
    { UNCHANGED, TRANSIENT, HOLLOW, HOLLOW, UNCHANGED, UNCHANGED,
      TRANSIENT_CLEAN, TRANSIENT, HOLLOW, UNCHANGED},

// rollback, retainValues = true
    { UNCHANGED, TRANSIENT, PERSISTENT_NONTRANSACTIONAL, PERSISTENT_NONTRANSACTIONAL,
      UNCHANGED, UNCHANGED, TRANSIENT_CLEAN, TRANSIENT, PERSISTENT_NONTRANSACTIONAL, UNCHANGED},

// refresh with active datastore transaction
    { UNCHANGED, UNCHANGED, UNCHANGED, PERSISTENT_CLEAN, UNCHANGED,
      UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED},

// refresh with active optimistic transaction
    { UNCHANGED, UNCHANGED, UNCHANGED, PERSISTENT_NONTRANSACTIONAL, UNCHANGED,
      UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED},

// evict
    { UNCHANGED, UNCHANGED, HOLLOW, UNCHANGED, UNCHANGED,
      UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED, HOLLOW},

// read field outside transaction
    { UNCHANGED, IMPOSSIBLE, IMPOSSIBLE, IMPOSSIBLE, PERSISTENT_NONTRANSACTIONAL,
      UNCHANGED, IMPOSSIBLE, IMPOSSIBLE, IMPOSSIBLE, UNCHANGED},

// read field with active optimistic transaction
    { UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED, PERSISTENT_NONTRANSACTIONAL,
      UNCHANGED, UNCHANGED, ERROR, ERROR, UNCHANGED},

// read field with active datastore transaction
    { UNCHANGED, UNCHANGED, UNCHANGED, UNCHANGED, PERSISTENT_CLEAN,
      UNCHANGED, UNCHANGED, ERROR, ERROR, PERSISTENT_CLEAN},

// write field outside transaction
    { UNCHANGED, IMPOSSIBLE, IMPOSSIBLE, IMPOSSIBLE, PERSISTENT_NONTRANSACTIONAL,
      UNCHANGED, IMPOSSIBLE, IMPOSSIBLE, IMPOSSIBLE, UNCHANGED},
    
// write field with active transaction
    { UNCHANGED, UNCHANGED, PERSISTENT_DIRTY, UNCHANGED, PERSISTENT_DIRTY,
      TRANSIENT_DIRTY, UNCHANGED, ERROR, ERROR, PERSISTENT_DIRTY}
};

	private static final int DATASTORE_TX = 0;
	private static final int OPTIMISTIC_TX = 1;
	private static final int NO_TX = 2;
	
	private static final String[] scenario_string = {
	    "datastore transaction", "optimistic transaction", "no transaction"
	};
	
	private static final boolean[][] applies_to_scenario = {
	//      Datastore   Optimistic      No tx
	    {   true,          true,        false },  // makePersistent
	    {   true,          true,        false },  // deletePersistent
	    {   true,          true,        false },  // makeTransactional
	    {   true,          true,        false },  // makeNontransactional
	    {   true,          true,        false },  // makeTransient
	    {   true,          true,        false },  // commit RetainValues = false
	    {   true,          true,        false },  // commit RetainValues = true
	    {   true,          true,        false },  // rollback RetainValues = false
	    {   true,          true,        false },  // rollback RetainValues = true
	    {   true,          false,       false },  // refresh with active datastore
	    {   false,         true,        false },  // refresh with active optimistic
	    {   true,          true,        false },  // evict
	    {   false,         false,       true  },  // read field outside transaction
	    {   false,         true,        false },  // read field with optimistic
	    {   true,          false,       false },  // read field with datastore
	    {   false,         false,       true  },  // write field outside transaction
	    {   true,          true,        false }  // write field with active transaction
	};
	
		
	/**
	 *Reports whether TransientTransactional is supported.
	 */
	public static boolean isTransientTransactionalSupported()
	{
		return options[OPTION_TRANSIENTTRANSACTIONAL];
	}
	
	/**
	 *Reports whether NontransactionalRead is supported.
	 */
	public static boolean isNontransactionalReadSupported()
	{
		return options[OPTION_NONTRANSACTIONALREAD];
	}
	
	/**
	 *Reports whether NontransactionalWrite is supported.
	 */
	public static boolean isNontransactionalWriteSupported()
	{
		return options[OPTION_NONTRANSACTIONALWRITE];
	}
	
	/**
	 *Reports whether RetainValues is supported.
	 */
	public static boolean isRetainValuesSupported()
	{
		return options[OPTION_RETAINVALUES];
	}
	
	/**
	 *Reports whether Optimistic is supported.
	 */
	public static boolean isOptimisticSupported()
	{
		return options[OPTION_OPTIMISTIC];
	}
	
	/**
	 *Reports whether Application Identity is supported.
	 */
	public static boolean isApplicationIdentitySupported()
	{
		return options[OPTION_APPLICATIONIDENTITY];
	}
	
	/**
	 * Reports whether Changing Application Identity is supported.
	 */
	public static boolean isChangeApplicationIdentitySupported()
	{
		return options[OPTION_CHANGEAPPLICATIONIDENTITY];
	}
	/**
	 *Reports whether Datastore Identity is supported.
	 */
	public static boolean isDatastoreIdentitySupported()
	{
		return options[OPTION_DATASTOREIDENTITY];
	}
	
	/**
	 *Reports whether Non-Durable Identity is supported.
	 */
	public static boolean isNonDurableIdentitySupported()
	{
		return options[OPTION_NONDURABLEIDENTITY];
	}
	
	/**
	 *Reports whether an <code>ArrayList</code> collection is supported.
	 */
	public static boolean isArrayListSupported()
	{
		return options[OPTION_ARRAYLIST];
	}
	
	/**
	 *Reports whether a <code>HashMap</code> collection is supported.
	 */
	public static boolean isHashMapSupported()
	{
		return options[OPTION_HASHMAP];
	}
	
	/**
	 *Reports whether a <code>Hashtable</code> collection is supported.
	 */
	public static boolean isHashtableSupported()
	{
		return options[OPTION_HASHTABLE];
	}
	
	/**
	 *Reports whether a <code>LinkedList</code> collection is supported.
	 */
	public static boolean isLinkedListSupported()
	{
		return options[OPTION_LINKEDLIST];
	}
	
	/**
	 *Reports whether a <code>TreeMap</code> collection is supported.
	 */
	public static boolean isTreeMapSupported()
	{
		return options[OPTION_TREEMAP];
	}
	
	/**
	 *Reports whether a <code>TreeSet</code> collection is supported.
	 */
	public static boolean isTreeSetSupported()
	{
		return options[OPTION_TREESET];
	}
	
	/**
	 *Reports whether a <code>Vector</code> collection is supported.
	 */
	public static boolean isVectorSupported()
	{
		return options[OPTION_VECTOR];
	}
	
	/**
	 *Reports whether a <code>Map</code> collection is supported.
	 */
	public static boolean isMapSupported()
	{
		return options[OPTION_MAP];
	}
	
	/**
	 *Reports whether a <code>List</code> collection is supported.
	 */
	public static boolean isListSupported()
	{
		return options[OPTION_LIST];
	}
	
	/**
	 *Reports whether arrays are supported.
	 */
	public static boolean isArraySupported()
	{
		return options[OPTION_ARRAY];
	}
	
	/**
	 *Reports whether a null collection is supported.
	 */
	public static boolean isNullCollectionSupported()
	{
		return options[OPTION_NULLCOLLECTION];
	}
	
	/**
	 * This method will return the current lifecycle state of an instance.
	 */
	public static int currentState(Object o)
	{
		boolean[] status = new boolean[5];
		status[IS_PERSISTENT]       = JDOHelper.isPersistent(o);
		status[IS_TRANSACTIONAL]    = JDOHelper.isTransactional(o);
		status[IS_DIRTY]            = JDOHelper.isDirty(o);
		status[IS_NEW]              = JDOHelper.isNew(o);
		status[IS_DELETED]          = JDOHelper.isDeleted(o);
		int i, j;
		outerloop:
			for( i = 0; i < NUM_STATES; ++i ){
				for( j = 0; j < NUM_STATUSES; ++j ){
					if( status[j] != state_statuses[i][j] )
						continue outerloop;
				}
				return i;
			}
			return NUM_STATES;
	}

	
	public JdoLifecycleTck(String s) {
		super(s);		
				
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.tck.JdoLifecycleTck";
	}

	public void testStateTransitions() {
		logger.log(BasicLevel.INFO, "testStateTransitions");
		try {
			pmf = getPMF();
			if( pmf == null ){
				fail("StateTransitions: Unable to acquire a PMF");
			}
			pm = pmf.getPersistenceManager();
			generatePersistentInstances();
			
			scenario = DATASTORE_TX;
			logger.log(BasicLevel.INFO, "###scenario = DATASTORE_TX###");
			checkTransitions();
			if( isOptimisticSupported() ){
				scenario = OPTIMISTIC_TX;
				logger.log(BasicLevel.INFO, "###scenario = OPTIMISTIC_TX###");
				checkTransitions();
			}
			if( isNontransactionalReadSupported() || isNontransactionalWriteSupported() ){
				scenario = NO_TX;
				logger.log(BasicLevel.INFO, "###scenario = NO_TX###");
				checkTransitions();
			}
		} catch (Exception e) {
			logger.log(BasicLevel.ERROR, "Unexception caught in testStateTransitions", e);
			fail(e.getMessage());	
		} finally {
			Transaction tx = pm.currentTransaction();
			if (tx.isActive()) {
				tx.rollback();
			}
			pm.close();
		}
		/*
		int errCnt = getErrorCount();
		if( errCnt == 0 ){
			return Status.passed("StateTransitions passed");
		} else{
			return Status.failed("StateTransitions failed, number of errors detected is " + errCnt);
		}
		*/
	}
	
	private void generatePersistentInstances()
	{
		if( doPersistentInstancesExist() ) return;
		int i;
		Transaction t = pm.currentTransaction();
		t.begin();
		for( i = 0; i < 50; ++i ){
			StateTransitionObj sto = new StateTransitionObj(i);
			sto.writeField(i);
			pm.makePersistent(sto);
		}
		t.commit();
		if( !doPersistentInstancesExist() )
			logger.log(BasicLevel.INFO, "StateTransitions unable to create instances of StateTransitionsObj");			
	}

	private boolean doPersistentInstancesExist()
	{
		boolean ret;
		Transaction t = pm.currentTransaction();
		t.begin();
		Extent e = pm.getExtent(StateTransitionObj.class, false);
		Iterator iter = e.iterator();
		ret = iter.hasNext();
		t.rollback();
		return ret;
	}

	void checkTransitions()
	{
		for( operation = 0; operation < NUM_OPERATIONS; ++operation ){
			
			
			// rule out situations that do not apply
			if( ! applies_to_scenario[operation][scenario] ) continue;
			if( operation == READOUTSIDETX && !isNontransactionalReadSupported() ) continue;
			if( operation == WRITEOUTSIDETX && !isNontransactionalWriteSupported() ) continue;
			if( (operation == COMMITRETAINVALUES || operation == ROLLBACKRETAINVALUES) &&
					!isRetainValuesSupported() ) continue;
			if( operation == MAKENONTRANSACTIONAL &&
					!(isNontransactionalReadSupported() || isNontransactionalWriteSupported()) )
				continue;
			
			for( current_state = 0; current_state < NUM_STATES; ++current_state){
				if( scenario == OPTIMISTIC_TX && current_state == PERSISTENT_CLEAN ) continue;
				if( (current_state == TRANSIENT_CLEAN || current_state == TRANSIENT_DIRTY) &&
						!isTransientTransactionalSupported() )
					continue;   // this state is not supported by implementation
				if( current_state == PERSISTENT_NONTRANSACTIONAL &&
						!(isNontransactionalReadSupported() || isNontransactionalWriteSupported()) )
					continue;   // this state is not supported by implementation
				expected_state = transitions[operation][current_state];
				if( expected_state == IMPOSSIBLE ) continue;
				if( expected_state == UNCHANGED ) expected_state = current_state;
				try {
					transaction = pm.currentTransaction();
					if( transaction.isActive()){
						logger.log(BasicLevel.INFO, "Transaction is active (but should not be), rolling back");
						transaction.rollback();
					}
					if( scenario != NO_TX ){

						if( operation == COMMITNORETAINVALUES || operation == ROLLBACKNORETAINVALUES )
							transaction.setRetainValues(false);
						if( operation == COMMITRETAINVALUES || operation == ROLLBACKRETAINVALUES )
							transaction.setRetainValues(true);
						/* possibly switch above 4 lines to the following line
						 transaction.setRetainValues( operation == COMMITRETAINVALUES || operation == ROLLBACKRETAINVALUES );
						 */
						transaction.setOptimistic(scenario == OPTIMISTIC_TX);
						transaction.begin();
						if( !transaction.isActive() )
							logger.log(BasicLevel.INFO, "StateTransitions: Transaction should be active, but it is not");
					}

					// get and verify object is in right initial state            
					StateTransitionObj obj = getInstanceInState(current_state);
					if( obj == null ){  // could not get object in state
						if( transaction.isActive() ) transaction.rollback();
						continue;
					}
					
					// Apply operation, catching possible exception
					Exception e = null;
					try {
						logger.log(BasicLevel.DEBUG, "##initial state="+states[current_state]);
						logger.log(BasicLevel.DEBUG, "##operation="+operations[operation]);
						
						applyOperation(operation, obj);
					} catch( Exception excep ){
						if( excep instanceof javax.jdo.JDOUserException ){
							e = excep;
						} else {
							logger.log(BasicLevel.ERROR, "StateTransitions: Unexpected exception:"+excep);
							printSituation();
							//incrementErrorCount();
							continue;
						}
					}
					
					// Get new state, verify correct transition and exceptions occurred
					logger.log(BasicLevel.DEBUG, "##expected state="+states[new_state]);
					new_state = currentState(obj);
					logger.log(BasicLevel.DEBUG, "##new state     ="+states[expected_state]);
					if( expected_state == ERROR || expected_state == NOT_APPLICABLE ){
						if( e == null ){							
							logger.log(BasicLevel.ERROR, "StateTransitions: JDOUserException should have been thrown");							
							printSituation();
							//incrementErrorCount();
						} else {
							if( new_state != current_state ){								
								logger.log(BasicLevel.ERROR, "JDOUserException properly thrown, but instance should remain in current state, instance changed state to "+states[new_state]);
								printSituation();
								//incrementErrorCount();
							}
						}
					}
					if( expected_state >= 0 && new_state != expected_state && 
							!((new_state == HOLLOW && expected_state == PERSISTENT_NONTRANSACTIONAL) ||
									(new_state == PERSISTENT_NONTRANSACTIONAL && expected_state == HOLLOW)) ){						
						logger.log(BasicLevel.ERROR, "StateTransitions: Invalid state transition to "+states[new_state] + ", new state should be "+states[expected_state]);
						printSituation();
						//incrementErrorCount();
					}
					if( transaction.isActive() ) transaction.rollback();
				} catch(Exception unexpected_exception){
					logger.log(BasicLevel.ERROR, "Unexpected exception caught in StateTransitions "+unexpected_exception);
					printSituation();
					//unexpected_exception.printStackTrace(out);
					//incrementErrorCount();
					if( transaction.isActive() ) transaction.rollback();
				}
			}
		}

	}

	void printSituation()
	{
		logger.log(BasicLevel.INFO, "========================");
		logger.log(BasicLevel.INFO, scenario_string[scenario]);
		logger.log(BasicLevel.INFO, "initial state="+states[current_state]);
		logger.log(BasicLevel.INFO, "operation="+operations[operation]);
		logger.log(BasicLevel.INFO, "========================");
	}

	void applyOperation(int operation, StateTransitionObj stobj)
	{
		StateTransitionObj obj = (StateTransitionObj) stobj;
		switch( operation ){
		case MAKEPERSISTENT:
		{
			pm.makePersistent(obj);
			break;
		}
		case DELETEPERSISTENT:
		{
			pm.deletePersistent(obj);
			break;
		}
		case MAKETRANSACTIONAL:
		{
			pm.makeTransactional(obj);
			break;
		}
		case MAKENONTRANSACTIONAL:
		{
			pm.makeNontransactional(obj);
			break;
		}
		case MAKETRANSIENT:
		{
			pm.makeTransient(obj);
			break;
		}
		case COMMITNORETAINVALUES:
		{
			pm.currentTransaction().commit();
			break;
		}
		case COMMITRETAINVALUES:
		{
			pm.currentTransaction().commit();
			break;
		}
		case ROLLBACKNORETAINVALUES:
		{
			pm.currentTransaction().rollback();
			break;
		}
		case ROLLBACKRETAINVALUES:
		{
			pm.currentTransaction().rollback();
			break;
		}
		case REFRESHDATASTORE:
		{
			pm.refresh(obj);
			break;
		}
		case REFRESHOPTIMISTIC:
		{
			pm.refresh(obj);
			break;
		}
		case EVICT:
		{
			pm.evict(obj);
			break;
		}
		case READOUTSIDETX:
		{
			int val = obj.readField();
			break;
		}
		case READOPTIMISTIC:
		{
			int val = obj.readField();
			break;
		}
		case READDATASTORE:
		{
			int val = obj.readField();
			break;
		}
		case WRITEOUTSIDETX:
		{
			obj.writeField(42);
			break;
		}
		case WRITEINSIDETX:
		{
			obj.writeField(42);
			break;
		}
		default:
		{
			logger.log(BasicLevel.ERROR, "StateTransitions internal error, illegal operation "+operation);
		}
		}
	}


	/**
	 * Get an instance in the specified state.
	 */
	private StateTransitionObj getInstanceInState(int state)
	{
		switch(state) {
		case TRANSIENT:
			return getTransientInstance();
		case PERSISTENT_NEW:
			return getPersistentNewInstance();
		case PERSISTENT_CLEAN:
			return getPersistentCleanInstance();
		case PERSISTENT_DIRTY:
			return getPersistentDirtyInstance();
		case HOLLOW:
			return getHollowInstance();
		case TRANSIENT_CLEAN:
			return getTransientCleanInstance();
		case TRANSIENT_DIRTY:
			return getTransientDirtyInstance();
		case PERSISTENT_NEW_DELETED:
			return getPersistentNewDeletedInstance();
		case PERSISTENT_DELETED:
			return getPersistentDeletedInstance();
		case PERSISTENT_NONTRANSACTIONAL:
			return getPersistentNontransactionalInstance();
		default:
		{
			return null;
		}
		}
	}

	private StateTransitionObj getTransientInstance()
	{
		StateTransitionObj obj = new StateTransitionObj(23);
		int curr = currentState(obj);
		if( curr != TRANSIENT ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create transient instance, state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	private StateTransitionObj getPersistentNewInstance()
	{
		StateTransitionObj obj = getTransientInstance();
		if( obj == null ) return null;
		pm.makePersistent(obj); // should transition to persistent-new
		int curr = currentState(obj);
		if( curr != PERSISTENT_NEW ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create persistent-new instance");
			logger.log(BasicLevel.INFO, " from transient instance via makePersistent(), state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	public StateTransitionObj getPersistentCleanInstance()
	{
		StateTransitionObj obj = getHollowInstance();
		if( obj == null ) return null;
		StateTransitionObj sto = (StateTransitionObj) obj;
		int val = sto.readField();
		int curr = currentState(sto);
		if( curr != PERSISTENT_CLEAN ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create persistent-clean instance");
			logger.log(BasicLevel.INFO, " from a hollow instance by reading a field, state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	public StateTransitionObj getPersistentDirtyInstance()
	{
		StateTransitionObj obj = getHollowInstance();
		if( obj == null ) return null;
		StateTransitionObj pcobj = (StateTransitionObj) obj;
		pcobj.writeField(23);
		int curr = currentState(obj);
		if( curr != PERSISTENT_DIRTY ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create persistent-dirty instance");
			logger.log(BasicLevel.INFO, " from a hollow instance by writing a field, state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	public StateTransitionObj getHollowInstance()
	{
		Extent extent = pm.getExtent(StateTransitionObj.class, false);
		Iterator iter = extent.iterator();
		if( !iter.hasNext() ){
			logger.log(BasicLevel.INFO, "Extent for StateTransitionObj should not be empty");
			return null;
		}
		StateTransitionObj obj = (StateTransitionObj) iter.next();
		int curr = currentState(obj);
		if( curr != HOLLOW && curr != PERSISTENT_NONTRANSACTIONAL ){
			logger.log(BasicLevel.INFO, "StateTransition: Attempt to get hollow instance via accessing extent failed, state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	public StateTransitionObj getTransientCleanInstance()
	{
		StateTransitionObj obj = getTransientInstance();
		if( obj == null ) return null;
		pm.makeTransactional(obj);
		int curr = currentState(obj);
		if( curr != TRANSIENT_CLEAN ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create transient-clean instance");
			logger.log(BasicLevel.INFO, " from a transient instance via makeTransactional(), state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	public StateTransitionObj getTransientDirtyInstance()
	{
		StateTransitionObj obj = getTransientCleanInstance();
		if( obj == null ) return null;
		StateTransitionObj pcobj = (StateTransitionObj) obj;
		pcobj.writeField(23);
		int curr = currentState(obj);
		if( curr != TRANSIENT_DIRTY ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create transient-dirty instance");
			logger.log(BasicLevel.INFO, " from a transient clean instance via modifying a field, state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	public StateTransitionObj getPersistentNewDeletedInstance()
	{
		StateTransitionObj obj = getPersistentNewInstance();
		if( obj == null ) return null;
		pm.deletePersistent(obj);   // should transition to persistent-new-deleted
		int curr = currentState(obj);
		if( curr != PERSISTENT_NEW_DELETED ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create transient-new-deleted instance");
			logger.log(BasicLevel.INFO, " from a persistent-new instance via deletePersistent, state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	public StateTransitionObj getPersistentDeletedInstance()
	{
		StateTransitionObj obj = getHollowInstance();
		if( obj == null ) return null;
		pm.deletePersistent(obj);
		int curr = currentState(obj);
		if( curr != PERSISTENT_DELETED ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create persistent-deleted instance");
			logger.log(BasicLevel.INFO, " from a persistent instance via deletePersistent(), state is "+states[curr]);
			printSituation();
			return null;
		}
		return obj;
	}

	public StateTransitionObj getPersistentNontransactionalInstance()
	{
		StateTransitionObj obj = getHollowInstance();
		if( obj == null ) return null;
		pm.makeNontransactional(obj);
		int curr = currentState(obj);
		if( curr != PERSISTENT_NONTRANSACTIONAL && curr != HOLLOW ){
			logger.log(BasicLevel.INFO, "StateTransitions: Unable to create persistent-nontransactional instance");
			logger.log(BasicLevel.INFO, " from a persistent clean instance via makeNontransactional(), state is "+states[curr]);
			printSituation();
			return null;
		}
		return null;
	}
}

