/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.userid;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.pobjects.userid.IntervenantHelper;
import org.objectweb.speedo.pobjects.userid.ConventionHelper;
import org.objectweb.util.monolog.api.BasicLevel;

import junit.framework.Assert;
import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Calendar;
import java.util.Date;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestGroupama extends SpeedoTestHelper {

	public TestGroupama(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + "rt.userid.groupama";
	}
    public void testA() {
        try {
            PersistenceManager pm = pmf.getPersistenceManager();
            pm.currentTransaction().begin();
			for(int i=0; i<5; i++) {
				IntervenantHelper ih = new IntervenantHelper();
				ih.setNomep(i);
				ih.setConventions(new ArrayList());
				for(int j=0; j<i; j++) {
					ConventionHelper ch = new ConventionHelper();
					ch.setCdmec("cdmec" + i + "_" + j);
					ch.setCdmem("cdmem" + i + "_" + j);
					ch.setCdmed("cdmed" + i + "_" + j);
					ch.setNomep(i);
					ch.setIntervenant(ih);
					ch.setDdmec(Calendar.getInstance().getTime());
					ch.setCdmese("cdmese" + i + "_" + j);
					ih.getConventions().add(ch);
				}
				pm.makePersistent(ih);
			}
            pm.currentTransaction().commit();

			pm.evictAll(); //clear the L2 cache
			
            pm.currentTransaction().begin();
			IntervenantHelper ih = (IntervenantHelper) pm.getObjectById(
				pm.newObjectIdInstance(IntervenantHelper.class, "" + 2), 
				false);
			assertEquals("Bad convention elements in collection", 
				2,
				ih.getConventions().size());
            pm.currentTransaction().commit();
			
            pm.close();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Exception", e);
            fail(e.getMessage());
        }
    }
	
}
