/*
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Created on 15 mars 2004
 * @author fmillevi@yahoo.com
 *
 */
package org.objectweb.speedo.j2eedo.common;

import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;

import org.objectweb.speedo.j2eedo.bo.PollsSynchronizations;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * This class is used to handle the JDO persistence manager and it's factory
 * @author fmillevi@yahoo.com
 */
public class PMHolder {
	private PersistenceManagerFactory pmf = null;
	private PersistenceManager pm = null;
	private int countGetPM = 0;

	static Logger logger = null;
	static {
		LoggerFactory lf = Monolog.initialize();
		logger = lf.getLogger(PMHolder.class.getName());
	}

	/**
	 * Persistence manager holder's constructor
	 * @param newPMF is the persistence manager factory
	 */
	public  PMHolder(PersistenceManagerFactory newPMF){
		this.pmf = newPMF;
		this.countGetPM = 0;
	}

	/**
	 * Returns the persistence manager.<p>When the persistence manager is allready
	 * not null, the existing PM is returned.</p>
	 */
	public synchronized PersistenceManager getPersistenceManager() {
		this.countGetPM ++;
		if (null != this.pm && this.pm.isClosed()) {
			logger.log(BasicLevel.WARN, "PersistenceManager is closed");
			this.countGetPM = 1;
			this.pm = null;
		}
		if (null == this.pm) {
			try {
				logger.log(BasicLevel.DEBUG, "Get   PersistenceManager");
				this.pm = this.pmf.getPersistenceManager();
			} catch (NullPointerException e) {
				throw e;
			} catch (JDOException e) {
				throw e;
			}
		}
		logger.log(BasicLevel.DEBUG, "Returned persistenceManager : " + pm);
		return this.pm;
	}

	/**
	 * Close the persistence manager.<p>When the persistence manager is allready
	 * close nothing is done.</p><p>When the PM has bo be closed, this method launchs
	 * the {@link org.objectweb.speedo.j2eedo.bo.PollsSynchronizations polls
	 * synchronization process} when needed (it means when the PM userObject
	 * is not null.</p>
	 * <i>Remark:</i> The getPersistenceManager can be called more than 1 time during
	 * each action. This object keep updated a counter in order to know when
	 * it necessary to close the PM.
	 */
	public void closePersistenceManager() {
		this.countGetPM --;
		if (null != this.pm && this.countGetPM == 0) {
			logger.log(BasicLevel.DEBUG, "Close PersistenceManager : " + this.pm);
			if (!this.pm.isClosed()) {
				//verify if some pool must be updated
				if (null != this.pm.getUserObject()) {
					((PollsSynchronizations)this.pm.getUserObject()).afterCompletion(-1);
				}
				this.pm.close();
			} else {
				logger.log(BasicLevel.WARN, "PersistenceManager allready closed : " + this.pm);
			}
			this.pm = null;
		}
	}
}