/*
 * Speedo: an implementation of JDO compliant personality on top of JORM
 * generic I/O sub-system. Copyright (C) 2001-2004 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Release: 1.0
 * 
 * Created on 27 f�vr. 2004 @author fmillevi@yahoo.com
 *  
 */
package org.objectweb.speedo.j2eedo.database;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * This class define the Java class to access to the project's data
 * @author fmillevi@yahoo.com
 *  
 */
public class Project implements DatabaseObjectInterface {
	private long proid;
	private String name;
	private BigDecimal budget;
	private Collection members; //element:Employee
	private Department department;

	public String getAsString() {
		StringBuffer sb = new StringBuffer();
		sb.append("\nProject (proid:");
		sb.append(proid);
		sb.append(" department:");
		if (null != this.department)
			sb.append(this.department.getName());
		else
			sb.append("is null");
		sb.append(")\n\tName:");
		sb.append(name);
		sb.append("\n\tbudget:");
		sb.append(budget);
		if (null != this.members) {
			Iterator i = this.members.iterator();
			if (i.hasNext())
				sb.append("\n\tMembers:");
			Employee e = null;
			while (i.hasNext()) {
				e = (Employee) i.next();
				sb.append(e.getFirstname());
				sb.append(" ");
				sb.append(e.getLastname());
				sb.append("  ");
			}
		}
		return sb.toString();
	}

	/**
	 * @see org.objectweb.speedo.j2eedo.database.DatabaseObjectInterface#getId()
	 */
	public long getId() {
		return this.proid;
	}

	/**
	 * @return Returns the budget.
	 */
	public BigDecimal getBudget() {
		return budget;
	}

	/**
	 * @param budget
	 *            The budget to set.
	 */
	public void setBudget(BigDecimal budget) {
		this.budget = budget;
	}

	/**
	 * @return Returns the members.
	 */
	public Collection getMembers() {
		return members;
	}

	/**
	 * @param members
	 *            The members to set.
	 */
	public void setMembers(Collection members) {
		this.members.addAll(members);
	}

	/**
	 * @param newEmployee
	 */
	public boolean addMember(Employee newEmployee) {
		if (!members.contains(newEmployee)) {
			this.members.add(newEmployee);
			return true;
		} else {
			return false;
		}
	}
	/**
	 * @param newEmployee
	 */
	public void removeMember(Employee newEmployee) {
		this.members.remove(newEmployee);
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return Returns the proid.
	 */
	public long getProid() {
		return proid;
	}

	/**
	 * @return Returns the department.
	 */
	public Department getDepartment() {
		return department;
	}

	/**
	 * @param department
	 *            The department to set.
	 */
	public void setDepartment(Department department) {
		this.department = department;
	}

	/**
	 * @param name
	 * @param department
	 */
	public Project(String name, Department department) {
		super();
		this.name = name;
		this.department = department;
		this.members = new ArrayList();
	}
	/**
	 * Empty constructor
	 */
	public Project() {
		super();
	}

}
