/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 *@author fmillevi@yahoo.com
 *
 */
package org.objectweb.speedo.j2eedo.web;
import java.io.IOException;
import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.objectweb.speedo.j2eedo.common.PMHolder;
import org.objectweb.speedo.j2eedo.ejb.StoreServices;
import org.objectweb.speedo.j2eedo.ejb.StoreServicesHome;

abstract class MainServlet extends HttpServlet {
	// define the class name parameter
	/**
	 * The constant <code>WITHOUT_TRANSACTION_PARAMETER</code> is used
	 * to retrive the HTTP request parameter <b>"withoutTrans"</b> used by the servlet
	 * to know if a transaction must be start at the servlet level.
	 */
	public final static String WITHOUT_TRANSACTION_PARAMETER = "withoutTrans";
	/**
	 * The constant <code>TYPE_CONTAINER_PARAMETER</code> is used
	 * to retrive the HTTP request parameter <b>"container"</b> used by the servlet
	 * to know if the action must be done through a Session Bean Container.
	 */
	public final static String TYPE_CONTAINER_PARAMETER = "container";
	/**
	 * The constant <code>WITH_GETPM_PARAMETER</code> is used
	 * to retrive the HTTP request parameter <b>"withGetPM"</b> used by the servlet
	 * to know if the persistence manager must be get by the servelt.
	 */
	public final static String WITH_GETPM_PARAMETER = "withGetPM";

	private final static String PMF_JNDI_NAME = "speedo";
	protected static Logger logger = null;
	static {
		Logger.getLogger(MainServlet.class.getName());
	}
	/**
	 * Comment for <code>persistenceManagerFactory</code> is the current
	 * persistence manager factory interface.
	 */
	protected PersistenceManagerFactory persistenceManagerFactory = null;
	/**
	 * This servlet executes current request:<ul>
	 * <li>Gets the Persistence manager when required (false by default),</li>
	 * <li>Starts a local transaction when requested (false by default),</li>
	 * <li>Calls the dedicated action directly or through a Session Bean Container
	 * (direct call by default),</li>
	 * <li>Close the PM when initialized at the servlet level.</li>
	 * </ul>
	 * @see javax.servlet.http.HttpServlet#service(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	public void service(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException {
		boolean withGetPM = false;
		boolean withTransaction = false;
		boolean containerSession = false;
		try {
			if (null != req.getParameter(MainServlet.WITH_GETPM_PARAMETER)
					&& "true".equals( req.getParameter(MainServlet.WITH_GETPM_PARAMETER)))
				withGetPM = true;
		} catch (Exception e) {
			withGetPM = false;
		}
		try {
			if (null != req.getParameter(MainServlet.TYPE_CONTAINER_PARAMETER)
					&& "true".equals( req.getParameter(MainServlet.TYPE_CONTAINER_PARAMETER)))
				containerSession = true;
		} catch (Exception e) {
			containerSession = false;
		}
		try {
			if (null
				!= req.getParameter(MainServlet.WITHOUT_TRANSACTION_PARAMETER)
				&& "true".equals( req.getParameter(MainServlet.WITHOUT_TRANSACTION_PARAMETER)))
				withTransaction = false;
		} catch (Exception e) {
			withTransaction = true;
		}
		// check sequence of parameters
		if (containerSession && withTransaction) {
			withTransaction = false;
			logger.warn(
				"Use local transaction is forbiden when using an ejb session");
		} else if (withTransaction && !withGetPM) {
			// to begin and close an transaction the pm is required
			withGetPM = true;
			logger.warn(
				"Use of the persistence manager is required when using transaction");
		}
		InitialContext context = null;
		PMHolder persistenceManagerHolder =
			new PMHolder(this.persistenceManagerFactory);
		PersistenceManager pm = null;
		javax.jdo.Transaction utx = null;
		try {
			context = new InitialContext();
		} catch (NamingException e) {
			e.printStackTrace();
			throw new ServletException("Impossible to find Initial context", e);
		}
		if (null == context)
			throw new ServletException("Initial context is null");
		try {
			if (withGetPM) {
				pm = persistenceManagerHolder.getPersistenceManager();
				logger.debug("GetPM");
			}
			if (withTransaction) {
				utx = pm.currentTransaction();
				utx.begin();
				logger.debug("Begin local Transaction");
			}
			try {
				if (containerSession) {
					StoreServicesHome lstoreServicesHome =
						(StoreServicesHome) PortableRemoteObject.narrow(
							context.lookup("ejb/storeService"),
							StoreServicesHome.class);
					if (lstoreServicesHome == null) {
						throw new ServletException("Method Store Data invocation  ko. No home");
					} else {
						StoreServices lstoreServices = null;
						try {
							lstoreServices = lstoreServicesHome.create();
						} catch (RemoteException e2) {
							e2.printStackTrace();
							throw new ServletException(e2);
						} catch (CreateException e2) {
							e2.printStackTrace();
							throw new ServletException(e2);
						}
						if (lstoreServices != null) {
							logger.debug("Use session bean");
							this.executeSessionBean(
								req,
								resp,
								lstoreServices,
								persistenceManagerHolder);
						}
					}
				} else {
					logger.debug("Use direct call");
					this.executeDirectCall(req, resp, persistenceManagerHolder);
				} // end else if containerSession
			} catch (JDOException e) {
				if (withTransaction)
					utx.rollback();
				throw e;
			}
			if (withTransaction) {
				utx.commit();
				logger.debug("Use local Transaction");
			}
			if (withGetPM) {
				persistenceManagerHolder.closePersistenceManager();
				logger.debug("Close local PM");
			}
		} catch (ClassCastException e1) {
			logger.error("Error ClassCastException", e1);
			throw new ServletException(e1);
		} catch (Exception e1) {
			logger.error("Error Exception", e1);
			if (withTransaction && null != utx)
				utx.rollback();
			throw new ServletException(e1);
		}
	}
	/**
	 * Initialize the Persistence manager factory
	 * @see javax.servlet.Servlet#init(javax.servlet.ServletConfig)
	 */
	public void init(ServletConfig config)
		throws ServletException, JDOException {
		super.init(config);
		//fetch the PersistenceManagerFactory
		InitialContext ictx = null;
		try {
			ictx = new InitialContext();
			this.persistenceManagerFactory =
				(PersistenceManagerFactory) ictx.lookup(PMF_JNDI_NAME);
			System.err.println(
				"VALUE lstoreServicesHome  = "
					+ ictx.lookup("ejb/storeService"));
		} catch (NamingException ne) {
			System.err.println("Impossible to find persistance manager" + ne);
			ne.printStackTrace();
			return;
		} finally {
			if (ictx != null)
				try {
					ictx.close();
				} catch (NamingException ne) {
					System.err.println(
						"Impossible to find  persistenceManagerFactory " + ne);
					ne.printStackTrace();
				}
		}
	}
	/**
	 * Empty threatment call for a ejb session
	 *
	 * @param lstoreServices
	 * @param persistenceManagerHolder
	 * @param out
	 * @throws JDOException
	 * @throws RemoteException
	 * @throws Exception
	 * @throws IOException
	 */
	protected abstract void executeSessionBean(
		HttpServletRequest req,
		HttpServletResponse resp,
		StoreServices storeServices,
		PMHolder persistenceManagerHolder)
		throws JDOException, RemoteException, Exception, IOException;
	/**
	 * Empty threatment call for a jdo direct call
	 *
	 * @param persistenceManagerHolder
	 * @param out
	 * @throws JDOException
	 * @throws Exception
	 * @throws IOException
	 */
	protected abstract void executeDirectCall(
		HttpServletRequest req,
		HttpServletResponse resp,
		PMHolder persistenceManagerHolder)
		throws JDOException, Exception, IOException ;
}