/*
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */
package org.objectweb.speedo.j2eedo.common;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class provides with a simple method to perform some HTTP requests
 * @author fmillevi@yahoo.com
 */
public class HTTPTools {
	static String redirectURL;

	/**
	 * Liste des cookies
	 */
	static Hashtable cookies = null;
	static TraceTime traceTime = new TraceTime();

	/**
	 * Method getURL. This static method provides with the way to get and http response.
	 *
	 * @param urlString is the URL to be invoked
	 * @param paramString is the list of parameters like
	 *            param1=value1&param2=value2&param3=....
	 * @param method POST or GET
	 * @return String flux HTTP de retour.
	 * @throws Exception when the getURL method fails due to unfound URL, or server errors
	 */
	public static String getURL(
		String urlString,
		String paramString,
		String method,
		Logger logger)
		throws Exception {
		if (cookies == null) // C'est la premi�re connection.
			{
			// create the hashtable used to store cookies values
			cookies = new Hashtable();
			//getURL(urlString, paramString, method, logger);
		}

		String s = "Ok"; // return string

		try {
			if (!method.equalsIgnoreCase("post") && !paramString.equals("")) {
				urlString = urlString + "?" + paramString;
			}

			URL url = new URL(urlString);

			traceTime.startTask("openConnection");
			HttpURLConnection conn = (HttpURLConnection) url.openConnection();
			traceTime.endTask();

			//String userPassword = "xxxx:yyyy";
			//String encoding =
			//	new sun.misc.BASE64Encoder().encode(userPassword.getBytes());
			//conn.setRequestProperty("Authorization", encoding);
			//conn.setRequestProperty("Authorization", "Basic " + encoding);
			conn.setRequestProperty("accept-language", "fr");
			conn.setRequestProperty(
				"user-agent",
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.0.3705)");

			conn.setRequestProperty ("connection", "Keep-Alive");
			//conn.setRequestProperty ("accept-encoding", "gzip, deflate");

			//accept http redirections
			HttpURLConnection.setFollowRedirects(false);
			//conn.setAllowUserInteraction(false);
			conn.setUseCaches(false);

			setCookies(conn);

			if (method.equalsIgnoreCase("post")) {
				traceTime.startTask("set post parameters");
				byte[] bytes = paramString.getBytes();
				conn.setDoOutput(true);
				conn.setDoInput(true);
				conn.setRequestMethod("POST");
				conn.setRequestProperty(
					"Content-length",
					String.valueOf(bytes.length));
				OutputStream out = conn.getOutputStream();
				out.write(bytes);
				out.flush();
				traceTime.endTask();
			}

			traceTime.startTask("getResponseCode");
			s = "" + conn.getResponseCode();
			traceTime.endTask();

			if (conn.getResponseCode() != 200
				&& conn.getResponseCode() != 302) {
				logger.log(BasicLevel.ERROR, "error " + conn.getResponseCode());
			} else if (conn.getResponseCode() == 200) { //parse HTML response
				//read HTML response
				BufferedReader is =
					new BufferedReader(
						new InputStreamReader(conn.getInputStream()));
				String line = null;
				StringBuffer sourceTxt = new StringBuffer("");
				while ((line = is.readLine()) != null)
					sourceTxt.append(line);
				is.close();
				s = sourceTxt.toString();
			}
			getCookies(conn);
			logger.log(
				BasicLevel.DEBUG,
				urlString + "\treturns:" + conn.getResponseCode() + " ");
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, traceTime.report());
			}

			while (s.equals("302")) {
				s = getURL(redirectURL, "", "", logger);
			}

			if (conn.getResponseCode() == 500)
				throw new Exception("the URL " + url + " fails.");

			if (conn.getResponseCode() == 404)
				throw new Exception("the URL " + url + " not found.");

		} catch (Exception ex) {
			s = "";
			logger.log(BasicLevel.DEBUG, urlString + " throws Execption!", ex);
			throw ex;
		} finally {
			traceTime.reset();
		}

		return s;
	}

	/**
	 * Method getCookies. Gets and stores in the Hashtable the cookies values found in the HTTP response.
	 *
	 * @param con HTTP Connexion
	 */
	private static void getCookies(HttpURLConnection con) {
		int n = 1;

		label0 : for (boolean done = false; !done; n++) {
			String headerKey = con.getHeaderFieldKey(n);
			String headerVal = con.getHeaderField(n);
			if (headerKey != null || headerVal != null) {
				if ("location".equalsIgnoreCase(headerKey)) {
					redirectURL = headerVal;
				}
				if (!"Set-Cookie".equals(headerKey))
					continue;
				StringTokenizer st = new StringTokenizer(headerVal, ";");
				do {
					if (!st.hasMoreTokens())
						continue label0;
					String pair = st.nextToken();
					StringTokenizer stt = new StringTokenizer(pair, "=");
					while (stt.hasMoreTokens()) {
						String cookName = stt.nextToken();
						String cookValue = "";
						try {
							cookValue = stt.nextToken();
						} catch (Exception exception) {
						}
						if (!cookName.trim().equalsIgnoreCase("path")
							&& !cookName.trim().equalsIgnoreCase("expires")) {
							//System.out.println("Set cookie :" + cookName +
							// ":" + cookValue);
							cookies.put(cookName.trim(), cookValue);
						}
					}
				}
				while (true);
			}
			done = true;
		}
	}

	/**
	 * Method setCookies initialize the cookies values in the http request
	 * using the values found in the Hashtable
	 *
	 * @param con HTTP Connexion
	 */
	private static void setCookies(URLConnection con) {
		Enumeration en = cookies.keys();
		String cookieString;
		String key = "";
		String val = "";
		for (cookieString = "";
			en.hasMoreElements();
			cookieString = cookieString + (key + "=" + val + ";")) {
			key = (String) en.nextElement();
			val = (String) cookies.get(key);
		}
		//System.out.println("Init cookies :" + cookieString);
		con.setRequestProperty("Cookie", cookieString);
	}
}
